searchData={"items":[{"type":"module","title":"Exdantic","doc":"Exdantic is a schema definition and validation library for Elixir.\n\nIt provides a DSL for defining schemas with rich metadata, validation rules,\nand JSON Schema generation capabilities.\n\nFor validating non-dictionary types at the root level (similar to Pydantic's RootModel),\nsee `Exdantic.RootSchema`.","ref":"Exdantic.html"},{"type":"module","title":"Struct Pattern Support - Exdantic","doc":"Exdantic now supports generating structs alongside validation schemas:\n\n    defmodule UserSchema do\n      use Exdantic, define_struct: true\n\n      schema \"User account information\" do\n        field :name, :string do\n          required()\n          min_length(2)\n        end\n\n        field :age, :integer do\n          optional()\n          gt(0)\n        end\n      end\n    end\n\nThe schema can then be used for validation and returns struct instances:\n\n    # Returns {:ok, %UserSchema{name: \"John\", age: 30}}\n    UserSchema.validate(%{name: \"John\", age: 30})\n\n    # Serialize struct back to map\n    {:ok, map} = UserSchema.dump(user_struct)","ref":"Exdantic.html#module-struct-pattern-support"},{"type":"module","title":"Examples - Exdantic","doc":"defmodule UserSchema do\n      use Exdantic\n\n      schema \"User registration data\" do\n        field :name, :string do\n          required()\n          min_length(2)\n        end\n\n        field :age, :integer do\n          optional()\n          gt(0)\n          lt(150)\n        end\n\n        field :email, Types.Email do\n          required()\n        end\n\n        config do\n          title(\"User Schema\")\n          strict(true)\n        end\n      end\n    end\n\nThe schema can then be used for validation and JSON Schema generation:\n\n    # Validation (returns map by default)\n    {:ok, user} = UserSchema.validate(%{\n      name: \"John Doe\",\n      email: \"john@example.com\",\n      age: 30\n    })\n\n    # JSON Schema generation\n    json_schema = UserSchema.json_schema()","ref":"Exdantic.html#module-examples"},{"type":"macro","title":"Exdantic.__before_compile__/1","doc":"Phase 6 Enhancement: Enhanced schema information with complete feature analysis.","ref":"Exdantic.html#__before_compile__/1"},{"type":"macro","title":"Examples - Exdantic.__before_compile__/1","doc":"iex> UserSchema.__enhanced_schema_info__()\n    %{\n      exdantic_version: \"Phase 6\",\n      phase_6_enhanced: true,\n      compatibility: %{...},\n      performance_profile: %{...},\n      llm_optimization: %{...}\n    }","ref":"Exdantic.html#__before_compile__/1-examples"},{"type":"macro","title":"Exdantic.__using__/1","doc":"Configures a module to be an Exdantic schema.","ref":"Exdantic.html#__using__/1"},{"type":"macro","title":"Options - Exdantic.__using__/1","doc":"* `:define_struct` - Whether to generate a struct for validated data.\n    When `true`, validation returns struct instances instead of maps.\n    Defaults to `false` for backwards compatibility.","ref":"Exdantic.html#__using__/1-options"},{"type":"macro","title":"Examples - Exdantic.__using__/1","doc":"# Traditional map-based validation\n    defmodule UserMapSchema do\n      use Exdantic\n\n      schema do\n        field :name, :string\n      end\n    end\n\n    # Struct-based validation\n    defmodule UserStructSchema do\n      use Exdantic, define_struct: true\n\n      schema do\n        field :name, :string\n      end\n    end","ref":"Exdantic.html#__using__/1-examples"},{"type":"module","title":"Exdantic.ComputedFieldMeta","doc":"Metadata structure for computed fields in Exdantic schemas.\n\nComputed fields are derived values that are calculated based on the validated\ndata after field and model validation. They extend the validated result with\nadditional computed information.","ref":"Exdantic.ComputedFieldMeta.html"},{"type":"function","title":"Exdantic.ComputedFieldMeta.function_reference/1","doc":"Returns a string representation of the computed field function reference.","ref":"Exdantic.ComputedFieldMeta.html#function_reference/1"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.function_reference/1","doc":"* `computed_field_meta` - The ComputedFieldMeta struct","ref":"Exdantic.ComputedFieldMeta.html#function_reference/1-parameters"},{"type":"function","title":"Returns - Exdantic.ComputedFieldMeta.function_reference/1","doc":"* String in the format \"Module.function/arity\"","ref":"Exdantic.ComputedFieldMeta.html#function_reference/1-returns"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.function_reference/1","doc":"iex> meta = Exdantic.ComputedFieldMeta.new(:uppercased, {:type, :string, []}, :upcase, String)\n    iex> Exdantic.ComputedFieldMeta.function_reference(meta)\n    \"String.upcase/1\"","ref":"Exdantic.ComputedFieldMeta.html#function_reference/1-examples"},{"type":"function","title":"Exdantic.ComputedFieldMeta.new/4","doc":"Creates a new ComputedFieldMeta struct.","ref":"Exdantic.ComputedFieldMeta.html#new/4"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.new/4","doc":"* `name` - The computed field name (atom)\n  * `type` - The computed field type definition\n  * `function_name` - The name of the function to call for computation\n  * `module` - The module containing the computation function","ref":"Exdantic.ComputedFieldMeta.html#new/4-parameters"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.new/4","doc":"iex> Exdantic.ComputedFieldMeta.new(:uppercased, {:type, :string, []}, :upcase, String)\n    %Exdantic.ComputedFieldMeta{\n      name: :uppercased,\n      type: {:type, :string, []},\n      function_name: :upcase,\n      module: String,\n      readonly: true\n    }","ref":"Exdantic.ComputedFieldMeta.html#new/4-examples"},{"type":"function","title":"Exdantic.ComputedFieldMeta.set_readonly/2","doc":"Sets the readonly flag for a computed field metadata.\n\nWhile computed fields are readonly by default, this function allows\nexplicit control over the readonly flag for special cases.","ref":"Exdantic.ComputedFieldMeta.html#set_readonly/2"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.set_readonly/2","doc":"* `computed_field_meta` - The ComputedFieldMeta struct\n  * `readonly` - Whether the field should be marked as readonly","ref":"Exdantic.ComputedFieldMeta.html#set_readonly/2-parameters"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.set_readonly/2","doc":"iex> meta = Exdantic.ComputedFieldMeta.new(:uppercased, {:type, :string, []}, :upcase, String)\n    iex> updated = Exdantic.ComputedFieldMeta.set_readonly(meta, false)\n    iex> updated.readonly\n    false","ref":"Exdantic.ComputedFieldMeta.html#set_readonly/2-examples"},{"type":"type","title":"Exdantic.ComputedFieldMeta.t/0","doc":"","ref":"Exdantic.ComputedFieldMeta.html#t:t/0"},{"type":"function","title":"Exdantic.ComputedFieldMeta.to_map/1","doc":"Converts computed field metadata to a map for debugging or serialization.","ref":"Exdantic.ComputedFieldMeta.html#to_map/1"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.to_map/1","doc":"* `computed_field_meta` - The ComputedFieldMeta struct","ref":"Exdantic.ComputedFieldMeta.html#to_map/1-parameters"},{"type":"function","title":"Returns - Exdantic.ComputedFieldMeta.to_map/1","doc":"* Map representation of the computed field metadata","ref":"Exdantic.ComputedFieldMeta.html#to_map/1-returns"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.to_map/1","doc":"iex> meta = Exdantic.ComputedFieldMeta.new(:uppercased, {:type, :string, []}, :upcase, String)\n    iex> |> Exdantic.ComputedFieldMeta.with_description(\"Uppercased version\")\n    iex> Exdantic.ComputedFieldMeta.to_map(meta)\n    %{\n      name: :uppercased,\n      type: {:type, :string, []},\n      function_name: :upcase,\n      module: String,\n      description: \"Uppercased version\",\n      example: nil,\n      readonly: true,\n      function_reference: \"String.upcase/1\"\n    }","ref":"Exdantic.ComputedFieldMeta.html#to_map/1-examples"},{"type":"function","title":"Exdantic.ComputedFieldMeta.validate_function/1","doc":"Validates that the computation function exists in the specified module.","ref":"Exdantic.ComputedFieldMeta.html#validate_function/1"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.validate_function/1","doc":"* `computed_field_meta` - The ComputedFieldMeta struct","ref":"Exdantic.ComputedFieldMeta.html#validate_function/1-parameters"},{"type":"function","title":"Returns - Exdantic.ComputedFieldMeta.validate_function/1","doc":"* `:ok` if the function exists and has the correct arity\n  * `{:error, reason}` if the function is invalid","ref":"Exdantic.ComputedFieldMeta.html#validate_function/1-returns"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.validate_function/1","doc":"# For a module that exists with the function\n    iex> meta = Exdantic.ComputedFieldMeta.new(:result, {:type, :string, []}, :upcase, String)\n    iex> Exdantic.ComputedFieldMeta.validate_function(meta)\n    :ok\n\n    # For a module with a missing function\n    iex> meta = Exdantic.ComputedFieldMeta.new(:bad_field, {:type, :string, []}, :missing_function, String)\n    iex> Exdantic.ComputedFieldMeta.validate_function(meta)\n    {:error, \"Function String.missing_function/1 is not defined\"}","ref":"Exdantic.ComputedFieldMeta.html#validate_function/1-examples"},{"type":"function","title":"Exdantic.ComputedFieldMeta.with_description/2","doc":"Updates the description of a computed field metadata.","ref":"Exdantic.ComputedFieldMeta.html#with_description/2"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.with_description/2","doc":"* `computed_field_meta` - The ComputedFieldMeta struct\n  * `description` - The description text","ref":"Exdantic.ComputedFieldMeta.html#with_description/2-parameters"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.with_description/2","doc":"iex> meta = Exdantic.ComputedFieldMeta.new(:uppercased, {:type, :string, []}, :upcase, String)\n    iex> updated = Exdantic.ComputedFieldMeta.with_description(meta, \"Uppercased version\")\n    iex> updated.description\n    \"Uppercased version\"","ref":"Exdantic.ComputedFieldMeta.html#with_description/2-examples"},{"type":"function","title":"Exdantic.ComputedFieldMeta.with_example/2","doc":"Adds an example value to a computed field metadata.","ref":"Exdantic.ComputedFieldMeta.html#with_example/2"},{"type":"function","title":"Parameters - Exdantic.ComputedFieldMeta.with_example/2","doc":"* `computed_field_meta` - The ComputedFieldMeta struct\n  * `example` - The example value","ref":"Exdantic.ComputedFieldMeta.html#with_example/2-parameters"},{"type":"function","title":"Examples - Exdantic.ComputedFieldMeta.with_example/2","doc":"iex> meta = Exdantic.ComputedFieldMeta.new(:uppercased, {:type, :string, []}, :upcase, String)\n    iex> updated = Exdantic.ComputedFieldMeta.with_example(meta, \"HELLO\")\n    iex> updated.example\n    \"HELLO\"","ref":"Exdantic.ComputedFieldMeta.html#with_example/2-examples"},{"type":"module","title":"Exdantic.Config","doc":"Advanced configuration with runtime modification support.\n\nThis module provides functionality for creating and manipulating validation\nconfiguration at runtime, supporting the DSPy pattern of dynamic config\nmodification like `ConfigDict(extra=\"forbid\", frozen=True)`.","ref":"Exdantic.Config.html"},{"type":"function","title":"Exdantic.Config.allow_extra_fields?/1","doc":"Checks if extra fields should be allowed based on configuration.","ref":"Exdantic.Config.html#allow_extra_fields?/1"},{"type":"function","title":"Parameters - Exdantic.Config.allow_extra_fields?/1","doc":"* `config` - The configuration to check","ref":"Exdantic.Config.html#allow_extra_fields?/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.allow_extra_fields?/1","doc":"* `true` if extra fields are allowed, `false` otherwise","ref":"Exdantic.Config.html#allow_extra_fields?/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.allow_extra_fields?/1","doc":"iex> config = Exdantic.Config.create(extra: :allow)\n    iex> Exdantic.Config.allow_extra_fields?(config)\n    true\n\n    iex> config = Exdantic.Config.create(extra: :forbid)\n    iex> Exdantic.Config.allow_extra_fields?(config)\n    false","ref":"Exdantic.Config.html#allow_extra_fields?/1-examples"},{"type":"function","title":"Exdantic.Config.builder/0","doc":"Creates a builder for fluent configuration creation.","ref":"Exdantic.Config.html#builder/0"},{"type":"function","title":"Returns - Exdantic.Config.builder/0","doc":"* ConfigBuilder struct for chaining configuration calls","ref":"Exdantic.Config.html#builder/0-returns"},{"type":"function","title":"Examples - Exdantic.Config.builder/0","doc":"iex> config = Exdantic.Config.builder()\n    ...> |> Exdantic.Config.Builder.strict(true)\n    ...> |> Exdantic.Config.Builder.forbid_extra()\n    ...> |> Exdantic.Config.Builder.safe_coercion()\n    ...> |> Exdantic.Config.Builder.build()\n    %Exdantic.Config{strict: true, extra: :forbid, coercion: :safe, ...}","ref":"Exdantic.Config.html#builder/0-examples"},{"type":"function","title":"Exdantic.Config.coercion_level/1","doc":"Gets the coercion aggressiveness level.","ref":"Exdantic.Config.html#coercion_level/1"},{"type":"function","title":"Parameters - Exdantic.Config.coercion_level/1","doc":"* `config` - The configuration to check","ref":"Exdantic.Config.html#coercion_level/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.coercion_level/1","doc":"* Coercion strategy atom","ref":"Exdantic.Config.html#coercion_level/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.coercion_level/1","doc":"iex> config = Exdantic.Config.create(coercion: :aggressive)\n    iex> Exdantic.Config.coercion_level(config)\n    :aggressive","ref":"Exdantic.Config.html#coercion_level/1-examples"},{"type":"type","title":"Exdantic.Config.coercion_strategy/0","doc":"","ref":"Exdantic.Config.html#t:coercion_strategy/0"},{"type":"function","title":"Exdantic.Config.create/1","doc":"Creates a new configuration with the specified options.","ref":"Exdantic.Config.html#create/1"},{"type":"function","title":"Parameters - Exdantic.Config.create/1","doc":"* `opts` - Configuration options as keyword list or map","ref":"Exdantic.Config.html#create/1-parameters"},{"type":"function","title":"Options - Exdantic.Config.create/1","doc":"* `:strict` - Enforce strict validation (default: false)\n  * `:extra` - How to handle extra fields (default: :allow)\n  * `:coercion` - Type coercion strategy (default: :safe)\n  * `:frozen` - Whether the config is immutable (default: false)\n  * `:validate_assignment` - Validate field assignments (default: false)\n  * `:error_format` - Error format style (default: :detailed)","ref":"Exdantic.Config.html#create/1-options"},{"type":"function","title":"Returns - Exdantic.Config.create/1","doc":"* New Config struct","ref":"Exdantic.Config.html#create/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.create/1","doc":"iex> config = Exdantic.Config.create(strict: true, extra: :forbid)\n    iex> config.strict\n    true\n\n    iex> config = Exdantic.Config.create(%{coercion: :aggressive, frozen: true})\n    iex> config.coercion\n    :aggressive","ref":"Exdantic.Config.html#create/1-examples"},{"type":"function","title":"Exdantic.Config.create_enhanced/1","doc":"Creates a configuration optimized for Phase 6 enhanced features.\n\nPhase 6 Enhancement: Configuration that supports all new features and LLM optimizations.","ref":"Exdantic.Config.html#create_enhanced/1"},{"type":"function","title":"Parameters - Exdantic.Config.create_enhanced/1","doc":"* `opts` - Configuration options with Phase 6 enhancements","ref":"Exdantic.Config.html#create_enhanced/1-parameters"},{"type":"function","title":"Phase 6 Options - Exdantic.Config.create_enhanced/1","doc":"* `:llm_provider` - Target LLM provider for optimization (:openai, :anthropic, :generic)\n  * `:dspy_compatible` - Ensure DSPy compatibility (default: false)\n  * `:enhanced_validation` - Enable enhanced validation pipeline (default: true)\n  * `:include_metadata` - Include enhanced metadata in schemas (default: true)\n  * `:performance_mode` - Optimize for performance (:speed, :memory, :balanced)","ref":"Exdantic.Config.html#create_enhanced/1-phase-6-options"},{"type":"function","title":"Examples - Exdantic.Config.create_enhanced/1","doc":"iex> config = Exdantic.Config.create_enhanced(%{\n    ...>   llm_provider: :openai,\n    ...>   dspy_compatible: true,\n    ...>   performance_mode: :balanced\n    ...> })\n    %Exdantic.Config{...}","ref":"Exdantic.Config.html#create_enhanced/1-examples"},{"type":"type","title":"Exdantic.Config.error_format/0","doc":"","ref":"Exdantic.Config.html#t:error_format/0"},{"type":"type","title":"Exdantic.Config.extra_strategy/0","doc":"","ref":"Exdantic.Config.html#t:extra_strategy/0"},{"type":"function","title":"Exdantic.Config.for_dspy/2","doc":"Creates a preset configuration for DSPy integration.\n\nPhase 6 Enhancement: Specialized configuration for DSPy patterns.","ref":"Exdantic.Config.html#for_dspy/2"},{"type":"function","title":"Parameters - Exdantic.Config.for_dspy/2","doc":"* `dspy_mode` - DSPy usage pattern (:signature, :chain_of_thought, :input_output, :general)\n  * `opts` - Additional configuration options","ref":"Exdantic.Config.html#for_dspy/2-parameters"},{"type":"function","title":"Examples - Exdantic.Config.for_dspy/2","doc":"iex> config = Exdantic.Config.for_dspy(:signature, provider: :openai)\n    %Exdantic.Config{strict: true, extra: :forbid, ...}","ref":"Exdantic.Config.html#for_dspy/2-examples"},{"type":"function","title":"Exdantic.Config.merge/2","doc":"Merges configuration options with an existing config.","ref":"Exdantic.Config.html#merge/2"},{"type":"function","title":"Parameters - Exdantic.Config.merge/2","doc":"* `base_config` - The base configuration\n  * `overrides` - Configuration options to merge/override","ref":"Exdantic.Config.html#merge/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.merge/2","doc":"* New Config struct with merged options\n  * Raises if base config is frozen and overrides are provided","ref":"Exdantic.Config.html#merge/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.merge/2","doc":"iex> base = Exdantic.Config.create(strict: true)\n    iex> merged = Exdantic.Config.merge(base, %{extra: :forbid, coercion: :none})\n    %Exdantic.Config{strict: true, extra: :forbid, coercion: :none, ...}\n\n    iex> frozen = Exdantic.Config.create(frozen: true)\n    iex> Exdantic.Config.merge(frozen, %{strict: true})\n    ** (RuntimeError) Cannot modify frozen configuration","ref":"Exdantic.Config.html#merge/2-examples"},{"type":"function","title":"Exdantic.Config.preset/1","doc":"Creates a preset configuration for common validation scenarios.","ref":"Exdantic.Config.html#preset/1"},{"type":"function","title":"Parameters - Exdantic.Config.preset/1","doc":"* `preset` - The preset name","ref":"Exdantic.Config.html#preset/1-parameters"},{"type":"function","title":"Available Presets - Exdantic.Config.preset/1","doc":"* `:strict` - Strict validation with no extra fields\n  * `:lenient` - Lenient validation allowing extra fields\n  * `:api` - Configuration suitable for API validation\n  * `:json_schema` - Configuration optimized for JSON Schema generation\n  * `:development` - Development-friendly configuration\n  * `:production` - Production-ready configuration","ref":"Exdantic.Config.html#preset/1-available-presets"},{"type":"function","title":"Returns - Exdantic.Config.preset/1","doc":"* Pre-configured Config struct","ref":"Exdantic.Config.html#preset/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.preset/1","doc":"iex> Exdantic.Config.preset(:strict)\n    %Exdantic.Config{strict: true, extra: :forbid, coercion: :none, ...}\n\n    iex> Exdantic.Config.preset(:lenient)\n    %Exdantic.Config{strict: false, extra: :allow, coercion: :safe, ...}","ref":"Exdantic.Config.html#preset/1-examples"},{"type":"function","title":"Exdantic.Config.should_coerce?/1","doc":"Checks if type coercion should be performed based on configuration.","ref":"Exdantic.Config.html#should_coerce?/1"},{"type":"function","title":"Parameters - Exdantic.Config.should_coerce?/1","doc":"* `config` - The configuration to check","ref":"Exdantic.Config.html#should_coerce?/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.should_coerce?/1","doc":"* `true` if coercion should be performed, `false` otherwise","ref":"Exdantic.Config.html#should_coerce?/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.should_coerce?/1","doc":"iex> config = Exdantic.Config.create(coercion: :safe)\n    iex> Exdantic.Config.should_coerce?(config)\n    true\n\n    iex> config = Exdantic.Config.create(coercion: :none)\n    iex> Exdantic.Config.should_coerce?(config)\n    false","ref":"Exdantic.Config.html#should_coerce?/1-examples"},{"type":"function","title":"Exdantic.Config.summary/1","doc":"Returns a summary of the configuration settings.","ref":"Exdantic.Config.html#summary/1"},{"type":"function","title":"Parameters - Exdantic.Config.summary/1","doc":"* `config` - The configuration to summarize","ref":"Exdantic.Config.html#summary/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.summary/1","doc":"* Map with configuration summary","ref":"Exdantic.Config.html#summary/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.summary/1","doc":"iex> config = Exdantic.Config.create(strict: true, extra: :forbid)\n    iex> Exdantic.Config.summary(config)\n    %{\n      validation_mode: \"strict\",\n      extra_fields: \"forbidden\",\n      coercion: \"safe\",\n      frozen: false,\n      features: [\"validate_assignment\", ...]\n    }","ref":"Exdantic.Config.html#summary/1-examples"},{"type":"type","title":"Exdantic.Config.t/0","doc":"","ref":"Exdantic.Config.html#t:t/0"},{"type":"function","title":"Exdantic.Config.to_json_schema_opts/1","doc":"Converts configuration to options suitable for JSON Schema generation.","ref":"Exdantic.Config.html#to_json_schema_opts/1"},{"type":"function","title":"Parameters - Exdantic.Config.to_json_schema_opts/1","doc":"* `config` - The configuration to convert","ref":"Exdantic.Config.html#to_json_schema_opts/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.to_json_schema_opts/1","doc":"* Keyword list of JSON Schema options","ref":"Exdantic.Config.html#to_json_schema_opts/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.to_json_schema_opts/1","doc":"iex> config = Exdantic.Config.create(strict: true, use_enum_values: true)\n    iex> Exdantic.Config.to_json_schema_opts(config)\n    [strict: true, use_enum_values: true, max_anyof_union_len: 5, ...]","ref":"Exdantic.Config.html#to_json_schema_opts/1-examples"},{"type":"function","title":"Exdantic.Config.to_validation_opts/1","doc":"Converts configuration to options suitable for validation functions.","ref":"Exdantic.Config.html#to_validation_opts/1"},{"type":"function","title":"Parameters - Exdantic.Config.to_validation_opts/1","doc":"* `config` - The configuration to convert","ref":"Exdantic.Config.html#to_validation_opts/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.to_validation_opts/1","doc":"* Keyword list of validation options","ref":"Exdantic.Config.html#to_validation_opts/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.to_validation_opts/1","doc":"iex> config = Exdantic.Config.create(strict: true, coercion: :safe)\n    iex> Exdantic.Config.to_validation_opts(config)\n    [strict: true, coerce: true, error_format: :detailed, ...]","ref":"Exdantic.Config.html#to_validation_opts/1-examples"},{"type":"function","title":"Exdantic.Config.validate_config/1","doc":"Validates configuration options for consistency.","ref":"Exdantic.Config.html#validate_config/1"},{"type":"function","title":"Parameters - Exdantic.Config.validate_config/1","doc":"* `config` - The configuration to validate","ref":"Exdantic.Config.html#validate_config/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.validate_config/1","doc":"* `:ok` if configuration is valid\n  * `{:error, reasons}` if configuration has issues","ref":"Exdantic.Config.html#validate_config/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.validate_config/1","doc":"iex> config = Exdantic.Config.create(strict: true, extra: :allow)\n    iex> Exdantic.Config.validate_config(config)\n    {:error, [\"strict mode conflicts with extra: :allow\"]}\n\n    iex> config = Exdantic.Config.create(strict: true, extra: :forbid)\n    iex> Exdantic.Config.validate_config(config)\n    :ok","ref":"Exdantic.Config.html#validate_config/1-examples"},{"type":"module","title":"Exdantic.Config.Builder","doc":"Fluent builder for creating Exdantic configurations.\n\nThis module provides a chainable API for building complex configurations\nin a readable and intuitive way.","ref":"Exdantic.Config.Builder.html"},{"type":"function","title":"Exdantic.Config.Builder.aggressive_coercion/1","doc":"Convenience method to enable aggressive coercion.","ref":"Exdantic.Config.Builder.html#aggressive_coercion/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.aggressive_coercion/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#aggressive_coercion/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.aggressive_coercion/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#aggressive_coercion/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.aggressive_coercion/1","doc":"iex> builder |> Exdantic.Config.Builder.aggressive_coercion()\n    %Exdantic.Config.Builder{opts: %{coercion: :aggressive}}","ref":"Exdantic.Config.Builder.html#aggressive_coercion/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.allow_extra/1","doc":"Convenience method to allow extra fields.","ref":"Exdantic.Config.Builder.html#allow_extra/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.allow_extra/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#allow_extra/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.allow_extra/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#allow_extra/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.allow_extra/1","doc":"iex> builder |> Exdantic.Config.Builder.allow_extra()\n    %Exdantic.Config.Builder{opts: %{extra: :allow}}","ref":"Exdantic.Config.Builder.html#allow_extra/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.apply_preset/2","doc":"Applies a preset configuration to the builder.","ref":"Exdantic.Config.Builder.html#apply_preset/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.apply_preset/2","doc":"* `builder` - The builder instance\n  * `preset` - The preset name (same as Exdantic.Config.preset/1)","ref":"Exdantic.Config.Builder.html#apply_preset/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.apply_preset/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#apply_preset/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.apply_preset/2","doc":"iex> builder |> Exdantic.Config.Builder.apply_preset(:strict)\n    %Exdantic.Config.Builder{opts: %{strict: true, extra: :forbid, ...}}","ref":"Exdantic.Config.Builder.html#apply_preset/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.build/1","doc":"Builds the final configuration from the builder.","ref":"Exdantic.Config.Builder.html#build/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.build/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#build/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.build/1","doc":"* Exdantic.Config struct with the configured options","ref":"Exdantic.Config.Builder.html#build/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.build/1","doc":"iex> config = Exdantic.Config.Builder.new()\n    ...> |> Exdantic.Config.Builder.strict()\n    ...> |> Exdantic.Config.Builder.forbid_extra()\n    ...> |> Exdantic.Config.Builder.build()\n    %Exdantic.Config{strict: true, extra: :forbid, ...}","ref":"Exdantic.Config.Builder.html#build/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.case_insensitive/1","doc":"Convenience method to disable case sensitivity.","ref":"Exdantic.Config.Builder.html#case_insensitive/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.case_insensitive/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#case_insensitive/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.case_insensitive/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#case_insensitive/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.case_insensitive/1","doc":"iex> builder |> Exdantic.Config.Builder.case_insensitive()\n    %Exdantic.Config.Builder{opts: %{case_sensitive: false}}","ref":"Exdantic.Config.Builder.html#case_insensitive/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.case_sensitive/2","doc":"Sets case sensitivity for field names.","ref":"Exdantic.Config.Builder.html#case_sensitive/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.case_sensitive/2","doc":"* `builder` - The builder instance\n  * `enabled` - Whether to enable case sensitivity (default: true)","ref":"Exdantic.Config.Builder.html#case_sensitive/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.case_sensitive/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#case_sensitive/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.case_sensitive/2","doc":"iex> builder |> Exdantic.Config.Builder.case_sensitive()\n    %Exdantic.Config.Builder{opts: %{case_sensitive: true}}\n\n    iex> builder |> Exdantic.Config.Builder.case_sensitive(false)\n    %Exdantic.Config.Builder{opts: %{case_sensitive: false}}","ref":"Exdantic.Config.Builder.html#case_sensitive/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.clone/1","doc":"Clones a builder instance.","ref":"Exdantic.Config.Builder.html#clone/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.clone/1","doc":"* `builder` - The builder instance to clone","ref":"Exdantic.Config.Builder.html#clone/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.clone/1","doc":"* New builder instance with the same configuration","ref":"Exdantic.Config.Builder.html#clone/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.clone/1","doc":"iex> original = Exdantic.Config.Builder.new() |> Exdantic.Config.Builder.strict()\n    iex> cloned = Exdantic.Config.Builder.clone(original)\n    %Exdantic.Config.Builder{opts: %{strict: true}}","ref":"Exdantic.Config.Builder.html#clone/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.coercion/2","doc":"Sets the coercion strategy.","ref":"Exdantic.Config.Builder.html#coercion/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.coercion/2","doc":"* `builder` - The builder instance\n  * `strategy` - The coercion strategy (:none, :safe, :aggressive)","ref":"Exdantic.Config.Builder.html#coercion/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.coercion/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#coercion/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.coercion/2","doc":"iex> builder |> Exdantic.Config.Builder.coercion(:safe)\n    %Exdantic.Config.Builder{opts: %{coercion: :safe}}","ref":"Exdantic.Config.Builder.html#coercion/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.description_generator/2","doc":"Sets a custom description generator function.","ref":"Exdantic.Config.Builder.html#description_generator/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.description_generator/2","doc":"* `builder` - The builder instance\n  * `generator_fn` - Function that takes a field name and returns a description","ref":"Exdantic.Config.Builder.html#description_generator/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.description_generator/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#description_generator/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.description_generator/2","doc":"iex> desc_fn = fn field -> \"Field for \" <> Atom.to_string(field) end\n    iex> builder |> Exdantic.Config.Builder.description_generator(desc_fn)\n    %Exdantic.Config.Builder{opts: %{description_generator: #Function<...>}}","ref":"Exdantic.Config.Builder.html#description_generator/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.detailed_errors/1","doc":"Convenience method to set detailed error format.","ref":"Exdantic.Config.Builder.html#detailed_errors/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.detailed_errors/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#detailed_errors/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.detailed_errors/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#detailed_errors/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.detailed_errors/1","doc":"iex> builder |> Exdantic.Config.Builder.detailed_errors()\n    %Exdantic.Config.Builder{opts: %{error_format: :detailed}}","ref":"Exdantic.Config.Builder.html#detailed_errors/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.error_format/2","doc":"Sets the error format style.","ref":"Exdantic.Config.Builder.html#error_format/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.error_format/2","doc":"* `builder` - The builder instance\n  * `format` - The error format (:detailed, :simple, :minimal)","ref":"Exdantic.Config.Builder.html#error_format/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.error_format/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#error_format/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.error_format/2","doc":"iex> builder |> Exdantic.Config.Builder.error_format(:simple)\n    %Exdantic.Config.Builder{opts: %{error_format: :simple}}","ref":"Exdantic.Config.Builder.html#error_format/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.extra/2","doc":"Sets how extra fields should be handled.","ref":"Exdantic.Config.Builder.html#extra/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.extra/2","doc":"* `builder` - The builder instance\n  * `strategy` - The extra field strategy (:allow, :forbid, :ignore)","ref":"Exdantic.Config.Builder.html#extra/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.extra/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#extra/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.extra/2","doc":"iex> builder |> Exdantic.Config.Builder.extra(:forbid)\n    %Exdantic.Config.Builder{opts: %{extra: :forbid}}","ref":"Exdantic.Config.Builder.html#extra/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.for_api/1","doc":"Creates a configuration for API validation scenarios.","ref":"Exdantic.Config.Builder.html#for_api/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.for_api/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#for_api/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.for_api/1","doc":"* Updated builder instance with API-friendly settings","ref":"Exdantic.Config.Builder.html#for_api/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.for_api/1","doc":"iex> builder |> Exdantic.Config.Builder.for_api()\n    %Exdantic.Config.Builder{opts: %{strict: true, extra: :forbid, ...}}","ref":"Exdantic.Config.Builder.html#for_api/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.for_development/1","doc":"Creates a configuration for development scenarios.","ref":"Exdantic.Config.Builder.html#for_development/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.for_development/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#for_development/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.for_development/1","doc":"* Updated builder instance with development-friendly settings","ref":"Exdantic.Config.Builder.html#for_development/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.for_development/1","doc":"iex> builder |> Exdantic.Config.Builder.for_development()\n    %Exdantic.Config.Builder{opts: %{strict: false, coercion: :aggressive, ...}}","ref":"Exdantic.Config.Builder.html#for_development/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.for_json_schema/1","doc":"Creates a configuration for JSON Schema generation scenarios.","ref":"Exdantic.Config.Builder.html#for_json_schema/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.for_json_schema/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#for_json_schema/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.for_json_schema/1","doc":"* Updated builder instance with JSON Schema-friendly settings","ref":"Exdantic.Config.Builder.html#for_json_schema/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.for_json_schema/1","doc":"iex> builder |> Exdantic.Config.Builder.for_json_schema()\n    %Exdantic.Config.Builder{opts: %{use_enum_values: true, error_format: :minimal, ...}}","ref":"Exdantic.Config.Builder.html#for_json_schema/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.for_production/1","doc":"Creates a configuration for production scenarios.","ref":"Exdantic.Config.Builder.html#for_production/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.for_production/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#for_production/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.for_production/1","doc":"* Updated builder instance with production-ready settings","ref":"Exdantic.Config.Builder.html#for_production/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.for_production/1","doc":"iex> builder |> Exdantic.Config.Builder.for_production()\n    %Exdantic.Config.Builder{opts: %{strict: true, frozen: true, ...}}","ref":"Exdantic.Config.Builder.html#for_production/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.forbid_extra/1","doc":"Convenience method to forbid extra fields.","ref":"Exdantic.Config.Builder.html#forbid_extra/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.forbid_extra/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#forbid_extra/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.forbid_extra/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#forbid_extra/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.forbid_extra/1","doc":"iex> builder |> Exdantic.Config.Builder.forbid_extra()\n    %Exdantic.Config.Builder{opts: %{extra: :forbid}}","ref":"Exdantic.Config.Builder.html#forbid_extra/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.frozen/2","doc":"Sets whether the configuration should be frozen (immutable).","ref":"Exdantic.Config.Builder.html#frozen/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.frozen/2","doc":"* `builder` - The builder instance\n  * `enabled` - Whether to freeze the config (default: true)","ref":"Exdantic.Config.Builder.html#frozen/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.frozen/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#frozen/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.frozen/2","doc":"iex> builder |> Exdantic.Config.Builder.frozen()\n    %Exdantic.Config.Builder{opts: %{frozen: true}}","ref":"Exdantic.Config.Builder.html#frozen/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.max_union_length/2","doc":"Sets the maximum length for anyOf unions in JSON Schema.","ref":"Exdantic.Config.Builder.html#max_union_length/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.max_union_length/2","doc":"* `builder` - The builder instance\n  * `max_length` - Maximum union length","ref":"Exdantic.Config.Builder.html#max_union_length/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.max_union_length/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#max_union_length/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.max_union_length/2","doc":"iex> builder |> Exdantic.Config.Builder.max_union_length(3)\n    %Exdantic.Config.Builder{opts: %{max_anyof_union_len: 3}}","ref":"Exdantic.Config.Builder.html#max_union_length/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.merge/2","doc":"Merges additional options into the builder.","ref":"Exdantic.Config.Builder.html#merge/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.merge/2","doc":"* `builder` - The builder instance\n  * `opts` - Additional options to merge","ref":"Exdantic.Config.Builder.html#merge/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.merge/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#merge/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.merge/2","doc":"iex> builder |> Exdantic.Config.Builder.merge(%{strict: true, frozen: true})\n    %Exdantic.Config.Builder{opts: %{strict: true, frozen: true}}","ref":"Exdantic.Config.Builder.html#merge/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.new/0","doc":"Creates a new configuration builder.","ref":"Exdantic.Config.Builder.html#new/0"},{"type":"function","title":"Returns - Exdantic.Config.Builder.new/0","doc":"* New ConfigBuilder instance","ref":"Exdantic.Config.Builder.html#new/0-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.new/0","doc":"iex> builder = Exdantic.Config.Builder.new()\n    %Exdantic.Config.Builder{opts: %{}}","ref":"Exdantic.Config.Builder.html#new/0-examples"},{"type":"function","title":"Exdantic.Config.Builder.no_coercion/1","doc":"Convenience method to disable coercion.","ref":"Exdantic.Config.Builder.html#no_coercion/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.no_coercion/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#no_coercion/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.no_coercion/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#no_coercion/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.no_coercion/1","doc":"iex> builder |> Exdantic.Config.Builder.no_coercion()\n    %Exdantic.Config.Builder{opts: %{coercion: :none}}","ref":"Exdantic.Config.Builder.html#no_coercion/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.reset/1","doc":"Resets the builder to an empty state.","ref":"Exdantic.Config.Builder.html#reset/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.reset/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#reset/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.reset/1","doc":"* Reset builder instance","ref":"Exdantic.Config.Builder.html#reset/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.reset/1","doc":"iex> builder = Exdantic.Config.Builder.new() |> Exdantic.Config.Builder.strict()\n    iex> reset_builder = Exdantic.Config.Builder.reset(builder)\n    %Exdantic.Config.Builder{opts: %{}}","ref":"Exdantic.Config.Builder.html#reset/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.safe_coercion/1","doc":"Convenience method to enable safe coercion.","ref":"Exdantic.Config.Builder.html#safe_coercion/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.safe_coercion/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#safe_coercion/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.safe_coercion/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#safe_coercion/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.safe_coercion/1","doc":"iex> builder |> Exdantic.Config.Builder.safe_coercion()\n    %Exdantic.Config.Builder{opts: %{coercion: :safe}}","ref":"Exdantic.Config.Builder.html#safe_coercion/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.simple_errors/1","doc":"Convenience method to set simple error format.","ref":"Exdantic.Config.Builder.html#simple_errors/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.simple_errors/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#simple_errors/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.simple_errors/1","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#simple_errors/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.simple_errors/1","doc":"iex> builder |> Exdantic.Config.Builder.simple_errors()\n    %Exdantic.Config.Builder{opts: %{error_format: :simple}}","ref":"Exdantic.Config.Builder.html#simple_errors/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.strict/2","doc":"Sets strict validation mode.","ref":"Exdantic.Config.Builder.html#strict/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.strict/2","doc":"* `builder` - The builder instance\n  * `enabled` - Whether to enable strict mode (default: true)","ref":"Exdantic.Config.Builder.html#strict/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.strict/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#strict/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.strict/2","doc":"iex> builder |> Exdantic.Config.Builder.strict()\n    %Exdantic.Config.Builder{opts: %{strict: true}}\n\n    iex> builder |> Exdantic.Config.Builder.strict(false)\n    %Exdantic.Config.Builder{opts: %{strict: false}}","ref":"Exdantic.Config.Builder.html#strict/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.summary/1","doc":"Returns a summary of the current builder configuration.","ref":"Exdantic.Config.Builder.html#summary/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.summary/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#summary/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.summary/1","doc":"* Map with current configuration options","ref":"Exdantic.Config.Builder.html#summary/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.summary/1","doc":"iex> builder = Exdantic.Config.Builder.new() |> Exdantic.Config.Builder.strict()\n    iex> Exdantic.Config.Builder.summary(builder)\n    %{options_set: [:strict], option_count: 1, ready_to_build: true}","ref":"Exdantic.Config.Builder.html#summary/1-examples"},{"type":"type","title":"Exdantic.Config.Builder.t/0","doc":"","ref":"Exdantic.Config.Builder.html#t:t/0"},{"type":"function","title":"Exdantic.Config.Builder.title_generator/2","doc":"Sets a custom title generator function.","ref":"Exdantic.Config.Builder.html#title_generator/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.title_generator/2","doc":"* `builder` - The builder instance\n  * `generator_fn` - Function that takes a field name and returns a title","ref":"Exdantic.Config.Builder.html#title_generator/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.title_generator/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#title_generator/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.title_generator/2","doc":"iex> title_fn = fn field -> field |> Atom.to_string() |> String.capitalize() end\n    iex> builder |> Exdantic.Config.Builder.title_generator(title_fn)\n    %Exdantic.Config.Builder{opts: %{title_generator: #Function<...>}}","ref":"Exdantic.Config.Builder.html#title_generator/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.use_enum_values/2","doc":"Sets whether to use enum values instead of names.","ref":"Exdantic.Config.Builder.html#use_enum_values/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.use_enum_values/2","doc":"* `builder` - The builder instance\n  * `enabled` - Whether to use enum values (default: true)","ref":"Exdantic.Config.Builder.html#use_enum_values/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.use_enum_values/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#use_enum_values/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.use_enum_values/2","doc":"iex> builder |> Exdantic.Config.Builder.use_enum_values()\n    %Exdantic.Config.Builder{opts: %{use_enum_values: true}}","ref":"Exdantic.Config.Builder.html#use_enum_values/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.validate_and_build/1","doc":"Validates the builder configuration before building.","ref":"Exdantic.Config.Builder.html#validate_and_build/1"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.validate_and_build/1","doc":"* `builder` - The builder instance","ref":"Exdantic.Config.Builder.html#validate_and_build/1-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.validate_and_build/1","doc":"* `{:ok, config}` if valid\n  * `{:error, reasons}` if invalid","ref":"Exdantic.Config.Builder.html#validate_and_build/1-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.validate_and_build/1","doc":"iex> valid_builder = Exdantic.Config.Builder.new() |> Exdantic.Config.Builder.strict()\n    iex> Exdantic.Config.Builder.validate_and_build(valid_builder)\n    {:ok, %Exdantic.Config{...}}\n\n    iex> invalid_builder = Exdantic.Config.Builder.new()\n    ...> |> Exdantic.Config.Builder.strict()\n    ...> |> Exdantic.Config.Builder.allow_extra()\n    iex> Exdantic.Config.Builder.validate_and_build(invalid_builder)\n    {:error, [\"strict mode conflicts with extra: :allow\"]}","ref":"Exdantic.Config.Builder.html#validate_and_build/1-examples"},{"type":"function","title":"Exdantic.Config.Builder.validate_assignment/2","doc":"Sets whether to validate field assignments.","ref":"Exdantic.Config.Builder.html#validate_assignment/2"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.validate_assignment/2","doc":"* `builder` - The builder instance\n  * `enabled` - Whether to enable assignment validation (default: true)","ref":"Exdantic.Config.Builder.html#validate_assignment/2-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.validate_assignment/2","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#validate_assignment/2-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.validate_assignment/2","doc":"iex> builder |> Exdantic.Config.Builder.validate_assignment()\n    %Exdantic.Config.Builder{opts: %{validate_assignment: true}}","ref":"Exdantic.Config.Builder.html#validate_assignment/2-examples"},{"type":"function","title":"Exdantic.Config.Builder.when_false/3","doc":"Applies configuration only if a condition is false.","ref":"Exdantic.Config.Builder.html#when_false/3"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.when_false/3","doc":"* `builder` - The builder instance\n  * `condition` - Boolean condition or function that returns boolean\n  * `config_fn` - Function to apply if condition is false","ref":"Exdantic.Config.Builder.html#when_false/3-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.when_false/3","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#when_false/3-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.when_false/3","doc":"iex> is_development = Application.get_env(:my_app, :env) == :dev\n    iex> builder |> Exdantic.Config.Builder.when_false(is_development, &Exdantic.Config.Builder.frozen/1)\n    %Exdantic.Config.Builder{...}","ref":"Exdantic.Config.Builder.html#when_false/3-examples"},{"type":"function","title":"Exdantic.Config.Builder.when_true/3","doc":"Conditionally applies a configuration based on a predicate.","ref":"Exdantic.Config.Builder.html#when_true/3"},{"type":"function","title":"Parameters - Exdantic.Config.Builder.when_true/3","doc":"* `builder` - The builder instance\n  * `condition` - Boolean condition or function that returns boolean\n  * `config_fn` - Function to apply if condition is true","ref":"Exdantic.Config.Builder.html#when_true/3-parameters"},{"type":"function","title":"Returns - Exdantic.Config.Builder.when_true/3","doc":"* Updated builder instance","ref":"Exdantic.Config.Builder.html#when_true/3-returns"},{"type":"function","title":"Examples - Exdantic.Config.Builder.when_true/3","doc":"iex> is_production = Application.get_env(:my_app, :env) == :prod\n    iex> builder |> Exdantic.Config.Builder.when_true(is_production, &Exdantic.Config.Builder.frozen/1)\n    %Exdantic.Config.Builder{...}\n\n    iex> builder |> Exdantic.Config.Builder.when_true(true, fn b ->\n    ...>   b |> Exdantic.Config.Builder.strict() |> Exdantic.Config.Builder.forbid_extra()\n    ...> end)\n    %Exdantic.Config.Builder{opts: %{strict: true, extra: :forbid}}","ref":"Exdantic.Config.Builder.html#when_true/3-examples"},{"type":"module","title":"Exdantic.EnhancedValidator","doc":"Enhanced validation functionality that integrates all new Exdantic features.\n\nThis module provides a unified interface for validation using the new runtime\ncapabilities, TypeAdapter functionality, advanced configuration, and wrapper\nsupport.","ref":"Exdantic.EnhancedValidator.html"},{"type":"function","title":"Exdantic.EnhancedValidator.comprehensive_validation_report/3","doc":"Comprehensive validation report with enhanced schema analysis.\n\nPhase 6 Enhancement: Complete integration testing and analysis.","ref":"Exdantic.EnhancedValidator.html#comprehensive_validation_report/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.comprehensive_validation_report/3","doc":"* `target` - What to validate against\n  * `input` - Data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#comprehensive_validation_report/3-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.comprehensive_validation_report/3","doc":"* Comprehensive validation and schema analysis report","ref":"Exdantic.EnhancedValidator.html#comprehensive_validation_report/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.comprehensive_validation_report/3","doc":"iex> report = Exdantic.EnhancedValidator.comprehensive_validation_report(\n    ...>   MySchema,\n    ...>   sample_data,\n    ...>   test_providers: [:openai, :anthropic],\n    ...>   include_performance_analysis: true\n    ...> )\n    %{\n      validation_result: {:ok, validated_data},\n      enhanced_schema: %{...},\n      provider_compatibility: %{...},\n      performance_metrics: %{...},\n      recommendations: [...]\n    }","ref":"Exdantic.EnhancedValidator.html#comprehensive_validation_report/3-examples"},{"type":"type","title":"Exdantic.EnhancedValidator.enhanced_options/0","doc":"","ref":"Exdantic.EnhancedValidator.html#t:enhanced_options/0"},{"type":"function","title":"Exdantic.EnhancedValidator.pipeline/3","doc":"Creates a basic validation pipeline for simple sequential validation.","ref":"Exdantic.EnhancedValidator.html#pipeline/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.pipeline/3","doc":"* `steps` - List of validation steps to execute in order\n  * `input` - The data to validate through the pipeline\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#pipeline/3-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.pipeline/3","doc":"* `{:ok, final_validated_data}` if all steps succeed\n  * `{:error, {step_index, errors}}` if any step fails","ref":"Exdantic.EnhancedValidator.html#pipeline/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.pipeline/3","doc":"iex> steps = [:string, fn s -> {:ok, String.upcase(s)} end, :string]\n    iex> Exdantic.EnhancedValidator.pipeline(steps, \"hello\", [])\n    {:ok, \"HELLO\"}","ref":"Exdantic.EnhancedValidator.html#pipeline/3-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate/3","doc":"Universal validation function that handles all types of validation targets.","ref":"Exdantic.EnhancedValidator.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate/3","doc":"* `target` - What to validate against (schema, type spec, etc.)\n  * `input` - The data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate/3-parameters"},{"type":"function","title":"Options - Exdantic.EnhancedValidator.validate/3","doc":"* `:config` - Exdantic.Config for validation behavior\n  * `:wrapper_field` - Field name if using wrapper validation\n  * `:json_schema_opts` - Options for JSON schema operations\n  * `:type_adapter_opts` - Options for TypeAdapter operations","ref":"Exdantic.EnhancedValidator.html#validate/3-options"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate/3","doc":"* `{:ok, validated_data}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.EnhancedValidator.html#validate/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate/3","doc":"# Validate against a compiled schema\n    iex> Exdantic.EnhancedValidator.validate(MySchema, %{name: \"John\"})\n    {:ok, %{name: \"John\"}}\n\n    # Validate against a runtime schema\n    iex> schema = Exdantic.Runtime.create_schema([{:name, :string}])\n    iex> Exdantic.EnhancedValidator.validate(schema, %{name: \"John\"})\n    {:ok, %{name: \"John\"}}\n\n    # Validate against a type specification\n    iex> Exdantic.EnhancedValidator.validate({:array, :string}, [\"a\", \"b\"])\n    {:ok, [\"a\", \"b\"]}\n\n    # Validate with custom configuration\n    iex> config = Exdantic.Config.create(strict: true, coercion: :safe)\n    iex> Exdantic.EnhancedValidator.validate(:integer, \"123\", config: config)\n    {:ok, 123}","ref":"Exdantic.EnhancedValidator.html#validate/3-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate_for_llm/4","doc":"Validates data for a specific LLM provider's structured output requirements.","ref":"Exdantic.EnhancedValidator.html#validate_for_llm/4"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate_for_llm/4","doc":"* `target` - What to validate against\n  * `input` - Data to validate\n  * `provider` - LLM provider (:openai, :anthropic, :generic)\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate_for_llm/4-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate_for_llm/4","doc":"* `{:ok, validated_data, provider_schema}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.EnhancedValidator.html#validate_for_llm/4-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate_for_llm/4","doc":"iex> Exdantic.EnhancedValidator.validate_for_llm(schema, data, :openai)\n    {:ok, validated_data, %{\"type\" => \"object\", \"additionalProperties\" => false}}","ref":"Exdantic.EnhancedValidator.html#validate_for_llm/4-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate_many/3","doc":"Validates multiple values efficiently against the same target.","ref":"Exdantic.EnhancedValidator.html#validate_many/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate_many/3","doc":"* `target` - What to validate against\n  * `inputs` - List of data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate_many/3-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate_many/3","doc":"* `{:ok, validated_list}` if all validations succeed\n  * `{:error, errors_by_index}` if any validation fails","ref":"Exdantic.EnhancedValidator.html#validate_many/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate_many/3","doc":"iex> Exdantic.EnhancedValidator.validate_many(:string, [\"a\", \"b\", \"c\"])\n    {:ok, [\"a\", \"b\", \"c\"]}\n\n    iex> Exdantic.EnhancedValidator.validate_many(:integer, [1, \"bad\", 3])\n    {:error, %{1 => [%Exdantic.Error{...}]}}","ref":"Exdantic.EnhancedValidator.html#validate_many/3-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate_with_enhanced_schema/3","doc":"Validates data with enhanced JSON schema generation.\n\nPhase 6 Enhancement: Integrates with EnhancedResolver for complete pipeline.","ref":"Exdantic.EnhancedValidator.html#validate_with_enhanced_schema/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate_with_enhanced_schema/3","doc":"* `target` - What to validate against\n  * `input` - Data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate_with_enhanced_schema/3-parameters"},{"type":"function","title":"Phase 6 Options - Exdantic.EnhancedValidator.validate_with_enhanced_schema/3","doc":"* `:generate_enhanced_schema` - Include enhanced JSON schema in result (default: false)\n  * `:optimize_for_provider` - LLM provider optimization (default: :generic)\n  * `:include_metadata` - Include validation metadata in result (default: false)","ref":"Exdantic.EnhancedValidator.html#validate_with_enhanced_schema/3-phase-6-options"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate_with_enhanced_schema/3","doc":"* `{:ok, validated_data}` or `{:ok, validated_data, enhanced_schema}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.EnhancedValidator.html#validate_with_enhanced_schema/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate_with_enhanced_schema/3","doc":"iex> Exdantic.EnhancedValidator.validate_with_enhanced_schema(\n    ...>   MySchema,\n    ...>   data,\n    ...>   generate_enhanced_schema: true,\n    ...>   optimize_for_provider: :openai\n    ...> )\n    {:ok, validated_data, %{\n      \"type\" => \"object\",\n      \"x-exdantic-enhanced\" => true,\n      \"x-openai-optimized\" => true,\n      ...\n    }}","ref":"Exdantic.EnhancedValidator.html#validate_with_enhanced_schema/3-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate_with_resolved_schema/3","doc":"Validates data and resolves all JSON schema references.","ref":"Exdantic.EnhancedValidator.html#validate_with_resolved_schema/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate_with_resolved_schema/3","doc":"* `target` - What to validate against\n  * `input` - Data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate_with_resolved_schema/3-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate_with_resolved_schema/3","doc":"* `{:ok, validated_data, resolved_schema}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.EnhancedValidator.html#validate_with_resolved_schema/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate_with_resolved_schema/3","doc":"iex> Exdantic.EnhancedValidator.validate_with_resolved_schema(MySchema, data)\n    {:ok, validated_data, %{\"type\" => \"object\", ...}}","ref":"Exdantic.EnhancedValidator.html#validate_with_resolved_schema/3-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate_with_schema/3","doc":"Validates data and generates a JSON schema for the validation target.","ref":"Exdantic.EnhancedValidator.html#validate_with_schema/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate_with_schema/3","doc":"* `target` - What to validate against\n  * `input` - Data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate_with_schema/3-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate_with_schema/3","doc":"* `{:ok, validated_data, json_schema}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.EnhancedValidator.html#validate_with_schema/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate_with_schema/3","doc":"iex> schema = Exdantic.Runtime.create_schema([{:name, :string}])\n    iex> Exdantic.EnhancedValidator.validate_with_schema(schema, %{name: \"John\"})\n    {:ok, %{name: \"John\"}, %{\"type\" => \"object\", ...}}","ref":"Exdantic.EnhancedValidator.html#validate_with_schema/3-examples"},{"type":"function","title":"Exdantic.EnhancedValidator.validate_wrapped/4","doc":"Validates data and wraps it in a temporary schema if needed.","ref":"Exdantic.EnhancedValidator.html#validate_wrapped/4"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validate_wrapped/4","doc":"* `field_name` - Name for the wrapper field\n  * `type_spec` - Type specification for the field\n  * `input` - Data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validate_wrapped/4-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validate_wrapped/4","doc":"* `{:ok, extracted_value}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.EnhancedValidator.html#validate_wrapped/4-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validate_wrapped/4","doc":"iex> Exdantic.EnhancedValidator.validate_wrapped(:result, :integer, \"123\",\n    ...>   config: Exdantic.Config.create(coercion: :safe))\n    {:ok, 123}\n\n    iex> Exdantic.EnhancedValidator.validate_wrapped(:items, {:array, :string}, [\"a\", \"b\"])\n    {:ok, [\"a\", \"b\"]}","ref":"Exdantic.EnhancedValidator.html#validate_wrapped/4-examples"},{"type":"type","title":"Exdantic.EnhancedValidator.validation_input/0","doc":"","ref":"Exdantic.EnhancedValidator.html#t:validation_input/0"},{"type":"function","title":"Exdantic.EnhancedValidator.validation_report/3","doc":"Creates a comprehensive validation report for debugging purposes.","ref":"Exdantic.EnhancedValidator.html#validation_report/3"},{"type":"function","title":"Parameters - Exdantic.EnhancedValidator.validation_report/3","doc":"* `target` - What to validate against\n  * `input` - Data to validate\n  * `opts` - Enhanced validation options","ref":"Exdantic.EnhancedValidator.html#validation_report/3-parameters"},{"type":"function","title":"Returns - Exdantic.EnhancedValidator.validation_report/3","doc":"* Map with detailed validation information","ref":"Exdantic.EnhancedValidator.html#validation_report/3-returns"},{"type":"function","title":"Examples - Exdantic.EnhancedValidator.validation_report/3","doc":"iex> report = Exdantic.EnhancedValidator.validation_report(schema, data)\n    %{\n      validation_result: {:ok, validated_data},\n      json_schema: %{...},\n      target_info: %{...},\n      input_analysis: %{...},\n      performance_metrics: %{...}\n    }","ref":"Exdantic.EnhancedValidator.html#validation_report/3-examples"},{"type":"type","title":"Exdantic.EnhancedValidator.validation_result/0","doc":"","ref":"Exdantic.EnhancedValidator.html#t:validation_result/0"},{"type":"type","title":"Exdantic.EnhancedValidator.validation_result_with_skip/0","doc":"","ref":"Exdantic.EnhancedValidator.html#t:validation_result_with_skip/0"},{"type":"type","title":"Exdantic.EnhancedValidator.validation_target/0","doc":"","ref":"Exdantic.EnhancedValidator.html#t:validation_target/0"},{"type":"module","title":"Exdantic.Error","doc":"Structured error representation for Exdantic validation errors.","ref":"Exdantic.Error.html"},{"type":"function","title":"Exdantic.Error.format/1","doc":"Formats an error into a human-readable string.","ref":"Exdantic.Error.html#format/1"},{"type":"function","title":"Examples - Exdantic.Error.format/1","doc":"iex> error = %Exdantic.Error{path: [:user, :email], code: :format, message: \"invalid format\"}\n    iex> Exdantic.Error.format(error)\n    \"user.email: invalid format\"\n\n    iex> error = %Exdantic.Error{path: [], code: :type, message: \"expected string\"}\n    iex> Exdantic.Error.format(error)\n    \": expected string\"","ref":"Exdantic.Error.html#format/1-examples"},{"type":"function","title":"Exdantic.Error.new/3","doc":"Creates a new validation error.","ref":"Exdantic.Error.html#new/3"},{"type":"function","title":"Parameters - Exdantic.Error.new/3","doc":"* `path` - Path to the field that caused the error\n  * `code` - Error code identifying the type of error\n  * `message` - Human-readable error message","ref":"Exdantic.Error.html#new/3-parameters"},{"type":"function","title":"Examples - Exdantic.Error.new/3","doc":"iex> Exdantic.Error.new([:user, :email], :format, \"invalid email format\")\n    %Exdantic.Error{path: [:user, :email], code: :format, message: \"invalid email format\"}\n\n    iex> Exdantic.Error.new(:name, :required, \"field is required\")\n    %Exdantic.Error{path: [:name], code: :required, message: \"field is required\"}","ref":"Exdantic.Error.html#new/3-examples"},{"type":"type","title":"Exdantic.Error.t/0","doc":"","ref":"Exdantic.Error.html#t:t/0"},{"type":"module","title":"Exdantic.FieldMeta","doc":"Struct for field metadata in Exdantic schemas.\n\nThis module defines the structure and types for field metadata used\nthroughout the Exdantic schema system.","ref":"Exdantic.FieldMeta.html"},{"type":"function","title":"Exdantic.FieldMeta.new/3","doc":"Creates a new FieldMeta struct with the given parameters.","ref":"Exdantic.FieldMeta.html#new/3"},{"type":"function","title":"Parameters - Exdantic.FieldMeta.new/3","doc":"* `name` - The field name (atom)\n  * `type` - The field type definition\n  * `required` - Whether the field is required (boolean)","ref":"Exdantic.FieldMeta.html#new/3-parameters"},{"type":"function","title":"Examples - Exdantic.FieldMeta.new/3","doc":"iex> Exdantic.FieldMeta.new(:email, {:type, :string, []}, true)\n    %Exdantic.FieldMeta{name: :email, type: {:type, :string, []}, required: true}","ref":"Exdantic.FieldMeta.html#new/3-examples"},{"type":"type","title":"Exdantic.FieldMeta.t/0","doc":"","ref":"Exdantic.FieldMeta.html#t:t/0"},{"type":"module","title":"Exdantic.JsonSchema","doc":"A module for converting Exdantic schema definitions into JSON Schema format.\nHandles field types, metadata, references, and definitions generation.","ref":"Exdantic.JsonSchema.html"},{"type":"function","title":"Exdantic.JsonSchema.extract_computed_field_info/1","doc":"Extracts computed field information from a JSON schema.\n\nThis function can parse a JSON schema generated by Exdantic and extract\ninformation about computed fields from the x-computed-field metadata.","ref":"Exdantic.JsonSchema.html#extract_computed_field_info/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.extract_computed_field_info/1","doc":"* `json_schema` - The JSON schema to analyze","ref":"Exdantic.JsonSchema.html#extract_computed_field_info/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.extract_computed_field_info/1","doc":"* List of computed field information maps","ref":"Exdantic.JsonSchema.html#extract_computed_field_info/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.extract_computed_field_info/1","doc":"iex> schema = Exdantic.JsonSchema.from_schema(UserSchema)\n    iex> Exdantic.JsonSchema.extract_computed_field_info(schema)\n    [\n      %{\n        name: \"full_name\",\n        type: %{\"type\" => \"string\"},\n        function: \"UserSchema.generate_full_name/1\",\n        readonly: true\n      }\n    ]","ref":"Exdantic.JsonSchema.html#extract_computed_field_info/1-examples"},{"type":"function","title":"Exdantic.JsonSchema.from_schema/1","doc":"Converts an Exdantic schema module to a JSON Schema representation.","ref":"Exdantic.JsonSchema.html#from_schema/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.from_schema/1","doc":"* `schema` - The schema module to convert","ref":"Exdantic.JsonSchema.html#from_schema/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.from_schema/1","doc":"* A map representing the JSON Schema","ref":"Exdantic.JsonSchema.html#from_schema/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.from_schema/1","doc":"iex> defmodule TestSchema do\n    ...>   use Exdantic\n    ...>   schema do\n    ...>     field :name, :string\n    ...>   end\n    ...> end\n    iex> Exdantic.JsonSchema.from_schema(TestSchema)\n    %{\"type\" => \"object\", \"properties\" => %{\"name\" => %{\"type\" => \"string\"}}}","ref":"Exdantic.JsonSchema.html#from_schema/1-examples"},{"type":"function","title":"Exdantic.JsonSchema.has_computed_fields?/1","doc":"Checks if a JSON schema contains computed fields.","ref":"Exdantic.JsonSchema.html#has_computed_fields?/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.has_computed_fields?/1","doc":"* `json_schema` - The JSON schema to check","ref":"Exdantic.JsonSchema.html#has_computed_fields?/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.has_computed_fields?/1","doc":"* `true` if the schema contains computed fields, `false` otherwise","ref":"Exdantic.JsonSchema.html#has_computed_fields?/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.has_computed_fields?/1","doc":"iex> schema = Exdantic.JsonSchema.from_schema(UserSchema)\n    iex> Exdantic.JsonSchema.has_computed_fields?(schema)\n    true","ref":"Exdantic.JsonSchema.html#has_computed_fields?/1-examples"},{"type":"type","title":"Exdantic.JsonSchema.json_schema/0","doc":"","ref":"Exdantic.JsonSchema.html#t:json_schema/0"},{"type":"function","title":"Exdantic.JsonSchema.remove_computed_fields/1","doc":"Removes computed fields from a JSON schema.\n\nThis can be useful when you want to generate a schema for input validation\nthat excludes computed fields, since computed fields are output-only.","ref":"Exdantic.JsonSchema.html#remove_computed_fields/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.remove_computed_fields/1","doc":"* `json_schema` - The JSON schema to process","ref":"Exdantic.JsonSchema.html#remove_computed_fields/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.remove_computed_fields/1","doc":"* JSON schema with computed fields removed","ref":"Exdantic.JsonSchema.html#remove_computed_fields/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.remove_computed_fields/1","doc":"iex> schema = Exdantic.JsonSchema.from_schema(UserSchema)\n    iex> input_schema = Exdantic.JsonSchema.remove_computed_fields(schema)\n    # input_schema will not contain computed fields","ref":"Exdantic.JsonSchema.html#remove_computed_fields/1-examples"},{"type":"module","title":"Exdantic.JsonSchema.EnhancedResolver","doc":"Enhanced JSON Schema resolution with full computed field and model validator metadata.\n\nThis module extends the basic resolver functionality to handle:\n- Enhanced runtime schemas with model validators and computed fields\n- Dynamic schema references with runtime resolution\n- Cross-schema dependency resolution for complex validation pipelines\n- LLM provider-specific optimizations with enhanced metadata\n\nPhase 6 Enhancement: Complete Integration\n- Seamlessly integrates all Exdantic features (struct patterns, model validators, computed fields)\n- Provides unified JSON Schema generation for all schema types\n- Optimizes for various LLM providers with enhanced metadata\n- Maintains backward compatibility with existing resolvers","ref":"Exdantic.JsonSchema.EnhancedResolver.html"},{"type":"function","title":"Exdantic.JsonSchema.EnhancedResolver.comprehensive_analysis/3","doc":"Creates a comprehensive validation and schema generation report.\n\nUseful for debugging, documentation, and understanding complex schema interactions.","ref":"Exdantic.JsonSchema.EnhancedResolver.html#comprehensive_analysis/3"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.EnhancedResolver.comprehensive_analysis/3","doc":"* `schema_or_spec` - Schema to analyze\n  * `sample_data` - Optional sample data for validation testing\n  * `opts` - Analysis options","ref":"Exdantic.JsonSchema.EnhancedResolver.html#comprehensive_analysis/3-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.EnhancedResolver.comprehensive_analysis/3","doc":"* Comprehensive report with validation results, schema analysis, and metadata","ref":"Exdantic.JsonSchema.EnhancedResolver.html#comprehensive_analysis/3-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.EnhancedResolver.comprehensive_analysis/3","doc":"iex> defmodule AnalysisTestSchema do\n    ...>   use Exdantic\n    ...>   schema do\n    ...>     field :name, :string, required: true\n    ...>   end\n    ...> end\n    iex> report = Exdantic.JsonSchema.EnhancedResolver.comprehensive_analysis(AnalysisTestSchema)\n    iex> report.schema_type\n    :compiled_schema\n    iex> is_map(report.performance_metrics)\n    true","ref":"Exdantic.JsonSchema.EnhancedResolver.html#comprehensive_analysis/3-examples"},{"type":"type","title":"Exdantic.JsonSchema.EnhancedResolver.enhanced_resolution_options/0","doc":"","ref":"Exdantic.JsonSchema.EnhancedResolver.html#t:enhanced_resolution_options/0"},{"type":"function","title":"Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy/2","doc":"Optimizes schemas specifically for DSPy and structured LLM output patterns.\n\nDSPy requires specific JSON Schema patterns for reliable structured output.\nThis function ensures schemas work optimally with DSPy's validation patterns.","ref":"Exdantic.JsonSchema.EnhancedResolver.html#optimize_for_dspy/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy/2","doc":"* `schema_or_spec` - Schema to optimize\n  * `dspy_opts` - DSPy-specific options","ref":"Exdantic.JsonSchema.EnhancedResolver.html#optimize_for_dspy/2-parameters"},{"type":"function","title":"DSPy Options - Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy/2","doc":"* `:signature_mode` - Generate schema for DSPy signature patterns (default: false)\n  * `:strict_types` - Enforce strict type constraints (default: true)\n  * `:remove_computed_fields` - Remove computed fields for input validation (default: false)\n  * `:field_descriptions` - Include field descriptions for better LLM understanding (default: true)","ref":"Exdantic.JsonSchema.EnhancedResolver.html#optimize_for_dspy/2-dspy-options"},{"type":"function","title":"Returns - Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy/2","doc":"* DSPy-optimized JSON Schema","ref":"Exdantic.JsonSchema.EnhancedResolver.html#optimize_for_dspy/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy/2","doc":"iex> defmodule DSPyTestSchema do\n    ...>   use Exdantic\n    ...>   schema do\n    ...>     field :input, :string, required: true\n    ...>   end\n    ...> end\n    iex> dspy_schema = Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy(DSPyTestSchema)\n    iex> dspy_schema[\"x-dspy-optimized\"]\n    true","ref":"Exdantic.JsonSchema.EnhancedResolver.html#optimize_for_dspy/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced/2","doc":"Resolves JSON Schema with full enhanced schema support.\n\nHandles all Exdantic schema types:\n- Compile-time schemas (with struct support, model validators, computed fields)\n- Runtime DynamicSchema instances\n- Runtime EnhancedSchema instances with full features\n- Mixed references between schema types","ref":"Exdantic.JsonSchema.EnhancedResolver.html#resolve_enhanced/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced/2","doc":"* `schema_or_spec` - Schema module, runtime schema, or type specification\n  * `opts` - Enhanced resolution options","ref":"Exdantic.JsonSchema.EnhancedResolver.html#resolve_enhanced/2-parameters"},{"type":"function","title":"Options - Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced/2","doc":"* `:include_model_validators` - Include model validator metadata (default: true)\n  * `:include_computed_fields` - Include computed field metadata (default: true)\n  * `:optimize_for_provider` - Optimize for specific LLM provider (default: :generic)\n  * `:resolve_runtime_refs` - Resolve references to runtime schemas (default: true)\n  * `:max_depth` - Maximum resolution depth (default: 10)\n  * `:flatten_for_llm` - Flatten complex structures for LLM consumption (default: false)","ref":"Exdantic.JsonSchema.EnhancedResolver.html#resolve_enhanced/2-options"},{"type":"function","title":"Returns - Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced/2","doc":"* Enhanced JSON Schema with full metadata","ref":"Exdantic.JsonSchema.EnhancedResolver.html#resolve_enhanced/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced/2","doc":"# Basic usage with a module\n    iex> defmodule TestSchema do\n    ...>   use Exdantic\n    ...>   schema do\n    ...>     field :name, :string, required: true\n    ...>   end\n    ...> end\n    iex> schema = Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced(TestSchema)\n    iex> schema[\"type\"]\n    \"object\"\n    iex> schema[\"x-exdantic-enhanced\"]\n    true","ref":"Exdantic.JsonSchema.EnhancedResolver.html#resolve_enhanced/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.EnhancedResolver.validate_schema_compatibility/2","doc":"Validates that a schema is compatible with enhanced validation pipeline.\n\nChecks for common issues that might cause problems with the full validation pipeline\nincluding model validators and computed fields.","ref":"Exdantic.JsonSchema.EnhancedResolver.html#validate_schema_compatibility/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.EnhancedResolver.validate_schema_compatibility/2","doc":"* `schema_or_spec` - Schema to validate\n  * `opts` - Validation options","ref":"Exdantic.JsonSchema.EnhancedResolver.html#validate_schema_compatibility/2-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.EnhancedResolver.validate_schema_compatibility/2","doc":"* `:ok` if schema is valid\n  * `{:error, issues}` if problems are found","ref":"Exdantic.JsonSchema.EnhancedResolver.html#validate_schema_compatibility/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.EnhancedResolver.validate_schema_compatibility/2","doc":"iex> defmodule CompatibilityTestSchema do\n    ...>   use Exdantic\n    ...>   schema do\n    ...>     field :name, :string, required: true\n    ...>   end\n    ...> end\n    iex> Exdantic.JsonSchema.EnhancedResolver.validate_schema_compatibility(CompatibilityTestSchema)\n    :ok","ref":"Exdantic.JsonSchema.EnhancedResolver.html#validate_schema_compatibility/2-examples"},{"type":"module","title":"Exdantic.JsonSchema.ReferenceStore","doc":"Manages schema references and definitions for JSON Schema generation.\n\nThis module provides a stateful store for tracking schema references\nand their corresponding JSON Schema definitions during conversion.","ref":"Exdantic.JsonSchema.ReferenceStore.html"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.add_definition/3","doc":"Adds a JSON Schema definition for a module.","ref":"Exdantic.JsonSchema.ReferenceStore.html#add_definition/3"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.add_definition/3","doc":"* `agent` - The reference store process PID\n  * `module` - The module for which to store the schema definition\n  * `schema` - The JSON Schema map representation","ref":"Exdantic.JsonSchema.ReferenceStore.html#add_definition/3-parameters"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.add_definition/3","doc":"iex> schema = %{\"type\" => \"object\", \"properties\" => %{}}\n    iex> Exdantic.JsonSchema.ReferenceStore.add_definition(store, MySchema, schema)\n    :ok","ref":"Exdantic.JsonSchema.ReferenceStore.html#add_definition/3-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.add_reference/2","doc":"Adds a schema module reference to track for processing.","ref":"Exdantic.JsonSchema.ReferenceStore.html#add_reference/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.add_reference/2","doc":"* `agent` - The reference store process PID\n  * `module` - The schema module to add as a reference","ref":"Exdantic.JsonSchema.ReferenceStore.html#add_reference/2-parameters"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.add_reference/2","doc":"iex> Exdantic.JsonSchema.ReferenceStore.add_reference(store, MySchema)\n    :ok","ref":"Exdantic.JsonSchema.ReferenceStore.html#add_reference/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.get_definitions/1","doc":"Gets all schema definitions currently stored.","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_definitions/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.get_definitions/1","doc":"* `agent` - The reference store process PID","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_definitions/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.ReferenceStore.get_definitions/1","doc":"* Map of module names to their JSON Schema definitions","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_definitions/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.get_definitions/1","doc":"iex> Exdantic.JsonSchema.ReferenceStore.get_definitions(store)\n    %{\"MySchema\" => %{\"type\" => \"object\"}}","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_definitions/1-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.get_references/1","doc":"Gets all module references currently tracked in the store.","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_references/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.get_references/1","doc":"* `agent` - The reference store process PID","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_references/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.ReferenceStore.get_references/1","doc":"* List of module atoms","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_references/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.get_references/1","doc":"iex> Exdantic.JsonSchema.ReferenceStore.get_references(store)\n    [MySchema, AnotherSchema]","ref":"Exdantic.JsonSchema.ReferenceStore.html#get_references/1-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.has_definition?/2","doc":"Checks if a schema definition exists for a module.","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_definition?/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.has_definition?/2","doc":"* `agent` - The reference store process PID\n  * `module` - The module to check for a definition","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_definition?/2-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.ReferenceStore.has_definition?/2","doc":"* `true` if a definition exists, `false` otherwise","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_definition?/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.has_definition?/2","doc":"iex> Exdantic.JsonSchema.ReferenceStore.has_definition?(store, MySchema)\n    false","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_definition?/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.has_reference?/2","doc":"Checks if a module reference is already tracked in the store.","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_reference?/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.has_reference?/2","doc":"* `agent` - The reference store process PID\n  * `module` - The module to check for","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_reference?/2-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.ReferenceStore.has_reference?/2","doc":"* `true` if the module is tracked, `false` otherwise","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_reference?/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.has_reference?/2","doc":"iex> Exdantic.JsonSchema.ReferenceStore.has_reference?(store, MySchema)\n    true","ref":"Exdantic.JsonSchema.ReferenceStore.html#has_reference?/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.ref_path/1","doc":"Generates a JSON Schema reference path for a module.","ref":"Exdantic.JsonSchema.ReferenceStore.html#ref_path/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.ref_path/1","doc":"* `module` - The module to generate a reference path for","ref":"Exdantic.JsonSchema.ReferenceStore.html#ref_path/1-parameters"},{"type":"function","title":"Returns - Exdantic.JsonSchema.ReferenceStore.ref_path/1","doc":"* JSON Schema reference string in the format \"#/definitions/ModuleName\"","ref":"Exdantic.JsonSchema.ReferenceStore.html#ref_path/1-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.ref_path/1","doc":"iex> Exdantic.JsonSchema.ReferenceStore.ref_path(MySchema)\n    \"#/definitions/MySchema\"","ref":"Exdantic.JsonSchema.ReferenceStore.html#ref_path/1-examples"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.start_link/0","doc":"Starts a new reference store process.","ref":"Exdantic.JsonSchema.ReferenceStore.html#start_link/0"},{"type":"function","title":"Returns - Exdantic.JsonSchema.ReferenceStore.start_link/0","doc":"* `{:ok, pid}` on success","ref":"Exdantic.JsonSchema.ReferenceStore.html#start_link/0-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.start_link/0","doc":"iex> {:ok, store} = Exdantic.JsonSchema.ReferenceStore.start_link()\n    {:ok, #PID<...>}","ref":"Exdantic.JsonSchema.ReferenceStore.html#start_link/0-examples"},{"type":"type","title":"Exdantic.JsonSchema.ReferenceStore.state/0","doc":"","ref":"Exdantic.JsonSchema.ReferenceStore.html#t:state/0"},{"type":"function","title":"Exdantic.JsonSchema.ReferenceStore.stop/1","doc":"Stops the reference store process.","ref":"Exdantic.JsonSchema.ReferenceStore.html#stop/1"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.ReferenceStore.stop/1","doc":"* `agent` - The reference store process PID","ref":"Exdantic.JsonSchema.ReferenceStore.html#stop/1-parameters"},{"type":"function","title":"Examples - Exdantic.JsonSchema.ReferenceStore.stop/1","doc":"iex> {:ok, store} = Exdantic.JsonSchema.ReferenceStore.start_link()\n    iex> Exdantic.JsonSchema.ReferenceStore.stop(store)\n    :ok","ref":"Exdantic.JsonSchema.ReferenceStore.html#stop/1-examples"},{"type":"module","title":"Exdantic.JsonSchema.Resolver","doc":"Advanced JSON schema reference resolution and manipulation.\n\nThis module provides functionality for resolving $ref entries, flattening\nnested schemas, and enforcing structured output requirements for various\nLLM providers (OpenAI, Anthropic, etc.).","ref":"Exdantic.JsonSchema.Resolver.html"},{"type":"function","title":"Exdantic.JsonSchema.Resolver.enforce_structured_output/2","doc":"Enforces structured output requirements for specific LLM providers.","ref":"Exdantic.JsonSchema.Resolver.html#enforce_structured_output/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.Resolver.enforce_structured_output/2","doc":"* `schema` - The JSON schema to enforce requirements on\n  * `opts` - Provider-specific options","ref":"Exdantic.JsonSchema.Resolver.html#enforce_structured_output/2-parameters"},{"type":"function","title":"Options - Exdantic.JsonSchema.Resolver.enforce_structured_output/2","doc":"* `:provider` - Target provider (:openai, :anthropic, :generic) (default: :generic)\n  * `:remove_unsupported` - Remove unsupported features (default: true)\n  * `:add_required_fields` - Add required fields for provider (default: true)","ref":"Exdantic.JsonSchema.Resolver.html#enforce_structured_output/2-options"},{"type":"function","title":"Returns - Exdantic.JsonSchema.Resolver.enforce_structured_output/2","doc":"* JSON schema compatible with the specified provider","ref":"Exdantic.JsonSchema.Resolver.html#enforce_structured_output/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.Resolver.enforce_structured_output/2","doc":"iex> schema = %{\"type\" => \"object\", \"additionalProperties\" => true}\n    iex> Exdantic.JsonSchema.Resolver.enforce_structured_output(schema, provider: :openai)\n    %{\"type\" => \"object\", \"additionalProperties\" => false}","ref":"Exdantic.JsonSchema.Resolver.html#enforce_structured_output/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.Resolver.flatten_schema/2","doc":"Flattens nested schemas by expanding all references inline.\n\nThis is useful for LLM providers that don't support complex reference structures.","ref":"Exdantic.JsonSchema.Resolver.html#flatten_schema/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.Resolver.flatten_schema/2","doc":"* `schema` - The JSON schema to flatten\n  * `opts` - Flattening options","ref":"Exdantic.JsonSchema.Resolver.html#flatten_schema/2-parameters"},{"type":"function","title":"Options - Exdantic.JsonSchema.Resolver.flatten_schema/2","doc":"* `:max_depth` - Maximum flattening depth (default: 5)\n  * `:inline_simple_refs` - Inline simple type references (default: true)\n  * `:preserve_complex_refs` - Keep complex object references as-is (default: false)","ref":"Exdantic.JsonSchema.Resolver.html#flatten_schema/2-options"},{"type":"function","title":"Returns - Exdantic.JsonSchema.Resolver.flatten_schema/2","doc":"* Flattened JSON schema","ref":"Exdantic.JsonSchema.Resolver.html#flatten_schema/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.Resolver.flatten_schema/2","doc":"iex> schema = %{\n    ...>   \"type\" => \"object\",\n    ...>   \"properties\" => %{\n    ...>     \"items\" => %{\n    ...>       \"type\" => \"array\",\n    ...>       \"items\" => %{\"$ref\" => \"#/definitions/Item\"}\n    ...>     }\n    ...>   },\n    ...>   \"definitions\" => %{\n    ...>     \"Item\" => %{\"type\" => \"string\", \"minLength\" => 1}\n    ...>   }\n    ...> }\n    iex> Exdantic.JsonSchema.Resolver.flatten_schema(schema)\n    %{\n      \"type\" => \"object\",\n      \"properties\" => %{\n        \"items\" => %{\n          \"type\" => \"array\",\n          \"items\" => %{\"type\" => \"string\", \"minLength\" => 1}\n        }\n      }\n    }","ref":"Exdantic.JsonSchema.Resolver.html#flatten_schema/2-examples"},{"type":"function","title":"Exdantic.JsonSchema.Resolver.optimize_for_llm/2","doc":"Optimizes a JSON schema for better LLM performance.","ref":"Exdantic.JsonSchema.Resolver.html#optimize_for_llm/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.Resolver.optimize_for_llm/2","doc":"* `schema` - The JSON schema to optimize\n  * `opts` - Optimization options","ref":"Exdantic.JsonSchema.Resolver.html#optimize_for_llm/2-parameters"},{"type":"function","title":"Options - Exdantic.JsonSchema.Resolver.optimize_for_llm/2","doc":"* `:remove_descriptions` - Remove verbose descriptions (default: false)\n  * `:simplify_unions` - Simplify complex union types (default: true)\n  * `:max_properties` - Maximum properties per object (default: nil)","ref":"Exdantic.JsonSchema.Resolver.html#optimize_for_llm/2-options"},{"type":"function","title":"Returns - Exdantic.JsonSchema.Resolver.optimize_for_llm/2","doc":"* Optimized JSON schema","ref":"Exdantic.JsonSchema.Resolver.html#optimize_for_llm/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.Resolver.optimize_for_llm/2","doc":"iex> schema = %{\"type\" => \"object\", \"properties\" => %{\"a\" => %{\"type\" => \"string\", \"description\" => \"very long description...\"}}}\n    iex> Exdantic.JsonSchema.Resolver.optimize_for_llm(schema, remove_descriptions: true)\n    %{\"type\" => \"object\", \"properties\" => %{\"a\" => %{\"type\" => \"string\"}}}","ref":"Exdantic.JsonSchema.Resolver.html#optimize_for_llm/2-examples"},{"type":"type","title":"Exdantic.JsonSchema.Resolver.resolution_options/0","doc":"","ref":"Exdantic.JsonSchema.Resolver.html#t:resolution_options/0"},{"type":"function","title":"Exdantic.JsonSchema.Resolver.resolve_references/2","doc":"Recursively resolves all $ref entries in a schema.","ref":"Exdantic.JsonSchema.Resolver.html#resolve_references/2"},{"type":"function","title":"Parameters - Exdantic.JsonSchema.Resolver.resolve_references/2","doc":"* `schema` - The JSON schema to resolve references in\n  * `opts` - Resolution options","ref":"Exdantic.JsonSchema.Resolver.html#resolve_references/2-parameters"},{"type":"function","title":"Options - Exdantic.JsonSchema.Resolver.resolve_references/2","doc":"* `:max_depth` - Maximum resolution depth to prevent infinite recursion (default: 10)\n  * `:preserve_titles` - Keep original titles when resolving (default: true)\n  * `:preserve_descriptions` - Keep original descriptions when resolving (default: true)","ref":"Exdantic.JsonSchema.Resolver.html#resolve_references/2-options"},{"type":"function","title":"Returns - Exdantic.JsonSchema.Resolver.resolve_references/2","doc":"* Resolved JSON schema with all references expanded","ref":"Exdantic.JsonSchema.Resolver.html#resolve_references/2-returns"},{"type":"function","title":"Examples - Exdantic.JsonSchema.Resolver.resolve_references/2","doc":"iex> schema = %{\n    ...>   \"type\" => \"object\",\n    ...>   \"properties\" => %{\n    ...>     \"user\" => %{\"$ref\" => \"#/definitions/User\"}\n    ...>   },\n    ...>   \"definitions\" => %{\n    ...>     \"User\" => %{\"type\" => \"object\", \"properties\" => %{\"name\" => %{\"type\" => \"string\"}}}\n    ...>   }\n    ...> }\n    iex> Exdantic.JsonSchema.Resolver.resolve_references(schema)\n    %{\n      \"type\" => \"object\",\n      \"properties\" => %{\n        \"user\" => %{\"type\" => \"object\", \"properties\" => %{\"name\" => %{\"type\" => \"string\"}}}\n      }\n    }","ref":"Exdantic.JsonSchema.Resolver.html#resolve_references/2-examples"},{"type":"type","title":"Exdantic.JsonSchema.Resolver.schema/0","doc":"","ref":"Exdantic.JsonSchema.Resolver.html#t:schema/0"},{"type":"module","title":"Exdantic.JsonSchema.TypeMapper","doc":"Converts Exdantic type definitions to JSON Schema type definitions.\n\nThis module handles the conversion between Exdantic's internal type system\nand JSON Schema representations, including complex types and constraints.","ref":"Exdantic.JsonSchema.TypeMapper.html"},{"type":"function","title":"Exdantic.JsonSchema.TypeMapper.to_json_schema/2","doc":"","ref":"Exdantic.JsonSchema.TypeMapper.html#to_json_schema/2"},{"type":"module","title":"Exdantic.RootSchema","doc":"RootSchema allows validation of non-dictionary types at the top level.\n\nSimilar to Pydantic's RootModel, this enables validation of values that are not\nmaps/objects, such as arrays, primitives, or other structured data at the root level.","ref":"Exdantic.RootSchema.html"},{"type":"module","title":"Examples - Exdantic.RootSchema","doc":"# Validate a list of integers\n    defmodule IntegerListSchema do\n      use Exdantic.RootSchema, root: {:array, :integer}\n    end\n\n    {:ok, [1, 2, 3]} = IntegerListSchema.validate([1, 2, 3])\n\n    # Validate a single string with constraints\n    defmodule EmailSchema do\n      use Exdantic.RootSchema,\n        root: {:type, :string, [format: ~r/^[^ @]+@[^ @]+.[^ @]+$/]}\n    end\n\n    {:ok, \"user@example.com\"} = EmailSchema.validate(\"user@example.com\")\n\n    # Validate a union type\n    defmodule StringOrNumberSchema do\n      use Exdantic.RootSchema, root: {:union, [:string, :integer]}\n    end\n\n    {:ok, \"hello\"} = StringOrNumberSchema.validate(\"hello\")\n    {:ok, 42} = StringOrNumberSchema.validate(42)","ref":"Exdantic.RootSchema.html#module-examples"},{"type":"macro","title":"Exdantic.RootSchema.__using__/1","doc":"Configures a module to be a RootSchema for validating non-dictionary types.","ref":"Exdantic.RootSchema.html#__using__/1"},{"type":"macro","title":"Options - Exdantic.RootSchema.__using__/1","doc":"* `:root` - The type definition for the root value. This can be any valid\n    Exdantic type definition including basic types, arrays, maps, unions, etc.","ref":"Exdantic.RootSchema.html#__using__/1-options"},{"type":"macro","title":"Examples - Exdantic.RootSchema.__using__/1","doc":"# Simple array validation\n    defmodule NumberListSchema do\n      use Exdantic.RootSchema, root: {:array, :integer}\n    end\n\n    # Complex nested structure\n    defmodule NestedSchema do\n      use Exdantic.RootSchema,\n        root: {:array, {:map, {:string, {:union, [:string, :integer]}}}}\n    end\n\n    # Reference to another schema\n    defmodule UserListSchema do\n      use Exdantic.RootSchema, root: {:array, UserSchema}\n    end","ref":"Exdantic.RootSchema.html#__using__/1-examples"},{"type":"function","title":"Exdantic.RootSchema.to_json_schema/1","doc":"Converts a root type definition to a JSON Schema.","ref":"Exdantic.RootSchema.html#to_json_schema/1"},{"type":"function","title":"Parameters - Exdantic.RootSchema.to_json_schema/1","doc":"* `root_type` - The type definition to convert","ref":"Exdantic.RootSchema.html#to_json_schema/1-parameters"},{"type":"function","title":"Returns - Exdantic.RootSchema.to_json_schema/1","doc":"* A map representing the JSON Schema","ref":"Exdantic.RootSchema.html#to_json_schema/1-returns"},{"type":"function","title":"Exdantic.RootSchema.validate_root/2","doc":"Validates data against a root type definition.\n\nThis is the core validation function used by RootSchema modules.","ref":"Exdantic.RootSchema.html#validate_root/2"},{"type":"function","title":"Parameters - Exdantic.RootSchema.validate_root/2","doc":"* `root_type` - The type definition to validate against\n  * `data` - The data to validate","ref":"Exdantic.RootSchema.html#validate_root/2-parameters"},{"type":"function","title":"Returns - Exdantic.RootSchema.validate_root/2","doc":"* `{:ok, validated_data}` on success\n  * `{:error, errors}` on validation failures","ref":"Exdantic.RootSchema.html#validate_root/2-returns"},{"type":"module","title":"Exdantic.Runtime","doc":"Runtime schema generation and validation capabilities.\n\nThis module enables dynamic schema creation from field definitions at runtime,\nsupporting the DSPy pattern of `pydantic.create_model(\"DSPyProgramOutputs\", **fields)`.","ref":"Exdantic.Runtime.html"},{"type":"module","title":"Phase 5 Enhancement: Enhanced Runtime Schemas - Exdantic.Runtime","doc":"Added support for enhanced runtime schemas with model validators and computed fields:\n\n    # Create enhanced schema with full validation pipeline\n    fields = [{:name, :string, [required: true]}, {:age, :integer, [optional: true]}]\n\n    # Model validators for cross-field validation\n    validators = [\n      fn data -> {:ok, %{data | name: String.trim(data.name)}} end,\n      {MyModule, :validate_age}\n    ]\n\n    # Computed fields for derived values\n    computed_fields = [\n      {:display_name, :string, fn data -> {:ok, String.upcase(data.name)} end},\n      {:age_group, :string, {MyModule, :compute_age_group}}\n    ]\n\n    # Create enhanced schema\n    schema = Runtime.create_enhanced_schema(fields,\n      title: \"User Schema\",\n      model_validators: validators,\n      computed_fields: computed_fields\n    )\n\n    # Validate with full pipeline\n    {:ok, result} = Runtime.validate_enhanced(%{name: \"  john  \", age: 25}, schema)\n    # Result: %{name: \"john\", age: 25, display_name: \"JOHN\", age_group: \"adult\"}\n\nEnhanced schemas support:\n- Model validators (both named functions and anonymous functions)\n- Computed fields (both named functions and anonymous functions)\n- Full validation pipeline execution (field → model → computed)\n- JSON Schema generation with enhanced metadata\n- Integration with existing validation infrastructure","ref":"Exdantic.Runtime.html#module-phase-5-enhancement-enhanced-runtime-schemas"},{"type":"module","title":"Basic Runtime Schemas - Exdantic.Runtime","doc":"For simple use cases without enhanced features:\n\n    # Basic runtime schema\n    fields = [{:name, :string, [required: true]}, {:age, :integer, [optional: true]}]\n    schema = Runtime.create_schema(fields, title: \"Basic User Schema\")\n\n    {:ok, validated} = Runtime.validate(%{name: \"John\", age: 30}, schema)\n    # Result: %{name: \"John\", age: 30}","ref":"Exdantic.Runtime.html#module-basic-runtime-schemas"},{"type":"module","title":"Schema Types - Exdantic.Runtime","doc":"- `DynamicSchema` - Basic runtime schema with field validation\n- `EnhancedSchema` - Advanced runtime schema with model validators and computed fields","ref":"Exdantic.Runtime.html#module-schema-types"},{"type":"module","title":"Phase 5 Migration Guide - Exdantic.Runtime","doc":"","ref":"Exdantic.Runtime.html#module-phase-5-migration-guide"},{"type":"module","title":"Upgrading to Enhanced Runtime Schemas - Exdantic.Runtime","doc":"Phase 5 adds enhanced runtime schemas while maintaining 100% backward compatibility with existing DynamicSchema usage.\n\n#### Existing Code (Still Works)\n```elixir\n# All existing runtime schema code continues to work unchanged\nfields = [{:name, :string, [required: true]}]\nschema = Exdantic.Runtime.create_schema(fields)\n{:ok, result} = Exdantic.Runtime.validate(data, schema)\n```\n\n#### New Enhanced Features\n```elixir\n# Create enhanced schema with model validators and computed fields\nfields = [{:name, :string, [required: true]}]\n\nvalidators = [fn data -> {:ok, %{data | name: String.trim(data.name)}} end]\ncomputed = [{:display_name, :string, fn data -> {:ok, String.upcase(data.name)} end}]\n\nenhanced_schema = Exdantic.Runtime.create_enhanced_schema(fields,\n  model_validators: validators,\n  computed_fields: computed\n)\n\n{:ok, result} = Exdantic.Runtime.validate_enhanced(data, enhanced_schema)\n```\n\n#### Unified Validation Interface\n```elixir\n# Use Exdantic.Runtime.Validator for unified interface\nalias Exdantic.Runtime.Validator\n\n# Works with both DynamicSchema and EnhancedSchema\n{:ok, result} = Validator.validate(data, any_runtime_schema)\njson_schema = Validator.to_json_schema(any_runtime_schema)\ninfo = Validator.schema_info(any_runtime_schema)\n```","ref":"Exdantic.Runtime.html#module-upgrading-to-enhanced-runtime-schemas"},{"type":"module","title":"Breaking Changes - Exdantic.Runtime","doc":"None. All existing code continues to work without modification.","ref":"Exdantic.Runtime.html#module-breaking-changes"},{"type":"module","title":"New Dependencies - Exdantic.Runtime","doc":"None. Phase 5 uses only existing Exdantic modules and standard library functions.","ref":"Exdantic.Runtime.html#module-new-dependencies"},{"type":"module","title":"Performance Impact - Exdantic.Runtime","doc":"- DynamicSchema validation performance unchanged\n- EnhancedSchema adds minimal overhead for model validator and computed field execution\n- JSON schema generation includes computed field metadata with negligible performance impact","ref":"Exdantic.Runtime.html#module-performance-impact"},{"type":"module","title":"Implementation Notes - Exdantic.Runtime","doc":"","ref":"Exdantic.Runtime.html#module-implementation-notes"},{"type":"module","title":"Function Storage - Exdantic.Runtime","doc":"Enhanced schemas store anonymous functions in a runtime function registry, ensuring they can be executed during validation while maintaining clean serialization for schema metadata.","ref":"Exdantic.Runtime.html#module-function-storage"},{"type":"module","title":"Error Handling - Exdantic.Runtime","doc":"Enhanced schemas provide comprehensive error handling:\n- Field validation errors maintain existing behavior\n- Model validator errors include clear context and function references\n- Computed field errors specify which computation failed and why\n- Type validation errors for computed field return values","ref":"Exdantic.Runtime.html#module-error-handling"},{"type":"module","title":"JSON Schema Integration - Exdantic.Runtime","doc":"Enhanced schemas generate JSON schemas that include:\n- All regular fields with their types and constraints\n- Computed fields marked as `readOnly: true`\n- Enhanced metadata (`x-enhanced-schema`, `x-model-validators`, `x-computed-fields`)\n- Full compatibility with existing JSON schema tooling","ref":"Exdantic.Runtime.html#module-json-schema-integration"},{"type":"module","title":"Memory Management - Exdantic.Runtime","doc":"Runtime functions are stored efficiently with unique generated names to prevent conflicts. The function registry is cleaned up when the schema is garbage collected.","ref":"Exdantic.Runtime.html#module-memory-management"},{"type":"module","title":"Testing Strategy - Exdantic.Runtime","doc":"Phase 5 includes comprehensive tests covering:\n- Basic enhanced schema creation and validation\n- Model validator execution (named and anonymous functions)\n- Computed field execution (named and anonymous functions)\n- Error handling at each pipeline stage\n- JSON schema generation with enhanced features\n- Integration with existing validation infrastructure\n- Performance benchmarks for enhanced vs basic schemas\n\nAll existing tests continue to pass, ensuring backward compatibility.","ref":"Exdantic.Runtime.html#module-testing-strategy"},{"type":"function","title":"Exdantic.Runtime.create_enhanced_schema/2","doc":"Creates an enhanced runtime schema with model validators and computed fields.\n\nThis function provides a convenient way to create schemas with enhanced features\nsimilar to compile-time schemas but generated at runtime.","ref":"Exdantic.Runtime.html#create_enhanced_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.create_enhanced_schema/2","doc":"* `field_definitions` - List of field definitions\n  * `opts` - Enhanced schema options","ref":"Exdantic.Runtime.html#create_enhanced_schema/2-parameters"},{"type":"function","title":"Options - Exdantic.Runtime.create_enhanced_schema/2","doc":"* `:model_validators` - List of model validator functions\n  * `:computed_fields` - List of computed field specifications\n  * Standard options: `:title`, `:description`, `:strict`, `:name`","ref":"Exdantic.Runtime.html#create_enhanced_schema/2-options"},{"type":"function","title":"Examples - Exdantic.Runtime.create_enhanced_schema/2","doc":"iex> fields = [{:name, :string, [required: true]}, {:age, :integer, [optional: true]}]\n    iex> validators = [fn data -> {:ok, %{data | name: String.trim(data.name)}} end]\n    iex> computed = [{:display_name, :string, fn data -> {:ok, String.upcase(data.name)} end}]\n    iex> schema = Exdantic.Runtime.create_enhanced_schema(fields,\n    ...>   model_validators: validators,\n    ...>   computed_fields: computed\n    ...> )\n    %Exdantic.Runtime.EnhancedSchema{...}","ref":"Exdantic.Runtime.html#create_enhanced_schema/2-examples"},{"type":"function","title":"Exdantic.Runtime.create_enhanced_schema_v6/2","doc":"Creates an enhanced schema with Phase 6 integration features.\n\nPhase 6 Enhancement: Complete integration with EnhancedResolver and validation pipeline.","ref":"Exdantic.Runtime.html#create_enhanced_schema_v6/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.create_enhanced_schema_v6/2","doc":"* `field_definitions` - Field definitions\n  * `opts` - Enhanced schema options with Phase 6 features","ref":"Exdantic.Runtime.html#create_enhanced_schema_v6/2-parameters"},{"type":"function","title":"Phase 6 Options - Exdantic.Runtime.create_enhanced_schema_v6/2","doc":"* `:auto_optimize_for_provider` - Automatically optimize for LLM provider (default: nil)\n  * `:include_validation_metadata` - Include validation metadata in schema (default: false)\n  * `:dspy_compatible` - Ensure DSPy compatibility (default: false)\n  * All existing options from previous phases","ref":"Exdantic.Runtime.html#create_enhanced_schema_v6/2-phase-6-options"},{"type":"function","title":"Examples - Exdantic.Runtime.create_enhanced_schema_v6/2","doc":"iex> schema = Exdantic.Runtime.create_enhanced_schema_v6(fields,\n    ...>   model_validators: validators,\n    ...>   computed_fields: computed,\n    ...>   auto_optimize_for_provider: :openai,\n    ...>   dspy_compatible: true\n    ...> )","ref":"Exdantic.Runtime.html#create_enhanced_schema_v6/2-examples"},{"type":"function","title":"Exdantic.Runtime.create_schema/2","doc":"Creates a schema at runtime from field definitions.","ref":"Exdantic.Runtime.html#create_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.create_schema/2","doc":"* `field_definitions` - List of field definitions in the format:\n    - `{field_name, type}`\n    - `{field_name, type, options}`\n  * `opts` - Schema configuration options","ref":"Exdantic.Runtime.html#create_schema/2-parameters"},{"type":"function","title":"Options - Exdantic.Runtime.create_schema/2","doc":"* `:title` - Schema title\n  * `:description` - Schema description\n  * `:strict` - Enable strict validation (default: false)\n  * `:name` - Schema name for references","ref":"Exdantic.Runtime.html#create_schema/2-options"},{"type":"function","title":"Examples - Exdantic.Runtime.create_schema/2","doc":"iex> fields = [\n    ...>   {:name, :string, [required: true, min_length: 2]},\n    ...>   {:age, :integer, [optional: true, gt: 0]},\n    ...>   {:email, :string, [required: true, format: ~r/@/]}\n    ...> ]\n    iex> schema = Exdantic.Runtime.create_schema(fields, title: \"User Schema\")\n    %Exdantic.Runtime.DynamicSchema{...}","ref":"Exdantic.Runtime.html#create_schema/2-examples"},{"type":"function","title":"Exdantic.Runtime.enhanced_to_json_schema/2","doc":"Generates JSON Schema for enhanced runtime schemas.","ref":"Exdantic.Runtime.html#enhanced_to_json_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.enhanced_to_json_schema/2","doc":"* `enhanced_schema` - An EnhancedSchema struct\n  * `opts` - JSON Schema generation options","ref":"Exdantic.Runtime.html#enhanced_to_json_schema/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.enhanced_to_json_schema/2","doc":"* JSON Schema map including computed field metadata","ref":"Exdantic.Runtime.html#enhanced_to_json_schema/2-returns"},{"type":"type","title":"Exdantic.Runtime.field_definition/0","doc":"","ref":"Exdantic.Runtime.html#t:field_definition/0"},{"type":"type","title":"Exdantic.Runtime.schema_option/0","doc":"","ref":"Exdantic.Runtime.html#t:schema_option/0"},{"type":"function","title":"Exdantic.Runtime.to_json_schema/2","doc":"Generates JSON Schema from a runtime schema.","ref":"Exdantic.Runtime.html#to_json_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.to_json_schema/2","doc":"* `dynamic_schema` - A DynamicSchema struct\n  * `opts` - JSON Schema generation options","ref":"Exdantic.Runtime.html#to_json_schema/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.to_json_schema/2","doc":"* JSON Schema map","ref":"Exdantic.Runtime.html#to_json_schema/2-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.to_json_schema/2","doc":"iex> json_schema = Exdantic.Runtime.to_json_schema(schema)\n    %{\"type\" => \"object\", \"properties\" => %{...}}","ref":"Exdantic.Runtime.html#to_json_schema/2-examples"},{"type":"type","title":"Exdantic.Runtime.type_spec/0","doc":"","ref":"Exdantic.Runtime.html#t:type_spec/0"},{"type":"function","title":"Exdantic.Runtime.validate/3","doc":"Validates data against a runtime-created schema.","ref":"Exdantic.Runtime.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.validate/3","doc":"* `data` - The data to validate (map)\n  * `dynamic_schema` - A DynamicSchema struct\n  * `opts` - Validation options","ref":"Exdantic.Runtime.html#validate/3-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.validate/3","doc":"* `{:ok, validated_data}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.Runtime.html#validate/3-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.validate/3","doc":"iex> data = %{name: \"John\", age: 30}\n    iex> Exdantic.Runtime.validate(data, schema)\n    {:ok, %{name: \"John\", age: 30}}","ref":"Exdantic.Runtime.html#validate/3-examples"},{"type":"function","title":"Exdantic.Runtime.validate_enhanced/3","doc":"Validates data against an enhanced runtime schema.","ref":"Exdantic.Runtime.html#validate_enhanced/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.validate_enhanced/3","doc":"* `data` - The data to validate (map)\n  * `enhanced_schema` - An EnhancedSchema struct\n  * `opts` - Validation options","ref":"Exdantic.Runtime.html#validate_enhanced/3-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.validate_enhanced/3","doc":"* `{:ok, validated_data}` on success (includes computed fields)\n  * `{:error, errors}` on validation failure","ref":"Exdantic.Runtime.html#validate_enhanced/3-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.validate_enhanced/3","doc":"iex> data = %{name: \"  John  \", age: 30}\n    iex> Exdantic.Runtime.validate_enhanced(data, schema)\n    {:ok, %{name: \"John\", age: 30, display_name: \"JOHN\"}}","ref":"Exdantic.Runtime.html#validate_enhanced/3-examples"},{"type":"function","title":"Exdantic.Runtime.validate_enhanced_v6/3","doc":"Validates an enhanced schema with complete pipeline testing.\n\nPhase 6 Enhancement: Comprehensive validation testing including all features.","ref":"Exdantic.Runtime.html#validate_enhanced_v6/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.validate_enhanced_v6/3","doc":"* `data` - Data to validate\n  * `enhanced_schema` - Enhanced schema\n  * `opts` - Validation options with Phase 6 features","ref":"Exdantic.Runtime.html#validate_enhanced_v6/3-parameters"},{"type":"function","title":"Phase 6 Options - Exdantic.Runtime.validate_enhanced_v6/3","doc":"* `:test_all_providers` - Test compatibility with all LLM providers (default: false)\n  * `:generate_performance_report` - Include performance metrics (default: false)\n  * `:validate_json_schema` - Validate generated JSON schema (default: false)","ref":"Exdantic.Runtime.html#validate_enhanced_v6/3-phase-6-options"},{"type":"function","title":"Returns - Exdantic.Runtime.validate_enhanced_v6/3","doc":"* Enhanced validation result with optional additional information","ref":"Exdantic.Runtime.html#validate_enhanced_v6/3-returns"},{"type":"module","title":"Exdantic.Runtime.DynamicSchema","doc":"Represents a schema created at runtime with field definitions and configuration.\n\nThis struct holds all the information needed to validate data against a\ndynamically created schema, including field metadata, configuration options,\nand runtime metadata.","ref":"Exdantic.Runtime.DynamicSchema.html"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.add_field/3","doc":"Adds a new field to the schema.","ref":"Exdantic.Runtime.DynamicSchema.html#add_field/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.add_field/3","doc":"* `schema` - The DynamicSchema instance\n  * `field_name` - The field name (atom)\n  * `field_meta` - The FieldMeta definition","ref":"Exdantic.Runtime.DynamicSchema.html#add_field/3-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.add_field/3","doc":"* Updated DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#add_field/3-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.add_field/3","doc":"iex> field_meta = %Exdantic.FieldMeta{name: :bio, type: {:type, :string, []}, required: false}\n    iex> updated = Exdantic.Runtime.DynamicSchema.add_field(schema, :bio, field_meta)\n    %Exdantic.Runtime.DynamicSchema{...}","ref":"Exdantic.Runtime.DynamicSchema.html#add_field/3-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.field_names/1","doc":"Lists all field names in the schema.","ref":"Exdantic.Runtime.DynamicSchema.html#field_names/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.field_names/1","doc":"* `schema` - The DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#field_names/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.field_names/1","doc":"* List of field names (atoms)","ref":"Exdantic.Runtime.DynamicSchema.html#field_names/1-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.field_names/1","doc":"iex> Exdantic.Runtime.DynamicSchema.field_names(schema)\n    [:name, :age, :email]","ref":"Exdantic.Runtime.DynamicSchema.html#field_names/1-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.get_field/2","doc":"Gets the field definition for a specific field name.","ref":"Exdantic.Runtime.DynamicSchema.html#get_field/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.get_field/2","doc":"* `schema` - The DynamicSchema instance\n  * `field_name` - The field name (atom)","ref":"Exdantic.Runtime.DynamicSchema.html#get_field/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.get_field/2","doc":"* `{:ok, field_meta}` if field exists\n  * `:error` if field not found","ref":"Exdantic.Runtime.DynamicSchema.html#get_field/2-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.get_field/2","doc":"iex> Exdantic.Runtime.DynamicSchema.get_field(schema, :name)\n    {:ok, %Exdantic.FieldMeta{...}}\n\n    iex> Exdantic.Runtime.DynamicSchema.get_field(schema, :nonexistent)\n    :error","ref":"Exdantic.Runtime.DynamicSchema.html#get_field/2-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.new/4","doc":"Creates a new DynamicSchema instance.","ref":"Exdantic.Runtime.DynamicSchema.html#new/4"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.new/4","doc":"* `name` - Unique identifier for the schema\n  * `fields` - Map of field name to FieldMeta\n  * `config` - Schema configuration options\n  * `metadata` - Optional runtime metadata","ref":"Exdantic.Runtime.DynamicSchema.html#new/4-parameters"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.new/4","doc":"iex> fields = %{name: %Exdantic.FieldMeta{...}}\n    iex> config = %{title: \"User Schema\", strict: true}\n    iex> Exdantic.Runtime.DynamicSchema.new(\"UserSchema\", fields, config)\n    %Exdantic.Runtime.DynamicSchema{...}","ref":"Exdantic.Runtime.DynamicSchema.html#new/4-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.optional_fields/1","doc":"Gets the optional field names from the schema.","ref":"Exdantic.Runtime.DynamicSchema.html#optional_fields/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.optional_fields/1","doc":"* `schema` - The DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#optional_fields/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.optional_fields/1","doc":"* List of optional field names (atoms)","ref":"Exdantic.Runtime.DynamicSchema.html#optional_fields/1-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.optional_fields/1","doc":"iex> Exdantic.Runtime.DynamicSchema.optional_fields(schema)\n    [:age, :bio]","ref":"Exdantic.Runtime.DynamicSchema.html#optional_fields/1-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.remove_field/2","doc":"Removes a field from the schema.","ref":"Exdantic.Runtime.DynamicSchema.html#remove_field/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.remove_field/2","doc":"* `schema` - The DynamicSchema instance\n  * `field_name` - The field name to remove (atom)","ref":"Exdantic.Runtime.DynamicSchema.html#remove_field/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.remove_field/2","doc":"* Updated DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#remove_field/2-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.remove_field/2","doc":"iex> updated = Exdantic.Runtime.DynamicSchema.remove_field(schema, :bio)\n    %Exdantic.Runtime.DynamicSchema{...}","ref":"Exdantic.Runtime.DynamicSchema.html#remove_field/2-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.required_fields/1","doc":"Gets the required field names from the schema.","ref":"Exdantic.Runtime.DynamicSchema.html#required_fields/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.required_fields/1","doc":"* `schema` - The DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#required_fields/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.required_fields/1","doc":"* List of required field names (atoms)","ref":"Exdantic.Runtime.DynamicSchema.html#required_fields/1-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.required_fields/1","doc":"iex> Exdantic.Runtime.DynamicSchema.required_fields(schema)\n    [:name, :email]","ref":"Exdantic.Runtime.DynamicSchema.html#required_fields/1-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.strict?/1","doc":"Checks if the schema is configured for strict validation.","ref":"Exdantic.Runtime.DynamicSchema.html#strict?/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.strict?/1","doc":"* `schema` - The DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#strict?/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.strict?/1","doc":"* `true` if strict mode is enabled, `false` otherwise","ref":"Exdantic.Runtime.DynamicSchema.html#strict?/1-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.strict?/1","doc":"iex> Exdantic.Runtime.DynamicSchema.strict?(schema)\n    true","ref":"Exdantic.Runtime.DynamicSchema.html#strict?/1-examples"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.summary/1","doc":"Returns a summary of the schema structure.","ref":"Exdantic.Runtime.DynamicSchema.html#summary/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.summary/1","doc":"* `schema` - The DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#summary/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.summary/1","doc":"* Map with schema summary information","ref":"Exdantic.Runtime.DynamicSchema.html#summary/1-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.summary/1","doc":"iex> Exdantic.Runtime.DynamicSchema.summary(schema)\n    %{\n      name: \"UserSchema\",\n      field_count: 3,\n      required_count: 2,\n      optional_count: 1,\n      strict: true\n    }","ref":"Exdantic.Runtime.DynamicSchema.html#summary/1-examples"},{"type":"type","title":"Exdantic.Runtime.DynamicSchema.t/0","doc":"","ref":"Exdantic.Runtime.DynamicSchema.html#t:t/0"},{"type":"function","title":"Exdantic.Runtime.DynamicSchema.update_config/2","doc":"Updates the schema configuration.","ref":"Exdantic.Runtime.DynamicSchema.html#update_config/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.DynamicSchema.update_config/2","doc":"* `schema` - The DynamicSchema instance\n  * `new_config` - Configuration options to merge","ref":"Exdantic.Runtime.DynamicSchema.html#update_config/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.DynamicSchema.update_config/2","doc":"* Updated DynamicSchema instance","ref":"Exdantic.Runtime.DynamicSchema.html#update_config/2-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.DynamicSchema.update_config/2","doc":"iex> updated = Exdantic.Runtime.DynamicSchema.update_config(schema, %{strict: true})\n    %Exdantic.Runtime.DynamicSchema{config: %{strict: true, ...}}","ref":"Exdantic.Runtime.DynamicSchema.html#update_config/2-examples"},{"type":"module","title":"Exdantic.Runtime.EnhancedSchema","doc":"Enhanced runtime schema with model validators and computed fields support.\n\nThis module extends DynamicSchema with support for:\n- Model validators with both named and anonymous functions\n- Computed fields with both named and anonymous functions\n- Full validation pipeline execution\n- JSON Schema generation with enhanced metadata","ref":"Exdantic.Runtime.EnhancedSchema.html"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.add_computed_field/4","doc":"Adds a computed field to an existing enhanced schema.","ref":"Exdantic.Runtime.EnhancedSchema.html#add_computed_field/4"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.add_computed_field/4","doc":"* `enhanced_schema` - An EnhancedSchema struct\n  * `field_name` - Name of the computed field\n  * `field_type` - Type specification for the field\n  * `computation` - Computation function or {module, function} tuple","ref":"Exdantic.Runtime.EnhancedSchema.html#add_computed_field/4-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.add_computed_field/4","doc":"* Updated EnhancedSchema struct","ref":"Exdantic.Runtime.EnhancedSchema.html#add_computed_field/4-returns"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.add_model_validator/2","doc":"Adds a model validator to an existing enhanced schema.","ref":"Exdantic.Runtime.EnhancedSchema.html#add_model_validator/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.add_model_validator/2","doc":"* `enhanced_schema` - An EnhancedSchema struct\n  * `validator` - Validator function or {module, function} tuple","ref":"Exdantic.Runtime.EnhancedSchema.html#add_model_validator/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.add_model_validator/2","doc":"* Updated EnhancedSchema struct","ref":"Exdantic.Runtime.EnhancedSchema.html#add_model_validator/2-returns"},{"type":"type","title":"Exdantic.Runtime.EnhancedSchema.computed_field_spec/0","doc":"","ref":"Exdantic.Runtime.EnhancedSchema.html#t:computed_field_spec/0"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.create/2","doc":"Creates an enhanced runtime schema with model validators and computed fields.","ref":"Exdantic.Runtime.EnhancedSchema.html#create/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.create/2","doc":"* `field_definitions` - List of field definitions\n  * `opts` - Enhanced schema options","ref":"Exdantic.Runtime.EnhancedSchema.html#create/2-parameters"},{"type":"function","title":"Options - Exdantic.Runtime.EnhancedSchema.create/2","doc":"* `:model_validators` - List of model validator functions or {module, function} tuples\n  * `:computed_fields` - List of computed field specifications\n  * `:title`, `:description`, `:strict` - Standard schema options\n  * `:name` - Schema name for references","ref":"Exdantic.Runtime.EnhancedSchema.html#create/2-options"},{"type":"function","title":"Examples - Exdantic.Runtime.EnhancedSchema.create/2","doc":"iex> fields = [{:name, :string, [required: true]}, {:age, :integer, [optional: true]}]\n    iex> validators = [fn data -> {:ok, %{data | name: String.trim(data.name)}} end]\n    iex> computed = [{:display_name, :string, fn data -> {:ok, String.upcase(data.name)} end}]\n    iex> schema = Exdantic.Runtime.EnhancedSchema.create(fields,\n    ...>   model_validators: validators,\n    ...>   computed_fields: computed\n    ...> )\n    %Exdantic.Runtime.EnhancedSchema{...}","ref":"Exdantic.Runtime.EnhancedSchema.html#create/2-examples"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.info/1","doc":"Returns information about the enhanced schema.","ref":"Exdantic.Runtime.EnhancedSchema.html#info/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.info/1","doc":"* `enhanced_schema` - An EnhancedSchema struct","ref":"Exdantic.Runtime.EnhancedSchema.html#info/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.info/1","doc":"* Map with enhanced schema information","ref":"Exdantic.Runtime.EnhancedSchema.html#info/1-returns"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.process_computed_fields/2","doc":"Processes computed field specifications and converts anonymous functions to named references.","ref":"Exdantic.Runtime.EnhancedSchema.html#process_computed_fields/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.process_computed_fields/2","doc":"* `computed_fields` - List of computed field specifications\n  * `initial_functions` - Map of existing runtime functions to extend","ref":"Exdantic.Runtime.EnhancedSchema.html#process_computed_fields/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.process_computed_fields/2","doc":"* Tuple of `{processed_fields, updated_functions}` where updated_functions\n    contains both initial and any new anonymous functions converted to named references","ref":"Exdantic.Runtime.EnhancedSchema.html#process_computed_fields/2-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.EnhancedSchema.process_computed_fields/2","doc":"iex> fields = [%{name: :full_name, function: fn x -> x.first <> \" \" <> x.last end}]\n    iex> {processed, functions} = process_computed_fields(fields, %{})\n    {[%{name: :full_name, function: {:runtime, :generated_name}}], %{generated_name: #Function<...>}}","ref":"Exdantic.Runtime.EnhancedSchema.html#process_computed_fields/2-examples"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.process_model_validators/1","doc":"Processes model validators and converts anonymous functions to named references.","ref":"Exdantic.Runtime.EnhancedSchema.html#process_model_validators/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.process_model_validators/1","doc":"* `validators` - List of validator specifications (module/function tuples or functions)","ref":"Exdantic.Runtime.EnhancedSchema.html#process_model_validators/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.process_model_validators/1","doc":"* Tuple of `{processed_validators, runtime_functions}` where runtime_functions\n    contains any anonymous functions converted to named references","ref":"Exdantic.Runtime.EnhancedSchema.html#process_model_validators/1-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.EnhancedSchema.process_model_validators/1","doc":"iex> validators = [{MyModule, :my_validator}, fn x -> x.valid end]\n    iex> {processed, functions} = process_model_validators(validators)\n    {[{MyModule, :my_validator}, {:runtime, :generated_name}], %{generated_name: #Function<...>}}","ref":"Exdantic.Runtime.EnhancedSchema.html#process_model_validators/1-examples"},{"type":"type","title":"Exdantic.Runtime.EnhancedSchema.t/0","doc":"","ref":"Exdantic.Runtime.EnhancedSchema.html#t:t/0"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.to_json_schema/2","doc":"Generates JSON Schema for an enhanced runtime schema.","ref":"Exdantic.Runtime.EnhancedSchema.html#to_json_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.to_json_schema/2","doc":"* `enhanced_schema` - An EnhancedSchema struct\n  * `opts` - JSON Schema generation options","ref":"Exdantic.Runtime.EnhancedSchema.html#to_json_schema/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.to_json_schema/2","doc":"* JSON Schema map including computed field metadata","ref":"Exdantic.Runtime.EnhancedSchema.html#to_json_schema/2-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.EnhancedSchema.to_json_schema/2","doc":"iex> json_schema = Exdantic.Runtime.EnhancedSchema.to_json_schema(schema)\n    %{\n      \"type\" => \"object\",\n      \"properties\" => %{\n        \"name\" => %{\"type\" => \"string\"},\n        \"display_name\" => %{\"type\" => \"string\", \"readOnly\" => true}\n      }\n    }","ref":"Exdantic.Runtime.EnhancedSchema.html#to_json_schema/2-examples"},{"type":"function","title":"Exdantic.Runtime.EnhancedSchema.validate/3","doc":"Validates data against an enhanced runtime schema.","ref":"Exdantic.Runtime.EnhancedSchema.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.EnhancedSchema.validate/3","doc":"* `data` - The data to validate (map)\n  * `enhanced_schema` - An EnhancedSchema struct\n  * `opts` - Validation options","ref":"Exdantic.Runtime.EnhancedSchema.html#validate/3-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.EnhancedSchema.validate/3","doc":"* `{:ok, validated_data}` on success (includes computed fields)\n  * `{:error, errors}` on validation failure","ref":"Exdantic.Runtime.EnhancedSchema.html#validate/3-returns"},{"type":"function","title":"Examples - Exdantic.Runtime.EnhancedSchema.validate/3","doc":"iex> data = %{name: \"  John  \", age: 30}\n    iex> Exdantic.Runtime.EnhancedSchema.validate(data, schema)\n    {:ok, %{name: \"John\", age: 30, display_name: \"JOHN\"}}","ref":"Exdantic.Runtime.EnhancedSchema.html#validate/3-examples"},{"type":"type","title":"Exdantic.Runtime.EnhancedSchema.validator_spec/0","doc":"","ref":"Exdantic.Runtime.EnhancedSchema.html#t:validator_spec/0"},{"type":"module","title":"Exdantic.Runtime.Validator","doc":"Validation functions specifically for runtime schemas.\n\nThis module provides validation logic that works with both DynamicSchema\nand EnhancedSchema, handling the full validation pipeline including\nmodel validators and computed fields.","ref":"Exdantic.Runtime.Validator.html"},{"type":"function","title":"Exdantic.Runtime.Validator.enhance_schema/2","doc":"Converts a DynamicSchema to an EnhancedSchema.","ref":"Exdantic.Runtime.Validator.html#enhance_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.Validator.enhance_schema/2","doc":"* `dynamic_schema` - DynamicSchema to convert\n  * `opts` - Options for enhancement","ref":"Exdantic.Runtime.Validator.html#enhance_schema/2-parameters"},{"type":"function","title":"Options - Exdantic.Runtime.Validator.enhance_schema/2","doc":"* `:model_validators` - Model validators to add\n  * `:computed_fields` - Computed fields to add","ref":"Exdantic.Runtime.Validator.html#enhance_schema/2-options"},{"type":"function","title":"Returns - Exdantic.Runtime.Validator.enhance_schema/2","doc":"* EnhancedSchema struct","ref":"Exdantic.Runtime.Validator.html#enhance_schema/2-returns"},{"type":"function","title":"Exdantic.Runtime.Validator.enhanced_schema?/1","doc":"Checks if a schema supports enhanced features.","ref":"Exdantic.Runtime.Validator.html#enhanced_schema?/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.Validator.enhanced_schema?/1","doc":"* `schema` - DynamicSchema or EnhancedSchema struct","ref":"Exdantic.Runtime.Validator.html#enhanced_schema?/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.Validator.enhanced_schema?/1","doc":"* `true` if schema supports enhanced features, `false` otherwise","ref":"Exdantic.Runtime.Validator.html#enhanced_schema?/1-returns"},{"type":"function","title":"Exdantic.Runtime.Validator.schema_info/1","doc":"Returns information about a runtime schema.","ref":"Exdantic.Runtime.Validator.html#schema_info/1"},{"type":"function","title":"Parameters - Exdantic.Runtime.Validator.schema_info/1","doc":"* `schema` - DynamicSchema or EnhancedSchema struct","ref":"Exdantic.Runtime.Validator.html#schema_info/1-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.Validator.schema_info/1","doc":"* Map with schema information","ref":"Exdantic.Runtime.Validator.html#schema_info/1-returns"},{"type":"function","title":"Exdantic.Runtime.Validator.to_json_schema/2","doc":"Generates JSON Schema for a runtime schema.","ref":"Exdantic.Runtime.Validator.html#to_json_schema/2"},{"type":"function","title":"Parameters - Exdantic.Runtime.Validator.to_json_schema/2","doc":"* `schema` - DynamicSchema or EnhancedSchema struct\n  * `opts` - JSON Schema generation options","ref":"Exdantic.Runtime.Validator.html#to_json_schema/2-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.Validator.to_json_schema/2","doc":"* JSON Schema map","ref":"Exdantic.Runtime.Validator.html#to_json_schema/2-returns"},{"type":"function","title":"Exdantic.Runtime.Validator.validate/3","doc":"Validates data against a runtime schema (DynamicSchema or EnhancedSchema).","ref":"Exdantic.Runtime.Validator.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.Validator.validate/3","doc":"* `data` - The data to validate\n  * `schema` - DynamicSchema or EnhancedSchema struct\n  * `opts` - Validation options","ref":"Exdantic.Runtime.Validator.html#validate/3-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.Validator.validate/3","doc":"* `{:ok, validated_data}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.Runtime.Validator.html#validate/3-returns"},{"type":"function","title":"Exdantic.Runtime.Validator.validate!/3","doc":"Validates data against a runtime schema, raising on failure.","ref":"Exdantic.Runtime.Validator.html#validate!/3"},{"type":"function","title":"Parameters - Exdantic.Runtime.Validator.validate!/3","doc":"* `data` - The data to validate\n  * `schema` - DynamicSchema or EnhancedSchema struct\n  * `opts` - Validation options","ref":"Exdantic.Runtime.Validator.html#validate!/3-parameters"},{"type":"function","title":"Returns - Exdantic.Runtime.Validator.validate!/3","doc":"* Validated data on success\n  * Raises `Exdantic.ValidationError` on failure","ref":"Exdantic.Runtime.Validator.html#validate!/3-returns"},{"type":"module","title":"Exdantic.Schema","doc":"Schema DSL for defining data schemas with validation rules and metadata.\n\nThis module provides macros and functions for defining structured data schemas\nwith rich validation capabilities, type safety, and comprehensive error reporting.","ref":"Exdantic.Schema.html"},{"type":"module","title":"Phase 4 Enhancement: Anonymous Function Support - Exdantic.Schema","doc":"Added support for inline anonymous functions in model validators and computed fields:\n\n    schema do\n      field :password, :string\n      field :password_confirmation, :string\n\n      # Named function (existing)\n      model_validator :validate_passwords_match\n\n      # Anonymous function (new)\n      model_validator fn input ->\n        if input.password == input.password_confirmation do\n          {:ok, input}\n        else\n          {:error, \"passwords do not match\"}\n        end\n      end\n\n      # Anonymous function with do-end block (new)\n      model_validator do\n        if input.password == input.password_confirmation do\n          {:ok, input}\n        else\n          {:error, \"passwords do not match\"}\n        end\n      end\n\n      computed_field :display_name, :string do\n        String.upcase(input.name)\n      end\n    end","ref":"Exdantic.Schema.html#module-phase-4-enhancement-anonymous-function-support"},{"type":"macro","title":"Exdantic.Schema.choices/1","doc":"Adds an enumeration constraint, limiting values to a predefined set.","ref":"Exdantic.Schema.html#choices/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.choices/1","doc":"* `values` - List of allowed values","ref":"Exdantic.Schema.html#choices/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.choices/1","doc":"field :status, :string do\n      choices([\"pending\", \"active\", \"completed\"])\n    end\n\n    field :priority, :integer do\n      choices([1, 2, 3])\n    end\n\n    field :size, :string do\n      choices([\"small\", \"medium\", \"large\"])\n    end","ref":"Exdantic.Schema.html#choices/1-examples"},{"type":"macro","title":"Exdantic.Schema.computed_field/3","doc":"Defines a computed field that generates a value based on validated data.\n\nComputed fields execute after field and model validation, generating additional\ndata that becomes part of the final validated result. They are particularly\nuseful for derived values, formatted representations, or aggregated data.","ref":"Exdantic.Schema.html#computed_field/3"},{"type":"macro","title":"Parameters - Exdantic.Schema.computed_field/3","doc":"* `name` - Field name (atom)\n  * `type` - Field type specification (same as regular fields)\n  * `function_name` - Name of the function to call for computation (atom) or anonymous function\n  * `opts` - Optional keyword list with :description and :example (when using named function)","ref":"Exdantic.Schema.html#computed_field/3-parameters"},{"type":"macro","title":"Function Signature - Exdantic.Schema.computed_field/3","doc":"The computation function must accept one parameter (the validated data) and return:\n  * `{:ok, computed_value}` - computation succeeds\n  * `{:error, message}` - computation fails with error message\n  * `{:error, %Exdantic.Error{}}` - computation fails with detailed error","ref":"Exdantic.Schema.html#computed_field/3-function-signature"},{"type":"macro","title":"Execution Order - Exdantic.Schema.computed_field/3","doc":"Computed fields execute after:\n1. Field validation\n2. Model validation\n\nThis ensures computed fields have access to fully validated and transformed data.","ref":"Exdantic.Schema.html#computed_field/3-execution-order"},{"type":"macro","title":"Examples - Exdantic.Schema.computed_field/3","doc":"# Using named function\n    defmodule UserSchema do\n      use Exdantic, define_struct: true\n\n      schema do\n        field :first_name, :string, required: true\n        field :last_name, :string, required: true\n        field :email, :string, required: true\n\n        computed_field :full_name, :string, :generate_full_name\n        computed_field :email_domain, :string, :extract_email_domain,\n          description: \"Domain part of the email address\",\n          example: \"example.com\"\n      end\n\n      def generate_full_name(input) do\n        {:ok, \"#{input.first_name} #{input.last_name}\"}\n      end\n\n      def extract_email_domain(input) do\n        domain = input.email |> String.split(\"@\") |> List.last()\n        {:ok, domain}\n      end\n    end\n\n    # Using anonymous function\n    schema do\n      field :first_name, :string\n      field :last_name, :string\n\n      computed_field :full_name, :string, fn input ->\n        {:ok, \"#{input.first_name} #{input.last_name}\"}\n      end\n\n      computed_field :initials, :string, fn input ->\n        first = String.first(input.first_name)\n        last = String.first(input.last_name)\n        {:ok, \"#{first}#{last}\"}\n      end\n    end","ref":"Exdantic.Schema.html#computed_field/3-examples"},{"type":"macro","title":"Error Handling - Exdantic.Schema.computed_field/3","doc":"Computed field functions can return errors that will be included in validation results:\n\n    def risky_computation(data) do\n      if valid_computation?(data) do\n        {:ok, compute_value(data)}\n      else\n        {:error, \"Computation failed due to invalid data\"}\n      end\n    end","ref":"Exdantic.Schema.html#computed_field/3-error-handling"},{"type":"macro","title":"Type Safety - Exdantic.Schema.computed_field/3","doc":"Computed field return values are validated against their declared types:\n\n    computed_field :score, :integer, :calculate_score\n\n    def calculate_score(data) do\n      # This will fail validation if score is not an integer\n      {:ok, \"not an integer\"}\n    end","ref":"Exdantic.Schema.html#computed_field/3-type-safety"},{"type":"macro","title":"JSON Schema Integration - Exdantic.Schema.computed_field/3","doc":"Computed fields are automatically included in generated JSON schemas and marked as `readOnly`:\n\n    %{\n      \"type\" => \"object\",\n      \"properties\" => %{\n        \"first_name\" => %{\"type\" => \"string\"},\n        \"full_name\" => %{\"type\" => \"string\", \"readOnly\" => true}\n      }\n    }","ref":"Exdantic.Schema.html#computed_field/3-json-schema-integration"},{"type":"macro","title":"With Struct Definition - Exdantic.Schema.computed_field/3","doc":"When using `define_struct: true`, computed fields are included in the struct definition:\n\n    defstruct [:first_name, :last_name, :email, :full_name, :email_domain]","ref":"Exdantic.Schema.html#computed_field/3-with-struct-definition"},{"type":"macro","title":"Exdantic.Schema.computed_field/4","doc":"","ref":"Exdantic.Schema.html#computed_field/4"},{"type":"macro","title":"Exdantic.Schema.config/1","doc":"Defines configuration settings for the schema.\n\nConfiguration options can include:\n  * title - Schema title\n  * description - Schema description\n  * strict - Whether to enforce strict validation","ref":"Exdantic.Schema.html#config/1"},{"type":"macro","title":"Examples - Exdantic.Schema.config/1","doc":"config do\n      title(\"User Schema\")\n      config_description(\"Validates user registration data\")\n      strict(true)\n    end\n\n    config do\n      strict(false)\n    end","ref":"Exdantic.Schema.html#config/1-examples"},{"type":"macro","title":"Exdantic.Schema.config_description/1","doc":"Sets the description for the schema configuration.","ref":"Exdantic.Schema.html#config_description/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.config_description/1","doc":"* `text` - String description of the schema","ref":"Exdantic.Schema.html#config_description/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.config_description/1","doc":"config do\n      config_description(\"Validates user data for registration\")\n    end\n\n    config do\n      title(\"User Schema\")\n      config_description(\"Comprehensive user validation with email format checking\")\n    end","ref":"Exdantic.Schema.html#config_description/1-examples"},{"type":"macro","title":"Exdantic.Schema.default/1","doc":"Sets a default value for the field and marks it as optional.\nThe default value will be used if the field is omitted from input data.","ref":"Exdantic.Schema.html#default/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.default/1","doc":"* `value` - The default value to use when the field is not provided","ref":"Exdantic.Schema.html#default/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.default/1","doc":"field :status, :string do\n      default(\"pending\")\n    end\n\n    field :active, :boolean do\n      default(true)\n    end\n\n    field :retry_count, :integer do\n      default(0)\n      gteq(0)\n    end","ref":"Exdantic.Schema.html#default/1-examples"},{"type":"macro","title":"Exdantic.Schema.description/1","doc":"Sets a description for the field.","ref":"Exdantic.Schema.html#description/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.description/1","doc":"* `text` - String description of the field's purpose or usage","ref":"Exdantic.Schema.html#description/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.description/1","doc":"field :age, :integer do\n      description(\"User's age in years\")\n    end\n\n    field :email, :string do\n      description(\"Primary contact email address\")\n      format(~r/@/)\n    end","ref":"Exdantic.Schema.html#description/1-examples"},{"type":"macro","title":"Exdantic.Schema.example/1","doc":"Sets a single example value for the field.","ref":"Exdantic.Schema.html#example/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.example/1","doc":"* `value` - An example value that would be valid for this field","ref":"Exdantic.Schema.html#example/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.example/1","doc":"field :age, :integer do\n      example(25)\n    end\n\n    field :name, :string do\n      example(\"John Doe\")\n    end","ref":"Exdantic.Schema.html#example/1-examples"},{"type":"macro","title":"Exdantic.Schema.examples/1","doc":"Sets multiple example values for the field.","ref":"Exdantic.Schema.html#examples/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.examples/1","doc":"* `values` - List of example values that would be valid for this field","ref":"Exdantic.Schema.html#examples/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.examples/1","doc":"field :status, :string do\n      examples([\"pending\", \"active\", \"completed\"])\n    end\n\n    field :score, :integer do\n      examples([85, 92, 78])\n    end","ref":"Exdantic.Schema.html#examples/1-examples"},{"type":"macro","title":"Exdantic.Schema.extra/2","doc":"Sets arbitrary extra metadata for the field.\n\nThis allows storing custom key-value pairs in the field metadata,\nwhich is particularly useful for DSPy-style field type annotations\nand other framework-specific metadata.","ref":"Exdantic.Schema.html#extra/2"},{"type":"macro","title":"Parameters - Exdantic.Schema.extra/2","doc":"* `key` - String key for the metadata\n  * `value` - The metadata value","ref":"Exdantic.Schema.html#extra/2-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.extra/2","doc":"field :answer, :string do\n      extra(\"__dspy_field_type\", \"output\")\n      extra(\"prefix\", \"Answer:\")\n    end\n\n    field :question, :string do\n      extra(\"__dspy_field_type\", \"input\")\n    end\n\n    # Can also be used with map\n    field :data, :string, extra: %{\"custom_key\" => \"custom_value\"}","ref":"Exdantic.Schema.html#extra/2-examples"},{"type":"macro","title":"Exdantic.Schema.field/3","doc":"Defines a field in the schema with a name, type, and optional constraints.","ref":"Exdantic.Schema.html#field/3"},{"type":"macro","title":"Parameters - Exdantic.Schema.field/3","doc":"* `name` - Atom representing the field name\n  * `type` - The field's type, which can be:\n    * A built-in type (`:string`, `:integer`, `:float`, `:boolean`, `:any`)\n    * An array type (`{:array, type}`)\n    * A map type (`{:map, {key_type, value_type}}`)\n    * A union type (`{:union, [type1, type2, ...]}`)\n    * A reference to another schema (atom)\n  * `opts` - Optional block containing field constraints and metadata","ref":"Exdantic.Schema.html#field/3-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.field/3","doc":"# Simple field\n    field :name, :string\n\n    # Field with constraints\n    field :age, :integer do\n      description(\"User's age in years\")\n      gt(0)\n      lt(150)\n    end\n\n    # Array field\n    field :tags, {:array, :string} do\n      min_items(1)\n      max_items(10)\n    end\n\n    # Map field\n    field :metadata, {:map, {:string, :any}}\n\n    # Reference to another schema\n    field :address, Address\n\n    # Optional field with default\n    field :active, :boolean do\n      default(true)\n    end","ref":"Exdantic.Schema.html#field/3-examples"},{"type":"macro","title":"Exdantic.Schema.format/1","doc":"Adds a format constraint to a string field.","ref":"Exdantic.Schema.html#format/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.format/1","doc":"* `value` - The format pattern (regular expression)","ref":"Exdantic.Schema.html#format/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.format/1","doc":"field :email, :string do\n      format(~r/^[^ @]+@[^ @]+.[^ @]+$/)\n    end\n\n    field :phone, :string do\n      format(~r/^+?[1-9]{1,14}$/)\n    end","ref":"Exdantic.Schema.html#format/1-examples"},{"type":"macro","title":"Exdantic.Schema.gt/1","doc":"Adds a greater than constraint to a numeric field.","ref":"Exdantic.Schema.html#gt/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.gt/1","doc":"* `value` - The minimum value (exclusive)","ref":"Exdantic.Schema.html#gt/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.gt/1","doc":"field :age, :integer do\n      gt(0)\n    end\n\n    field :score, :float do\n      gt(0.0)\n      lt(100.0)\n    end","ref":"Exdantic.Schema.html#gt/1-examples"},{"type":"macro","title":"Exdantic.Schema.gteq/1","doc":"Adds a greater than or equal to constraint to a numeric field.","ref":"Exdantic.Schema.html#gteq/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.gteq/1","doc":"* `value` - The minimum value (inclusive)","ref":"Exdantic.Schema.html#gteq/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.gteq/1","doc":"field :age, :integer do\n      gteq(18)\n    end\n\n    field :rating, :float do\n      gteq(0.0)\n      lteq(5.0)\n    end","ref":"Exdantic.Schema.html#gteq/1-examples"},{"type":"macro","title":"Exdantic.Schema.lt/1","doc":"Adds a less than constraint to a numeric field.","ref":"Exdantic.Schema.html#lt/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.lt/1","doc":"* `value` - The maximum value (exclusive)","ref":"Exdantic.Schema.html#lt/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.lt/1","doc":"field :age, :integer do\n      lt(100)\n    end\n\n    field :temperature, :float do\n      gt(-50.0)\n      lt(100.0)\n    end","ref":"Exdantic.Schema.html#lt/1-examples"},{"type":"macro","title":"Exdantic.Schema.lteq/1","doc":"Adds a less than or equal to constraint to a numeric field.","ref":"Exdantic.Schema.html#lteq/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.lteq/1","doc":"* `value` - The maximum value (inclusive)","ref":"Exdantic.Schema.html#lteq/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.lteq/1","doc":"field :rating, :float do\n      lteq(5.0)\n    end\n\n    field :percentage, :integer do\n      gteq(0)\n      lteq(100)\n    end","ref":"Exdantic.Schema.html#lteq/1-examples"},{"type":"type","title":"Exdantic.Schema.macro_ast/0","doc":"","ref":"Exdantic.Schema.html#t:macro_ast/0"},{"type":"macro","title":"Exdantic.Schema.max_items/1","doc":"Adds a maximum items constraint to an array field.","ref":"Exdantic.Schema.html#max_items/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.max_items/1","doc":"* `value` - The maximum number of items allowed (must be a non-negative integer)","ref":"Exdantic.Schema.html#max_items/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.max_items/1","doc":"field :tags, {:array, :string} do\n      max_items(10)\n    end\n\n    field :favorites, {:array, :integer} do\n      min_items(1)\n      max_items(3)\n    end","ref":"Exdantic.Schema.html#max_items/1-examples"},{"type":"macro","title":"Exdantic.Schema.max_length/1","doc":"Adds a maximum length constraint to a string field.","ref":"Exdantic.Schema.html#max_length/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.max_length/1","doc":"* `value` - The maximum length allowed (must be a non-negative integer)","ref":"Exdantic.Schema.html#max_length/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.max_length/1","doc":"field :username, :string do\n      max_length(20)\n    end\n\n    field :description, :string do\n      max_length(500)\n    end","ref":"Exdantic.Schema.html#max_length/1-examples"},{"type":"macro","title":"Exdantic.Schema.min_items/1","doc":"Adds a minimum items constraint to an array field.","ref":"Exdantic.Schema.html#min_items/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.min_items/1","doc":"* `value` - The minimum number of items required (must be a non-negative integer)","ref":"Exdantic.Schema.html#min_items/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.min_items/1","doc":"field :tags, {:array, :string} do\n      min_items(1)\n    end\n\n    field :categories, {:array, :string} do\n      min_items(2)\n      max_items(5)\n    end","ref":"Exdantic.Schema.html#min_items/1-examples"},{"type":"macro","title":"Exdantic.Schema.min_length/1","doc":"Adds a minimum length constraint to a string field.","ref":"Exdantic.Schema.html#min_length/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.min_length/1","doc":"* `value` - The minimum length required (must be a non-negative integer)","ref":"Exdantic.Schema.html#min_length/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.min_length/1","doc":"field :username, :string do\n      min_length(3)\n    end\n\n    field :password, :string do\n      min_length(8)\n      max_length(100)\n    end","ref":"Exdantic.Schema.html#min_length/1-examples"},{"type":"macro","title":"Exdantic.Schema.model_validator/1","doc":"Defines a model-level validator that runs after field validation.\n\nModel validators receive the validated data (as a map or struct) and can perform\ncross-field validation, data transformation, or complex business logic validation.","ref":"Exdantic.Schema.html#model_validator/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.model_validator/1","doc":"* `function_name` - Name of the function to call for model validation (when using named function)\n  * `validator_fn` - Anonymous function that accepts validated data and returns result (when using anonymous function)\n  * `do` block - Block of code with implicit `input` variable (when using do-end block)","ref":"Exdantic.Schema.html#model_validator/1-parameters"},{"type":"macro","title":"Function Signature - Exdantic.Schema.model_validator/1","doc":"The validator must accept one parameter (the validated data) and return:\n  * `{:ok, data}` - validation succeeds, optionally with transformed data\n  * `{:error, message}` - validation fails with error message\n  * `{:error, %Exdantic.Error{}}` - validation fails with detailed error","ref":"Exdantic.Schema.html#model_validator/1-function-signature"},{"type":"macro","title":"Examples - Exdantic.Schema.model_validator/1","doc":"defmodule UserSchema do\n      use Exdantic, define_struct: true\n\n      schema do\n        field :password, :string, required: true\n        field :password_confirmation, :string, required: true\n\n        # Using named function\n        model_validator :validate_passwords_match\n\n        # Using anonymous function\n        model_validator fn input ->\n          if input.password == input.password_confirmation do\n            {:ok, input}\n          else\n            {:error, \"passwords do not match\"}\n          end\n        end\n\n        # Using do-end block with implicit input\n        model_validator do\n          if input.password == input.password_confirmation do\n            {:ok, input}\n          else\n            {:error, \"passwords do not match\"}\n          end\n        end\n      end\n\n      def validate_passwords_match(input) do\n        if input.password == input.password_confirmation do\n          {:ok, input}\n        else\n          {:error, \"passwords do not match\"}\n        end\n      end\n    end","ref":"Exdantic.Schema.html#model_validator/1-examples"},{"type":"macro","title":"Multiple Validators - Exdantic.Schema.model_validator/1","doc":"Multiple model validators can be defined and will execute in the order they are declared:\n\n    schema do\n      field :username, :string, required: true\n      field :email, :string, required: true\n\n      model_validator :validate_username_unique\n      model_validator :validate_email_format\n      model_validator :send_welcome_email\n    end","ref":"Exdantic.Schema.html#model_validator/1-multiple-validators"},{"type":"macro","title":"Data Transformation - Exdantic.Schema.model_validator/1","doc":"Model validators can transform the data by returning modified data:\n\n    def normalize_email(input) do\n      normalized = %{input | email: String.downcase(input.email)}\n      {:ok, normalized}\n    end","ref":"Exdantic.Schema.html#model_validator/1-data-transformation"},{"type":"type","title":"Exdantic.Schema.model_validator_ast/0","doc":"","ref":"Exdantic.Schema.html#t:model_validator_ast/0"},{"type":"macro","title":"Exdantic.Schema.optional/0","doc":"Marks the field as optional.\nAn optional field may be omitted from the input data during validation.","ref":"Exdantic.Schema.html#optional/0"},{"type":"macro","title":"Examples - Exdantic.Schema.optional/0","doc":"field :middle_name, :string do\n      optional()\n    end\n\n    field :bio, :string do\n      optional()\n      max_length(500)\n    end","ref":"Exdantic.Schema.html#optional/0-examples"},{"type":"macro","title":"Exdantic.Schema.required/0","doc":"Marks the field as required (this is the default behavior).\nA required field must be present in the input data during validation.","ref":"Exdantic.Schema.html#required/0"},{"type":"macro","title":"Examples - Exdantic.Schema.required/0","doc":"field :email, :string do\n      required()\n      format(~r/@/)\n    end\n\n    field :name, :string do\n      required()\n      min_length(1)\n    end","ref":"Exdantic.Schema.html#required/0-examples"},{"type":"macro","title":"Exdantic.Schema.schema/2","doc":"Defines a new schema with optional description.","ref":"Exdantic.Schema.html#schema/2"},{"type":"macro","title":"Parameters - Exdantic.Schema.schema/2","doc":"* `description` - Optional string describing the schema's purpose\n  * `do` - Block containing field definitions and configuration","ref":"Exdantic.Schema.html#schema/2-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.schema/2","doc":"schema \"User registration data\" do\n      field :name, :string do\n        required()\n        min_length(2)\n      end\n\n      field :age, :integer do\n        optional()\n        gt(0)\n      end\n    end\n\n    schema do\n      field :email, :string\n      field :active, :boolean, default: true\n    end","ref":"Exdantic.Schema.html#schema/2-examples"},{"type":"type","title":"Exdantic.Schema.schema_config/0","doc":"","ref":"Exdantic.Schema.html#t:schema_config/0"},{"type":"macro","title":"Exdantic.Schema.strict/1","doc":"Sets whether the schema should enforce strict validation.\nWhen strict is true, unknown fields will cause validation to fail.","ref":"Exdantic.Schema.html#strict/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.strict/1","doc":"* `bool` - Boolean indicating if strict validation should be enabled","ref":"Exdantic.Schema.html#strict/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.strict/1","doc":"config do\n      strict(true)\n    end\n\n    config do\n      title(\"Flexible Schema\")\n      strict(false)\n    end","ref":"Exdantic.Schema.html#strict/1-examples"},{"type":"macro","title":"Exdantic.Schema.title/1","doc":"Sets the title for the schema configuration.","ref":"Exdantic.Schema.html#title/1"},{"type":"macro","title":"Parameters - Exdantic.Schema.title/1","doc":"* `text` - String title for the schema","ref":"Exdantic.Schema.html#title/1-parameters"},{"type":"macro","title":"Examples - Exdantic.Schema.title/1","doc":"config do\n      title(\"User Schema\")\n    end\n\n    config do\n      title(\"Product Validation Schema\")\n      strict(true)\n    end","ref":"Exdantic.Schema.html#title/1-examples"},{"type":"module","title":"Exdantic.StructValidator","doc":"Validator that optionally returns struct instances and executes model validators.\n\nThis module extends the existing validation logic to support:\n1. Returning struct instances when a schema is defined with `define_struct: true`\n2. Executing model validators after field validation succeeds\n3. Computing derived fields after model validation succeeds\n\nThe validation pipeline:\n1. Field validation (existing logic)\n2. Model validation (Phase 2)\n3. Computed field execution (Phase 3)\n4. Struct creation (Phase 1)","ref":"Exdantic.StructValidator.html"},{"type":"module","title":"Phase 4 Enhancement: Anonymous Function Support - Exdantic.StructValidator","doc":"Enhanced to properly handle both named functions and generated anonymous functions\nin model validators and computed fields. The validator can now execute:\n\n1. Named function model validators: `{MyModule, :validate_something}`\n2. Generated anonymous function validators: `{MyModule, :__generated_model_validator_123_456}`\n3. Named function computed fields: `{field_name, %ComputedFieldMeta{function_name: :my_function}}`\n4. Generated anonymous computed fields: `{field_name, %ComputedFieldMeta{function_name: :__generated_computed_field_name_123_456}}`\n\nAll function types are handled uniformly through the same execution pipeline.","ref":"Exdantic.StructValidator.html#module-phase-4-enhancement-anonymous-function-support"},{"type":"function","title":"Exdantic.StructValidator.validate_schema/3","doc":"Validates data against a schema with full pipeline support including computed fields.","ref":"Exdantic.StructValidator.html#validate_schema/3"},{"type":"function","title":"Enhanced Validation Pipeline - Exdantic.StructValidator.validate_schema/3","doc":"1. **Field Validation**: Validates individual fields using existing logic\n2. **Model Validation**: Executes model validators in sequence (Phase 2)\n3. **Computed Field Execution**: Executes computed fields to derive additional data (Phase 3)\n4. **Struct Creation**: Optionally creates struct instance (Phase 1)","ref":"Exdantic.StructValidator.html#validate_schema/3-enhanced-validation-pipeline"},{"type":"function","title":"Parameters - Exdantic.StructValidator.validate_schema/3","doc":"* `schema_module` - The schema module to validate against\n  * `data` - The data to validate (must be a map)\n  * `path` - Current validation path for error reporting (defaults to `[]`)","ref":"Exdantic.StructValidator.html#validate_schema/3-parameters"},{"type":"function","title":"Returns - Exdantic.StructValidator.validate_schema/3","doc":"* `{:ok, validated_data}` - where `validated_data` includes computed fields and is a struct if the schema\n    defines one, otherwise a map\n  * `{:error, errors}` - list of validation errors","ref":"Exdantic.StructValidator.html#validate_schema/3-returns"},{"type":"function","title":"Model Validator Execution - Exdantic.StructValidator.validate_schema/3","doc":"Model validators are executed in the order they are declared in the schema.\nNote: The validators are stored in reverse order due to Elixir's accumulate attribute behavior,\nbut they are reversed back during execution to maintain declaration order.\nIf any model validator returns an error, execution stops and the error is returned.\nModel validators can transform data by returning modified data in the success case.","ref":"Exdantic.StructValidator.html#validate_schema/3-model-validator-execution"},{"type":"function","title":"Computed Field Execution - Exdantic.StructValidator.validate_schema/3","doc":"Computed fields are executed after model validation succeeds. Each computed field function:\n- Receives the validated data (including any transformations from model validators)\n- Must return `{:ok, computed_value}` or `{:error, reason}`\n- Has its return value validated against the declared field type\n- Contributes to the final validated result","ref":"Exdantic.StructValidator.html#validate_schema/3-computed-field-execution"},{"type":"function","title":"Examples - Exdantic.StructValidator.validate_schema/3","doc":"# Basic model validation with computed fields\n    defmodule UserSchema do\n      use Exdantic, define_struct: true\n\n      schema do\n        field :first_name, :string, required: true\n        field :last_name, :string, required: true\n        field :email, :string, required: true\n\n        model_validator :normalize_names\n        computed_field :full_name, :string, :generate_full_name\n        computed_field :email_domain, :string, :extract_email_domain\n      end\n\n      def normalize_names(validated_data) do\n        normalized = %{\n          validated_data |\n          first_name: String.trim(validated_data.first_name),\n          last_name: String.trim(validated_data.last_name)\n        }\n        {:ok, normalized}\n      end\n\n      def generate_full_name(validated_data) do\n        {:ok, \"#{validated_data.first_name} #{validated_data.last_name}\"}\n      end\n\n      def extract_email_domain(validated_data) do\n        {:ok, validated_data.email |> String.split(\"@\") |> List.last()}\n      end\n    end\n\n    iex> UserSchema.validate(%{\n    ...>   first_name: \"  John  \",\n    ...>   last_name: \"  Doe  \",\n    ...>   email: \"john@example.com\"\n    ...> })\n    {:ok, %UserSchema{\n      first_name: \"John\",      # normalized by model validator\n      last_name: \"Doe\",       # normalized by model validator\n      email: \"john@example.com\",\n      full_name: \"John Doe\",   # computed field\n      email_domain: \"example.com\"  # computed field\n    }}","ref":"Exdantic.StructValidator.html#validate_schema/3-examples"},{"type":"function","title":"Error Handling - Exdantic.StructValidator.validate_schema/3","doc":"Model validators can return errors in several formats:\n- `{:error, \"string message\"}` - converted to validation error\n- `{:error, %Exdantic.Error{}}` - used directly\n- Exception during execution - caught and converted to validation error\n\nComputed field errors are handled gracefully:\n- Function execution errors are caught and converted to validation errors\n- Type validation errors for computed values are reported with field context\n- Computed field errors include the field path and computation function reference","ref":"Exdantic.StructValidator.html#validate_schema/3-error-handling"},{"type":"behaviour","title":"Exdantic.Type","doc":"Behaviour and macros for defining custom types.\n\nThis module provides the behaviour and utility functions for creating\ncustom types in Exdantic schemas with validation and coercion capabilities.","ref":"Exdantic.Type.html"},{"type":"macro","title":"Exdantic.Type.__using__/1","doc":"Provides functionality for defining custom types in Exdantic schemas.\n\nWhen you `use Exdantic.Type`, your module gets:\n- The `Exdantic.Type` behaviour\n- Import of `Exdantic.Types` functions\n- Type aliases for coercion functions\n- A default implementation of validation with coercion support","ref":"Exdantic.Type.html#__using__/1"},{"type":"macro","title":"Examples - Exdantic.Type.__using__/1","doc":"defmodule MyApp.Types.Email do\n      use Exdantic.Type\n\n      def type_definition do\n        {:type, :string, [format: ~r/^[^@]+@[^@]+.[^@]+$/]}\n      end\n\n      def json_schema do\n        %{\"type\" => \"string\", \"format\" => \"email\"}\n      end\n\n      def validate(value) do\n        case type_definition() |> Exdantic.Validator.validate(value) do\n          {:ok, validated} -> {:ok, validated}\n          {:error, _} -> {:error, \"invalid email format\"}\n        end\n      end\n    end","ref":"Exdantic.Type.html#__using__/1-examples"},{"type":"type","title":"Exdantic.Type.coerce_function/0","doc":"","ref":"Exdantic.Type.html#t:coerce_function/0"},{"type":"callback","title":"Exdantic.Type.coerce_rule/0","doc":"","ref":"Exdantic.Type.html#c:coerce_rule/0"},{"type":"type","title":"Exdantic.Type.coerce_rule/0","doc":"","ref":"Exdantic.Type.html#t:coerce_rule/0"},{"type":"callback","title":"Exdantic.Type.custom_rules/0","doc":"","ref":"Exdantic.Type.html#c:custom_rules/0"},{"type":"callback","title":"Exdantic.Type.json_schema/0","doc":"","ref":"Exdantic.Type.html#c:json_schema/0"},{"type":"callback","title":"Exdantic.Type.type_definition/0","doc":"","ref":"Exdantic.Type.html#c:type_definition/0"},{"type":"callback","title":"Exdantic.Type.validate/1","doc":"","ref":"Exdantic.Type.html#c:validate/1"},{"type":"module","title":"Exdantic.TypeAdapter","doc":"Runtime type validation and serialization without a schema.\n\nThis module provides the equivalent of Pydantic's TypeAdapter functionality,\nallowing validation and serialization of values against type specifications\nwithout requiring a full schema definition.\n\nSupports the DSPy pattern: `TypeAdapter(type(value)).validate_python(value)`","ref":"Exdantic.TypeAdapter.html"},{"type":"function","title":"Exdantic.TypeAdapter.create/2","doc":"Creates a TypeAdapter instance for reuse with the same type specification.","ref":"Exdantic.TypeAdapter.html#create/2"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.create/2","doc":"* `type_spec` - The type specification to create an adapter for\n  * `opts` - Configuration options for the adapter","ref":"Exdantic.TypeAdapter.html#create/2-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.create/2","doc":"* TypeAdapter struct","ref":"Exdantic.TypeAdapter.html#create/2-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.create/2","doc":"iex> adapter = Exdantic.TypeAdapter.create({:array, :string})\n    %Exdantic.TypeAdapter.Instance{...}\n\n    iex> Exdantic.TypeAdapter.Instance.validate(adapter, [\"a\", \"b\", \"c\"])\n    {:ok, [\"a\", \"b\", \"c\"]}","ref":"Exdantic.TypeAdapter.html#create/2-examples"},{"type":"function","title":"Exdantic.TypeAdapter.dump/3","doc":"Serializes a value according to a type specification.","ref":"Exdantic.TypeAdapter.html#dump/3"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.dump/3","doc":"* `type_spec` - The type specification to serialize according to\n  * `value` - The value to serialize\n  * `opts` - Serialization options","ref":"Exdantic.TypeAdapter.html#dump/3-parameters"},{"type":"function","title":"Options - Exdantic.TypeAdapter.dump/3","doc":"* `:exclude_none` - Exclude nil values (default: false)\n  * `:exclude_defaults` - Exclude default values (default: false)","ref":"Exdantic.TypeAdapter.html#dump/3-options"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.dump/3","doc":"* `{:ok, serialized_value}` on success\n  * `{:error, reason}` on serialization failure","ref":"Exdantic.TypeAdapter.html#dump/3-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.dump/3","doc":"iex> Exdantic.TypeAdapter.dump(:string, \"hello\")\n    {:ok, \"hello\"}\n\n    iex> Exdantic.TypeAdapter.dump({:map, {:string, :any}}, %{name: \"John\", age: 30})\n    {:ok, %{\"name\" => \"John\", \"age\" => 30}}","ref":"Exdantic.TypeAdapter.html#dump/3-examples"},{"type":"type","title":"Exdantic.TypeAdapter.dump_options/0","doc":"","ref":"Exdantic.TypeAdapter.html#t:dump_options/0"},{"type":"function","title":"Exdantic.TypeAdapter.json_schema/2","doc":"Generates JSON Schema for a type specification.","ref":"Exdantic.TypeAdapter.html#json_schema/2"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.json_schema/2","doc":"* `type_spec` - The type specification to generate schema for\n  * `opts` - JSON Schema generation options","ref":"Exdantic.TypeAdapter.html#json_schema/2-parameters"},{"type":"function","title":"Options - Exdantic.TypeAdapter.json_schema/2","doc":"* `:title` - Schema title\n  * `:description` - Schema description\n  * `:resolve_refs` - Resolve all references inline (default: false)","ref":"Exdantic.TypeAdapter.html#json_schema/2-options"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.json_schema/2","doc":"* JSON Schema map","ref":"Exdantic.TypeAdapter.html#json_schema/2-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.json_schema/2","doc":"iex> Exdantic.TypeAdapter.json_schema(:string)\n    %{\"type\" => \"string\"}\n\n    iex> Exdantic.TypeAdapter.json_schema({:array, :integer})\n    %{\"type\" => \"array\", \"items\" => %{\"type\" => \"integer\"}}\n\n    iex> Exdantic.TypeAdapter.json_schema({:union, [:string, :integer]})\n    %{\"oneOf\" => [%{\"type\" => \"string\"}, %{\"type\" => \"integer\"}]}","ref":"Exdantic.TypeAdapter.html#json_schema/2-examples"},{"type":"type","title":"Exdantic.TypeAdapter.type_spec/0","doc":"","ref":"Exdantic.TypeAdapter.html#t:type_spec/0"},{"type":"function","title":"Exdantic.TypeAdapter.validate/3","doc":"Validates a value against a type specification.","ref":"Exdantic.TypeAdapter.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.validate/3","doc":"* `type_spec` - The type specification to validate against\n  * `value` - The value to validate\n  * `opts` - Validation options","ref":"Exdantic.TypeAdapter.html#validate/3-parameters"},{"type":"function","title":"Options - Exdantic.TypeAdapter.validate/3","doc":"* `:coerce` - Enable type coercion (default: false)\n  * `:strict` - Enable strict validation (default: false)\n  * `:path` - Validation path for error reporting (default: [])","ref":"Exdantic.TypeAdapter.html#validate/3-options"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.validate/3","doc":"* `{:ok, validated_value}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.TypeAdapter.html#validate/3-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.validate/3","doc":"iex> Exdantic.TypeAdapter.validate(:string, \"hello\")\n    {:ok, \"hello\"}\n\n    iex> Exdantic.TypeAdapter.validate(:integer, \"123\", coerce: true)\n    {:ok, 123}\n\n    iex> Exdantic.TypeAdapter.validate({:array, :string}, [\"a\", \"b\", \"c\"])\n    {:ok, [\"a\", \"b\", \"c\"]}\n\n    iex> Exdantic.TypeAdapter.validate(:integer, \"not a number\")\n    {:error, [%Exdantic.Error{...}]}","ref":"Exdantic.TypeAdapter.html#validate/3-examples"},{"type":"type","title":"Exdantic.TypeAdapter.validation_options/0","doc":"","ref":"Exdantic.TypeAdapter.html#t:validation_options/0"},{"type":"module","title":"Exdantic.TypeAdapter.Instance","doc":"A reusable TypeAdapter instance for efficient validation and serialization.\n\nThis module provides a struct that encapsulates a type specification and\nconfiguration options, allowing for efficient reuse of the same type\nvalidation and serialization logic.","ref":"Exdantic.TypeAdapter.Instance.html"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.dump/3","doc":"Serializes a value using this TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#dump/3"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.dump/3","doc":"* `instance` - The TypeAdapter instance\n  * `value` - The value to serialize\n  * `opts` - Additional serialization options","ref":"Exdantic.TypeAdapter.Instance.html#dump/3-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.dump/3","doc":"* `{:ok, serialized_value}` on success\n  * `{:error, reason}` on serialization failure","ref":"Exdantic.TypeAdapter.Instance.html#dump/3-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.dump/3","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new({:map, {:string, :any}})\n    iex> Exdantic.TypeAdapter.Instance.dump(adapter, %{name: \"John\"})\n    {:ok, %{\"name\" => \"John\"}}","ref":"Exdantic.TypeAdapter.Instance.html#dump/3-examples"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.dump_many/3","doc":"Serializes multiple values efficiently using the same TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#dump_many/3"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.dump_many/3","doc":"* `instance` - The TypeAdapter instance\n  * `values` - List of values to serialize\n  * `opts` - Serialization options","ref":"Exdantic.TypeAdapter.Instance.html#dump_many/3-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.dump_many/3","doc":"* `{:ok, serialized_values}` if all values serialize successfully\n  * `{:error, errors_by_index}` if any serialization fails","ref":"Exdantic.TypeAdapter.Instance.html#dump_many/3-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.dump_many/3","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new(:string)\n    iex> Exdantic.TypeAdapter.Instance.dump_many(adapter, [\"a\", \"b\", \"c\"])\n    {:ok, [\"a\", \"b\", \"c\"]}","ref":"Exdantic.TypeAdapter.Instance.html#dump_many/3-examples"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.info/1","doc":"Returns information about the TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#info/1"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.info/1","doc":"* `instance` - The TypeAdapter instance","ref":"Exdantic.TypeAdapter.Instance.html#info/1-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.info/1","doc":"* Map with instance information","ref":"Exdantic.TypeAdapter.Instance.html#info/1-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.info/1","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new({:array, :string})\n    iex> Exdantic.TypeAdapter.Instance.info(adapter)\n    %{\n      type_spec: {:array, :string},\n      normalized_type: {:array, {:type, :string, []}, []},\n      config: %{coerce: false, strict: false},\n      has_cached_schema: true\n    }","ref":"Exdantic.TypeAdapter.Instance.html#info/1-examples"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.json_schema/2","doc":"Gets the JSON schema for this TypeAdapter instance.\n\nIf the schema was cached during creation, returns the cached version.\nOtherwise, generates it on demand.","ref":"Exdantic.TypeAdapter.Instance.html#json_schema/2"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.json_schema/2","doc":"* `instance` - The TypeAdapter instance\n  * `opts` - JSON schema generation options","ref":"Exdantic.TypeAdapter.Instance.html#json_schema/2-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.json_schema/2","doc":"* JSON Schema map","ref":"Exdantic.TypeAdapter.Instance.html#json_schema/2-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.json_schema/2","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new(:string)\n    iex> Exdantic.TypeAdapter.Instance.json_schema(adapter)\n    %{\"type\" => \"string\"}","ref":"Exdantic.TypeAdapter.Instance.html#json_schema/2-examples"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.new/2","doc":"Creates a new TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#new/2"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.new/2","doc":"* `type_spec` - The type specification to create an adapter for\n  * `opts` - Configuration options","ref":"Exdantic.TypeAdapter.Instance.html#new/2-parameters"},{"type":"function","title":"Options - Exdantic.TypeAdapter.Instance.new/2","doc":"* `:coerce` - Enable type coercion by default (default: false)\n  * `:strict` - Enable strict validation by default (default: false)\n  * `:cache_json_schema` - Pre-generate and cache JSON schema (default: true)","ref":"Exdantic.TypeAdapter.Instance.html#new/2-options"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.new/2","doc":"* TypeAdapter.Instance struct","ref":"Exdantic.TypeAdapter.Instance.html#new/2-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.new/2","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new(:string)\n    %Exdantic.TypeAdapter.Instance{...}\n\n    iex> adapter = Exdantic.TypeAdapter.Instance.new({:array, :integer}, coerce: true)\n    %Exdantic.TypeAdapter.Instance{...}","ref":"Exdantic.TypeAdapter.Instance.html#new/2-examples"},{"type":"type","title":"Exdantic.TypeAdapter.Instance.t/0","doc":"","ref":"Exdantic.TypeAdapter.Instance.html#t:t/0"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.update_config/2","doc":"Updates the configuration of a TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#update_config/2"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.update_config/2","doc":"* `instance` - The TypeAdapter instance\n  * `new_config` - Configuration options to merge","ref":"Exdantic.TypeAdapter.Instance.html#update_config/2-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.update_config/2","doc":"* Updated TypeAdapter instance","ref":"Exdantic.TypeAdapter.Instance.html#update_config/2-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.update_config/2","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new(:string)\n    iex> updated = Exdantic.TypeAdapter.Instance.update_config(adapter, %{coerce: true})\n    %Exdantic.TypeAdapter.Instance{config: %{coerce: true, ...}}","ref":"Exdantic.TypeAdapter.Instance.html#update_config/2-examples"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.validate/3","doc":"Validates a value using this TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.validate/3","doc":"* `instance` - The TypeAdapter instance\n  * `value` - The value to validate\n  * `opts` - Additional validation options (override instance defaults)","ref":"Exdantic.TypeAdapter.Instance.html#validate/3-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.validate/3","doc":"* `{:ok, validated_value}` on success\n  * `{:error, errors}` on validation failure","ref":"Exdantic.TypeAdapter.Instance.html#validate/3-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.validate/3","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new(:string)\n    iex> Exdantic.TypeAdapter.Instance.validate(adapter, \"hello\")\n    {:ok, \"hello\"}\n\n    iex> adapter = Exdantic.TypeAdapter.Instance.new(:integer, coerce: true)\n    iex> Exdantic.TypeAdapter.Instance.validate(adapter, \"123\")\n    {:ok, 123}","ref":"Exdantic.TypeAdapter.Instance.html#validate/3-examples"},{"type":"function","title":"Exdantic.TypeAdapter.Instance.validate_many/3","doc":"Validates multiple values efficiently using the same TypeAdapter instance.","ref":"Exdantic.TypeAdapter.Instance.html#validate_many/3"},{"type":"function","title":"Parameters - Exdantic.TypeAdapter.Instance.validate_many/3","doc":"* `instance` - The TypeAdapter instance\n  * `values` - List of values to validate\n  * `opts` - Validation options","ref":"Exdantic.TypeAdapter.Instance.html#validate_many/3-parameters"},{"type":"function","title":"Returns - Exdantic.TypeAdapter.Instance.validate_many/3","doc":"* `{:ok, validated_values}` if all values are valid\n  * `{:error, errors_by_index}` if any validation fails","ref":"Exdantic.TypeAdapter.Instance.html#validate_many/3-returns"},{"type":"function","title":"Examples - Exdantic.TypeAdapter.Instance.validate_many/3","doc":"iex> adapter = Exdantic.TypeAdapter.Instance.new(:integer)\n    iex> Exdantic.TypeAdapter.Instance.validate_many(adapter, [1, 2, 3])\n    {:ok, [1, 2, 3]}\n\n    iex> Exdantic.TypeAdapter.Instance.validate_many(adapter, [1, \"bad\", 3])\n    {:error, %{1 => [%Exdantic.Error{...}]}}","ref":"Exdantic.TypeAdapter.Instance.html#validate_many/3-examples"},{"type":"module","title":"Exdantic.Types","doc":"Core type system for Exdantic schemas.\n\nProvides functions for defining and working with types:\n- Basic types (:string, :integer, :float, :boolean)\n- Complex types (arrays, maps, unions)\n- Type constraints\n- Type validation\n- Type coercion","ref":"Exdantic.Types.html"},{"type":"module","title":"Basic Types - Exdantic.Types","doc":"# String type\n    Types.string()\n\n    # Integer type with constraints\n    Types.integer()\n    |> Types.with_constraints(gt: 0, lt: 100)","ref":"Exdantic.Types.html#module-basic-types"},{"type":"module","title":"Complex Types - Exdantic.Types","doc":"# Array of strings\n    Types.array(Types.string())\n\n    # Map with string keys and integer values\n    Types.map(Types.string(), Types.integer())\n\n    # Union of types\n    Types.union([Types.string(), Types.integer()])","ref":"Exdantic.Types.html#module-complex-types"},{"type":"module","title":"Type Constraints - Exdantic.Types","doc":"Constraints can be added to types to enforce additional rules:\n\n    Types.string()\n    |> Types.with_constraints([\n      min_length: 3,\n      max_length: 10,\n      format: ~r/^[a-z]+$/\n    ])","ref":"Exdantic.Types.html#module-type-constraints"},{"type":"function","title":"Exdantic.Types.array/1","doc":"","ref":"Exdantic.Types.html#array/1"},{"type":"function","title":"Exdantic.Types.boolean/0","doc":"","ref":"Exdantic.Types.html#boolean/0"},{"type":"function","title":"Exdantic.Types.coerce/2","doc":"Coerces a value to the specified type.","ref":"Exdantic.Types.html#coerce/2"},{"type":"function","title":"Parameters - Exdantic.Types.coerce/2","doc":"* `type` - The target type to coerce to\n  * `value` - The value to coerce","ref":"Exdantic.Types.html#coerce/2-parameters"},{"type":"function","title":"Returns - Exdantic.Types.coerce/2","doc":"* `{:ok, coerced_value}` on success\n  * `{:error, reason}` on failure","ref":"Exdantic.Types.html#coerce/2-returns"},{"type":"function","title":"Examples - Exdantic.Types.coerce/2","doc":"iex> Exdantic.Types.coerce(:string, 42)\n    {:ok, \"42\"}\n\n    iex> Exdantic.Types.coerce(:integer, \"123\")\n    {:ok, 123}\n\n    iex> Exdantic.Types.coerce(:integer, \"abc\")\n    {:error, \"invalid integer format\"}","ref":"Exdantic.Types.html#coerce/2-examples"},{"type":"type","title":"Exdantic.Types.constraint_with_message/0","doc":"","ref":"Exdantic.Types.html#t:constraint_with_message/0"},{"type":"function","title":"Exdantic.Types.float/0","doc":"","ref":"Exdantic.Types.html#float/0"},{"type":"function","title":"Exdantic.Types.integer/0","doc":"","ref":"Exdantic.Types.html#integer/0"},{"type":"function","title":"Exdantic.Types.map/2","doc":"","ref":"Exdantic.Types.html#map/2"},{"type":"function","title":"Exdantic.Types.normalize_type/1","doc":"Normalizes a type definition to the standard internal format.","ref":"Exdantic.Types.html#normalize_type/1"},{"type":"function","title":"Parameters - Exdantic.Types.normalize_type/1","doc":"* `type` - The type definition to normalize","ref":"Exdantic.Types.html#normalize_type/1-parameters"},{"type":"function","title":"Returns - Exdantic.Types.normalize_type/1","doc":"* A normalized type definition tuple","ref":"Exdantic.Types.html#normalize_type/1-returns"},{"type":"function","title":"Examples - Exdantic.Types.normalize_type/1","doc":"iex> Exdantic.Types.normalize_type(:string)\n    {:type, :string, []}\n\n    iex> Exdantic.Types.normalize_type({:array, :integer})\n    {:array, {:type, :integer, []}, []}","ref":"Exdantic.Types.html#normalize_type/1-examples"},{"type":"function","title":"Exdantic.Types.object/1","doc":"","ref":"Exdantic.Types.html#object/1"},{"type":"function","title":"Exdantic.Types.ref/1","doc":"","ref":"Exdantic.Types.html#ref/1"},{"type":"function","title":"Exdantic.Types.string/0","doc":"","ref":"Exdantic.Types.html#string/0"},{"type":"function","title":"Exdantic.Types.tuple/1","doc":"","ref":"Exdantic.Types.html#tuple/1"},{"type":"function","title":"Exdantic.Types.type/1","doc":"","ref":"Exdantic.Types.html#type/1"},{"type":"type","title":"Exdantic.Types.type_definition/0","doc":"","ref":"Exdantic.Types.html#t:type_definition/0"},{"type":"function","title":"Exdantic.Types.union/1","doc":"","ref":"Exdantic.Types.html#union/1"},{"type":"function","title":"Exdantic.Types.validate/2","doc":"Validates a value against a basic type.","ref":"Exdantic.Types.html#validate/2"},{"type":"function","title":"Parameters - Exdantic.Types.validate/2","doc":"* `type` - The type to validate against\n  * `value` - The value to validate","ref":"Exdantic.Types.html#validate/2-parameters"},{"type":"function","title":"Returns - Exdantic.Types.validate/2","doc":"* `{:ok, value}` if validation succeeds\n  * `{:error, Exdantic.Error.t()}` if validation fails","ref":"Exdantic.Types.html#validate/2-returns"},{"type":"function","title":"Examples - Exdantic.Types.validate/2","doc":"iex> Exdantic.Types.validate(:string, \"hello\")\n    {:ok, \"hello\"}\n\n    iex> Exdantic.Types.validate(:integer, \"not a number\")\n    {:error, %Exdantic.Error{path: [], code: :type, message: \"expected integer, got \"not a number\"\"}}","ref":"Exdantic.Types.html#validate/2-examples"},{"type":"function","title":"Exdantic.Types.with_constraints/2","doc":"Adds constraints to a type definition.","ref":"Exdantic.Types.html#with_constraints/2"},{"type":"function","title":"Parameters - Exdantic.Types.with_constraints/2","doc":"* `type` - The type definition to add constraints to\n  * `constraints` - List of constraints to add","ref":"Exdantic.Types.html#with_constraints/2-parameters"},{"type":"function","title":"Returns - Exdantic.Types.with_constraints/2","doc":"* Updated type definition with constraints","ref":"Exdantic.Types.html#with_constraints/2-returns"},{"type":"function","title":"Examples - Exdantic.Types.with_constraints/2","doc":"iex> string_type = Exdantic.Types.string()\n    iex> Exdantic.Types.with_constraints(string_type, [min_length: 3, max_length: 10])\n    {:type, :string, [min_length: 3, max_length: 10]}","ref":"Exdantic.Types.html#with_constraints/2-examples"},{"type":"function","title":"Exdantic.Types.with_error_message/3","doc":"Adds a custom error message for a specific constraint to a type definition.","ref":"Exdantic.Types.html#with_error_message/3"},{"type":"function","title":"Parameters - Exdantic.Types.with_error_message/3","doc":"* `type` - The type definition to add the custom error message to\n  * `constraint` - The constraint name (atom) to customize the error for\n  * `message` - The custom error message to use when this constraint fails","ref":"Exdantic.Types.html#with_error_message/3-parameters"},{"type":"function","title":"Returns - Exdantic.Types.with_error_message/3","doc":"* Updated type definition with custom error message","ref":"Exdantic.Types.html#with_error_message/3-returns"},{"type":"function","title":"Examples - Exdantic.Types.with_error_message/3","doc":"iex> string_type = Exdantic.Types.string()\n    iex> |> Exdantic.Types.with_constraints([min_length: 3])\n    iex> |> Exdantic.Types.with_error_message(:min_length, \"Name must be at least 3 characters long\")\n    {:type, :string, [min_length: 3, {:error_message, :min_length, \"Name must be at least 3 characters long\"}]}","ref":"Exdantic.Types.html#with_error_message/3-examples"},{"type":"function","title":"Exdantic.Types.with_error_messages/2","doc":"Adds multiple custom error messages for constraints to a type definition.","ref":"Exdantic.Types.html#with_error_messages/2"},{"type":"function","title":"Parameters - Exdantic.Types.with_error_messages/2","doc":"* `type` - The type definition to add the custom error messages to\n  * `error_messages` - A keyword list or map of constraint => message pairs","ref":"Exdantic.Types.html#with_error_messages/2-parameters"},{"type":"function","title":"Returns - Exdantic.Types.with_error_messages/2","doc":"* Updated type definition with custom error messages","ref":"Exdantic.Types.html#with_error_messages/2-returns"},{"type":"function","title":"Examples - Exdantic.Types.with_error_messages/2","doc":"iex> string_type = Exdantic.Types.string()\n    iex> |> Exdantic.Types.with_constraints([min_length: 3, max_length: 50])\n    iex> |> Exdantic.Types.with_error_messages([\n    iex>      min_length: \"Name must be at least 3 characters long\",\n    iex>      max_length: \"Name cannot exceed 50 characters\"\n    iex>    ])\n    {:type, :string, [min_length: 3, max_length: 50, {:error_message, :min_length, \"Name must be at least 3 characters long\"}, {:error_message, :max_length, \"Name cannot exceed 50 characters\"}]}","ref":"Exdantic.Types.html#with_error_messages/2-examples"},{"type":"function","title":"Exdantic.Types.with_validator/2","doc":"Adds a custom validation function to a type definition.","ref":"Exdantic.Types.html#with_validator/2"},{"type":"function","title":"Parameters - Exdantic.Types.with_validator/2","doc":"* `type` - The type definition to add the custom validator to\n  * `validator_fn` - A function that takes a value and returns {:ok, value} | {:error, message}","ref":"Exdantic.Types.html#with_validator/2-parameters"},{"type":"function","title":"Returns - Exdantic.Types.with_validator/2","doc":"* Updated type definition with custom validator","ref":"Exdantic.Types.html#with_validator/2-returns"},{"type":"function","title":"Examples - Exdantic.Types.with_validator/2","doc":"iex> email_type = Exdantic.Types.string()\n    iex> |> Exdantic.Types.with_constraints([min_length: 3])\n    iex> |> Exdantic.Types.with_validator(fn value ->\n    iex>      if String.contains?(value, \"@\"), do: {:ok, value}, else: {:error, \"Must contain @\"}\n    iex>    end)\n    {:type, :string, [min_length: 3, {:validator, #Function<...>}]}","ref":"Exdantic.Types.html#with_validator/2-examples"},{"type":"module","title":"Exdantic.Validator","doc":"Validates values against type definitions and schemas.\n\nThis module provides the core validation logic for Exdantic schemas,\nhandling field validation, constraints, and error reporting.","ref":"Exdantic.Validator.html"},{"type":"function","title":"Exdantic.Validator.validate/2","doc":"","ref":"Exdantic.Validator.html#validate/2"},{"type":"function","title":"Exdantic.Validator.validate/3","doc":"Validates a value against a type definition.","ref":"Exdantic.Validator.html#validate/3"},{"type":"function","title":"Parameters - Exdantic.Validator.validate/3","doc":"* `type` - The type definition or schema module to validate against\n  * `value` - The value to validate\n  * `path` - Current validation path for error messages (defaults to `[]`)","ref":"Exdantic.Validator.html#validate/3-parameters"},{"type":"function","title":"Returns - Exdantic.Validator.validate/3","doc":"* `{:ok, validated_value}` on success\n  * `{:error, errors}` on validation failures","ref":"Exdantic.Validator.html#validate/3-returns"},{"type":"function","title":"Examples - Exdantic.Validator.validate/3","doc":"iex> Exdantic.Validator.validate({:type, :string, []}, \"hello\")\n    {:ok, \"hello\"}\n\n    iex> Exdantic.Validator.validate({:type, :integer, []}, \"not a number\")\n    {:error, %Exdantic.Error{...}}","ref":"Exdantic.Validator.html#validate/3-examples"},{"type":"function","title":"Exdantic.Validator.validate_schema/3","doc":"Validates data against a schema module, checking for required fields,\nfield-level validations, and strict mode constraints if enabled.","ref":"Exdantic.Validator.html#validate_schema/3"},{"type":"function","title":"Parameters - Exdantic.Validator.validate_schema/3","doc":"* `schema` - Schema module to validate against\n  * `data` - Data to validate (map)\n  * `path` - Current validation path for error messages (defaults to `[]`)","ref":"Exdantic.Validator.html#validate_schema/3-parameters"},{"type":"function","title":"Returns - Exdantic.Validator.validate_schema/3","doc":"* `{:ok, validated_data}` on success\n  * `{:error, errors}` on validation failures","ref":"Exdantic.Validator.html#validate_schema/3-returns"},{"type":"function","title":"Examples - Exdantic.Validator.validate_schema/3","doc":"iex> defmodule TestSchema do\n    ...>   use Exdantic\n    ...>   schema do\n    ...>     field :name, :string\n    ...>   end\n    ...> end\n    iex> Exdantic.Validator.validate_schema(TestSchema, %{name: \"John\"})\n    {:ok, %{name: \"John\"}}","ref":"Exdantic.Validator.html#validate_schema/3-examples"},{"type":"type","title":"Exdantic.Validator.validation_path/0","doc":"","ref":"Exdantic.Validator.html#t:validation_path/0"},{"type":"type","title":"Exdantic.Validator.validation_result/0","doc":"","ref":"Exdantic.Validator.html#t:validation_result/0"},{"type":"module","title":"Exdantic.Wrapper","doc":"Temporary validation schemas for type coercion patterns.\n\nThis module supports the DSPy pattern of creating temporary, single-field\nvalidation schemas for complex type coercion, equivalent to Pydantic's:\n`create_model(\"Wrapper\", value=(target_type, ...))`\n\nWrapper schemas are useful when you need to:\n- Validate a single value against a complex type specification\n- Apply field-level constraints and coercion\n- Extract and unwrap validated values\n- Perform temporary schema-based validation without defining a full schema","ref":"Exdantic.Wrapper.html"},{"type":"function","title":"Exdantic.Wrapper.create_flexible_wrapper/3","doc":"Creates a wrapper schema that can handle multiple input formats.\n\nThis wrapper can accept:\n- The raw value directly\n- A map with the field name as key\n- A map with string keys","ref":"Exdantic.Wrapper.html#create_flexible_wrapper/3"},{"type":"function","title":"Parameters - Exdantic.Wrapper.create_flexible_wrapper/3","doc":"* `field_name` - The field name for the wrapper\n  * `type_spec` - The type specification\n  * `opts` - Wrapper options","ref":"Exdantic.Wrapper.html#create_flexible_wrapper/3-parameters"},{"type":"function","title":"Examples - Exdantic.Wrapper.create_flexible_wrapper/3","doc":"iex> wrapper = Exdantic.Wrapper.create_flexible_wrapper(:age, :integer, coerce: true)\n    iex> Exdantic.Wrapper.validate_flexible(wrapper, 25, :age)        # Raw value\n    {:ok, 25}\n    iex> Exdantic.Wrapper.validate_flexible(wrapper, %{age: 25}, :age) # Map with atom key\n    {:ok, 25}\n    iex> Exdantic.Wrapper.validate_flexible(wrapper, %{\"age\" => 25}, :age) # Map with string key\n    {:ok, 25}","ref":"Exdantic.Wrapper.html#create_flexible_wrapper/3-examples"},{"type":"function","title":"Exdantic.Wrapper.create_multiple_wrappers/2","doc":"Creates multiple wrapper schemas for batch validation.","ref":"Exdantic.Wrapper.html#create_multiple_wrappers/2"},{"type":"function","title":"Parameters - Exdantic.Wrapper.create_multiple_wrappers/2","doc":"* `field_specs` - List of {field_name, type_spec, opts} tuples\n  * `global_opts` - Options applied to all wrappers","ref":"Exdantic.Wrapper.html#create_multiple_wrappers/2-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.create_multiple_wrappers/2","doc":"* Map of field_name => wrapper_schema","ref":"Exdantic.Wrapper.html#create_multiple_wrappers/2-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.create_multiple_wrappers/2","doc":"iex> specs = [\n    ...>   {:name, :string, [constraints: [min_length: 1]]},\n    ...>   {:age, :integer, [constraints: [gt: 0]]},\n    ...>   {:email, :string, [constraints: [format: ~r/@/]]}\n    ...> ]\n    iex> wrappers = Exdantic.Wrapper.create_multiple_wrappers(specs)\n    %{name: %DynamicSchema{...}, age: %DynamicSchema{...}, email: %DynamicSchema{...}}","ref":"Exdantic.Wrapper.html#create_multiple_wrappers/2-examples"},{"type":"function","title":"Exdantic.Wrapper.create_wrapper/3","doc":"Creates a temporary wrapper schema for validating a single value.","ref":"Exdantic.Wrapper.html#create_wrapper/3"},{"type":"function","title":"Parameters - Exdantic.Wrapper.create_wrapper/3","doc":"* `field_name` - The name for the wrapper field (atom)\n  * `type_spec` - The type specification for the field\n  * `opts` - Wrapper configuration options","ref":"Exdantic.Wrapper.html#create_wrapper/3-parameters"},{"type":"function","title":"Options - Exdantic.Wrapper.create_wrapper/3","doc":"* `:required` - Whether the field is required (default: true)\n  * `:coerce` - Enable type coercion (default: false)\n  * `:constraints` - Additional field constraints (default: [])\n  * `:description` - Field description for documentation\n  * `:example` - Example value for the field\n  * `:default` - Default value if field is missing","ref":"Exdantic.Wrapper.html#create_wrapper/3-options"},{"type":"function","title":"Returns - Exdantic.Wrapper.create_wrapper/3","doc":"* Wrapper schema that can be used for validation","ref":"Exdantic.Wrapper.html#create_wrapper/3-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.create_wrapper/3","doc":"iex> wrapper = Exdantic.Wrapper.create_wrapper(:result, :integer, coerce: true, constraints: [gt: 0])\n    %Exdantic.Runtime.DynamicSchema{...}\n\n    iex> wrapper = Exdantic.Wrapper.create_wrapper(:email, :string,\n    ...>   constraints: [format: ~r/@/], description: \"Email address\")\n    %Exdantic.Runtime.DynamicSchema{...}","ref":"Exdantic.Wrapper.html#create_wrapper/3-examples"},{"type":"function","title":"Exdantic.Wrapper.create_wrapper_factory/2","doc":"Creates a reusable wrapper factory for a specific type and constraints.","ref":"Exdantic.Wrapper.html#create_wrapper_factory/2"},{"type":"function","title":"Parameters - Exdantic.Wrapper.create_wrapper_factory/2","doc":"* `type_spec` - The type specification for the wrapper\n  * `base_opts` - Base options applied to all wrappers created by this factory","ref":"Exdantic.Wrapper.html#create_wrapper_factory/2-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.create_wrapper_factory/2","doc":"* Function that creates wrappers with the specified type and base options","ref":"Exdantic.Wrapper.html#create_wrapper_factory/2-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.create_wrapper_factory/2","doc":"iex> email_wrapper_factory = Exdantic.Wrapper.create_wrapper_factory(\n    ...>   :string,\n    ...>   constraints: [format: ~r/@/],\n    ...>   description: \"Email address\"\n    ...> )\n    iex> user_email_wrapper = email_wrapper_factory.(:user_email)\n    iex> admin_email_wrapper = email_wrapper_factory.(:admin_email, required: false)","ref":"Exdantic.Wrapper.html#create_wrapper_factory/2-examples"},{"type":"function","title":"Exdantic.Wrapper.to_json_schema/2","doc":"Converts a wrapper schema back to its JSON Schema representation.","ref":"Exdantic.Wrapper.html#to_json_schema/2"},{"type":"function","title":"Parameters - Exdantic.Wrapper.to_json_schema/2","doc":"* `wrapper_schema` - The wrapper schema to convert\n  * `opts` - JSON Schema generation options","ref":"Exdantic.Wrapper.html#to_json_schema/2-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.to_json_schema/2","doc":"* JSON Schema map representation of the wrapper","ref":"Exdantic.Wrapper.html#to_json_schema/2-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.to_json_schema/2","doc":"iex> wrapper = Exdantic.Wrapper.create_wrapper(:count, :integer, constraints: [gt: 0])\n    iex> Exdantic.Wrapper.to_json_schema(wrapper)\n    %{\n      \"type\" => \"object\",\n      \"properties\" => %{\n        \"count\" => %{\"type\" => \"integer\", \"exclusiveMinimum\" => 0}\n      },\n      \"required\" => [\"count\"]\n    }","ref":"Exdantic.Wrapper.html#to_json_schema/2-examples"},{"type":"function","title":"Exdantic.Wrapper.unwrap_result/2","doc":"Unwraps a validated result, extracting just the field value.\n\nUtility function for extracting values from wrapper validation results.","ref":"Exdantic.Wrapper.html#unwrap_result/2"},{"type":"function","title":"Parameters - Exdantic.Wrapper.unwrap_result/2","doc":"* `validated_result` - Result from wrapper validation (map)\n  * `field_name` - The field name to extract","ref":"Exdantic.Wrapper.html#unwrap_result/2-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.unwrap_result/2","doc":"* The unwrapped field value","ref":"Exdantic.Wrapper.html#unwrap_result/2-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.unwrap_result/2","doc":"iex> validated = %{score: 85}\n    iex> Exdantic.Wrapper.unwrap_result(validated, :score)\n    85","ref":"Exdantic.Wrapper.html#unwrap_result/2-examples"},{"type":"function","title":"Exdantic.Wrapper.validate_and_extract/3","doc":"Validates data using a wrapper schema and extracts the field value.","ref":"Exdantic.Wrapper.html#validate_and_extract/3"},{"type":"function","title":"Parameters - Exdantic.Wrapper.validate_and_extract/3","doc":"* `wrapper_schema` - The wrapper schema created by create_wrapper/3\n  * `data` - The data to validate (can be the raw value or a map)\n  * `field_name` - The field name to extract from the validated result","ref":"Exdantic.Wrapper.html#validate_and_extract/3-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.validate_and_extract/3","doc":"* `{:ok, extracted_value}` on successful validation and extraction\n  * `{:error, errors}` on validation failure","ref":"Exdantic.Wrapper.html#validate_and_extract/3-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.validate_and_extract/3","doc":"iex> wrapper = Exdantic.Wrapper.create_wrapper(:count, :integer, coerce: true)\n    iex> Exdantic.Wrapper.validate_and_extract(wrapper, %{count: \"42\"}, :count)\n    {:ok, 42}\n\n    iex> Exdantic.Wrapper.validate_and_extract(wrapper, \"42\", :count)  # Auto-wrap\n    {:ok, 42}\n\n    iex> Exdantic.Wrapper.validate_and_extract(wrapper, %{count: \"abc\"}, :count)\n    {:error, [%Exdantic.Error{...}]}","ref":"Exdantic.Wrapper.html#validate_and_extract/3-examples"},{"type":"function","title":"Exdantic.Wrapper.validate_flexible/3","doc":"Validates data against a flexible wrapper that can handle multiple input formats.","ref":"Exdantic.Wrapper.html#validate_flexible/3"},{"type":"function","title":"Parameters - Exdantic.Wrapper.validate_flexible/3","doc":"* `wrapper_schema` - The wrapper schema\n  * `data` - The input data (raw value, or map with atom/string keys)\n  * `field_name` - The field name to extract","ref":"Exdantic.Wrapper.html#validate_flexible/3-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.validate_flexible/3","doc":"* `{:ok, validated_value}` on success\n  * `{:error, errors}` on failure","ref":"Exdantic.Wrapper.html#validate_flexible/3-returns"},{"type":"function","title":"Exdantic.Wrapper.validate_multiple/2","doc":"Validates multiple values using their respective wrapper schemas.","ref":"Exdantic.Wrapper.html#validate_multiple/2"},{"type":"function","title":"Parameters - Exdantic.Wrapper.validate_multiple/2","doc":"* `wrappers` - Map of field_name => wrapper_schema\n  * `data` - Map of field_name => value to validate","ref":"Exdantic.Wrapper.html#validate_multiple/2-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.validate_multiple/2","doc":"* `{:ok, validated_values}` if all validations succeed\n  * `{:error, errors_by_field}` if any validation fails","ref":"Exdantic.Wrapper.html#validate_multiple/2-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.validate_multiple/2","doc":"iex> wrappers = %{\n    ...>   name: Exdantic.Wrapper.create_wrapper(:name, :string),\n    ...>   age: Exdantic.Wrapper.create_wrapper(:age, :integer)\n    ...> }\n    iex> data = %{name: \"John\", age: 30}\n    iex> Exdantic.Wrapper.validate_multiple(wrappers, data)\n    {:ok, %{name: \"John\", age: 30}}","ref":"Exdantic.Wrapper.html#validate_multiple/2-examples"},{"type":"function","title":"Exdantic.Wrapper.wrap_and_validate/4","doc":"Validates data using a wrapper schema and extracts the value in one step.\n\nThis is a convenience function that combines create_wrapper/3 and validate_and_extract/3.","ref":"Exdantic.Wrapper.html#wrap_and_validate/4"},{"type":"function","title":"Parameters - Exdantic.Wrapper.wrap_and_validate/4","doc":"* `field_name` - The name for the wrapper field\n  * `type_spec` - The type specification for the field\n  * `input` - The data to validate\n  * `opts` - Wrapper configuration options (same as create_wrapper/3)","ref":"Exdantic.Wrapper.html#wrap_and_validate/4-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.wrap_and_validate/4","doc":"* `{:ok, validated_value}` on successful validation\n  * `{:error, errors}` on validation failure","ref":"Exdantic.Wrapper.html#wrap_and_validate/4-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.wrap_and_validate/4","doc":"iex> Exdantic.Wrapper.wrap_and_validate(:score, :integer, \"85\", coerce: true, constraints: [gteq: 0, lteq: 100])\n    {:ok, 85}\n\n    iex> Exdantic.Wrapper.wrap_and_validate(:email, :string, \"invalid\", constraints: [format: ~r/@/])\n    {:error, [%Exdantic.Error{...}]}\n\n    iex> Exdantic.Wrapper.wrap_and_validate(:items, {:array, :string}, [\"a\", \"b\", \"c\"])\n    {:ok, [\"a\", \"b\", \"c\"]}","ref":"Exdantic.Wrapper.html#wrap_and_validate/4-examples"},{"type":"function","title":"Exdantic.Wrapper.wrapper_info/1","doc":"Gets metadata about a wrapper schema.","ref":"Exdantic.Wrapper.html#wrapper_info/1"},{"type":"function","title":"Parameters - Exdantic.Wrapper.wrapper_info/1","doc":"* `wrapper_schema` - The wrapper schema to inspect","ref":"Exdantic.Wrapper.html#wrapper_info/1-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.wrapper_info/1","doc":"* Map with wrapper metadata","ref":"Exdantic.Wrapper.html#wrapper_info/1-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.wrapper_info/1","doc":"iex> wrapper = Exdantic.Wrapper.create_wrapper(:email, :string, description: \"User email\")\n    iex> Exdantic.Wrapper.wrapper_info(wrapper)\n    %{\n      is_wrapper: true,\n      field_name: :email,\n      field_count: 1,\n      wrapper_type: :single_field,\n      created_at: ~U[...]\n    }","ref":"Exdantic.Wrapper.html#wrapper_info/1-examples"},{"type":"type","title":"Exdantic.Wrapper.wrapper_options/0","doc":"","ref":"Exdantic.Wrapper.html#t:wrapper_options/0"},{"type":"type","title":"Exdantic.Wrapper.wrapper_schema/0","doc":"","ref":"Exdantic.Wrapper.html#t:wrapper_schema/0"},{"type":"function","title":"Exdantic.Wrapper.wrapper_schema?/1","doc":"Checks if a schema is a wrapper schema created by this module.","ref":"Exdantic.Wrapper.html#wrapper_schema?/1"},{"type":"function","title":"Parameters - Exdantic.Wrapper.wrapper_schema?/1","doc":"* `schema` - The schema to check","ref":"Exdantic.Wrapper.html#wrapper_schema?/1-parameters"},{"type":"function","title":"Returns - Exdantic.Wrapper.wrapper_schema?/1","doc":"* `true` if it's a wrapper schema, `false` otherwise","ref":"Exdantic.Wrapper.html#wrapper_schema?/1-returns"},{"type":"function","title":"Examples - Exdantic.Wrapper.wrapper_schema?/1","doc":"iex> wrapper = Exdantic.Wrapper.create_wrapper(:test, :string)\n    iex> Exdantic.Wrapper.wrapper_schema?(wrapper)\n    true\n\n    iex> regular_schema = Exdantic.Runtime.create_schema([{:name, :string}])\n    iex> Exdantic.Wrapper.wrapper_schema?(regular_schema)\n    false","ref":"Exdantic.Wrapper.html#wrapper_schema?/1-examples"},{"type":"exception","title":"Exdantic.ValidationError","doc":"Exception raised when schema validation fails.\n\nThis exception is raised when using the validate!/1 functions\nand validation fails, providing detailed error information.","ref":"Exdantic.ValidationError.html"},{"type":"function","title":"Exdantic.ValidationError.message/1","doc":"Formats the validation errors into a human-readable message.","ref":"Exdantic.ValidationError.html#message/1"},{"type":"function","title":"Parameters - Exdantic.ValidationError.message/1","doc":"* `exception` - The ValidationError exception struct","ref":"Exdantic.ValidationError.html#message/1-parameters"},{"type":"function","title":"Returns - Exdantic.ValidationError.message/1","doc":"* A formatted error message string","ref":"Exdantic.ValidationError.html#message/1-returns"},{"type":"function","title":"Examples - Exdantic.ValidationError.message/1","doc":"iex> errors = [%Exdantic.Error{path: [:name], code: :required, message: \"field is required\"}]\n    iex> exception = %Exdantic.ValidationError{errors: errors}\n    iex> Exdantic.ValidationError.message(exception)\n    \"name: field is required\"","ref":"Exdantic.ValidationError.html#message/1-examples"},{"type":"type","title":"Exdantic.ValidationError.t/0","doc":"","ref":"Exdantic.ValidationError.html#t:t/0"},{"type":"extras","title":"Exdantic","doc":"# Exdantic\n\n**A powerful, flexible schema definition and validation library for Elixir, inspired by Python's Pydantic.**\n\nThis project is directly based on [Elixact](https://github.com/LiboShen/elixact) by LiboShen.\n\n[![CI](https://github.com/nshkrdotcom/exdantic/actions/workflows/ci.yml/badge.svg)](https://github.com/nshkrdotcom/exdantic/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/exdantic.svg)](https://hex.pm/packages/exdantic)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/exdantic/)\n\nExdantic provides a comprehensive toolset for data validation, serialization, and schema generation. Perfect for building robust APIs, managing complex configurations, and creating data processing pipelines. With advanced runtime features, Exdantic is uniquely suited for AI and LLM applications, enabling dynamic, DSPy-style programming patterns in Elixir.","ref":"readme.html"},{"type":"extras","title":"Original Project - Exdantic","doc":"The original Elixact project can be found at: https://github.com/LiboShen/elixact","ref":"readme.html#original-project"},{"type":"extras","title":"✨ Key Features - Exdantic","doc":"- 🎯 **Rich Type System**: Support for basic types, complex nested structures (arrays, maps, unions), and custom types\n- 🚀 **Runtime & Compile-Time Schemas**: Define schemas dynamically at runtime or at compile-time for maximum flexibility\n- 🏗️ **Struct Support**: Optional struct generation for type-safe data structures with automatic serialization\n- 🔧 **Model Validators**: Cross-field validation and data transformation after field validation\n- ⚡ **Computed Fields**: Derive additional fields from validated data automatically\n- 🎨 **Pydantic-Inspired Patterns**: Support for `create_model`, `TypeAdapter`, `Wrapper`, and `RootModel` patterns\n- 🔍 **Advanced Validation**: Rich constraints plus custom validation functions\n- 🔄 **Type Coercion**: Automatic and configurable type coercion\n- 📊 **Advanced JSON Schema**: Generate optimized JSON Schema for LLM providers (OpenAI, Anthropic)\n- 🚨 **Structured Errors**: Path-aware error messages for precise debugging\n- ⚙️ **Dynamic Configuration**: Runtime configuration with preset patterns","ref":"readme.html#key-features"},{"type":"extras","title":"🚀 Quick Start - Exdantic","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Installation - Exdantic","doc":"Add `exdantic` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:exdantic, \"~> 0.0.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Basic Schema Definition - Exdantic","doc":"```elixir\ndefmodule UserSchema do\n  use Exdantic, define_struct: true\n\n  schema \"User account information\" do\n    field :name, :string do\n      required()\n      min_length(2)\n      description(\"User's full name\")\n    end\n\n    field :email, :string do\n      required()\n      format(~r/^[^\\s@]+@[^\\s@]+\\.[^\\s@]+$/)\n      description(\"Primary email address\")\n    end\n\n    field :age, :integer do\n      optional()\n      gt(0)\n      lt(150)\n      description(\"User's age in years\")\n    end\n\n    field :active, :boolean do\n      default(true)\n      description(\"Whether the account is active\")\n    end\n\n    # Cross-field validation\n    model_validator :validate_adult_email\n\n    # Computed field derived from other fields\n    computed_field :display_name, :string, :generate_display_name\n\n    config do\n      title(\"User Schema\")\n      strict(true)\n    end\n  end\n\n  def validate_adult_email(input) do\n    if input.age && input.age >= 18 && String.contains?(input.email, \"example.com\") do\n      {:error, \"Adult users cannot use example.com emails\"}\n    else\n      {:ok, input}\n    end\n  end\n\n  def generate_display_name(input) do\n    display = if input.age do\n      \"#{input.name} (#{input.age})\"\n    else\n      input.name\n    end\n    {:ok, display}\n  end\nend\n\n# Validation returns struct instances when define_struct: true\ncase UserSchema.validate(%{\n  name: \"John Doe\",\n  email: \"john@company.com\",\n  age: 30\n}) do\n  {:ok, %UserSchema{} = user} ->\n    IO.puts(\"User: #{user.display_name}\")\n    # Outputs: \"User: John Doe (30)\"\n    \n  {:error, errors} ->\n    Enum.each(errors, &IO.puts(Exdantic.Error.format(&1)))\nend\n```","ref":"readme.html#basic-schema-definition"},{"type":"extras","title":"Dynamic Runtime Schemas - Exdantic","doc":"Perfect for DSPy-style applications and dynamic validation:\n\n```elixir\n# Define fields programmatically\nfields = [\n  {:reasoning, :string, [description: \"Chain of thought reasoning\"]},\n  {:answer, :string, [required: true, min_length: 1]},\n  {:confidence, :float, [required: true, gteq: 0.0, lteq: 1.0]},\n  {:sources, {:array, :string}, [optional: true]}\n]\n\n# Create schema at runtime (like Pydantic's create_model)\nllm_output_schema = Exdantic.Runtime.create_schema(fields, \n  title: \"LLM_Output_Schema\",\n  description: \"Schema for LLM structured output\"\n)\n\n# Validate with coercion\nconfig = Exdantic.Config.create(coercion: :safe, strict: true)\nllm_response = %{\n  \"reasoning\" => \"Based on the context provided...\",\n  \"answer\" => \"The answer is 42\",\n  \"confidence\" => \"0.95\"  # String that needs coercion to float\n}\n\n{:ok, validated} = Exdantic.EnhancedValidator.validate(\n  llm_output_schema, \n  llm_response, \n  config: config\n)\n\n# Generate JSON Schema for LLM providers\njson_schema = Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced(\n  llm_output_schema,\n  optimize_for_provider: :openai,\n  flatten_for_llm: true\n)\n```","ref":"readme.html#dynamic-runtime-schemas"},{"type":"extras","title":"TypeAdapter for Schemaless Validation - Exdantic","doc":"Validate individual values without full schema definition:\n\n```elixir\n# Simple type validation with coercion\n{:ok, 123} = Exdantic.TypeAdapter.validate(:integer, \"123\", coerce: true)\n\n# Complex type validation\ntype_spec = {:array, {:map, {:string, {:union, [:string, :integer]}}}}\ndata = [\n  %{\"name\" => \"John\", \"score\" => 85},\n  %{\"name\" => \"Jane\", \"score\" => \"92\"}  # String score gets coerced\n]\n\n{:ok, validated} = Exdantic.TypeAdapter.validate(type_spec, data, coerce: true)\n\n# Reusable TypeAdapter instances for performance\nadapter = Exdantic.TypeAdapter.create({:array, :string}, coerce: true)\n{:ok, results} = Exdantic.TypeAdapter.Instance.validate_many(adapter, [\n  [\"a\", \"b\"],\n  [1, 2],     # Numbers get coerced to strings\n  [\"c\", \"d\"]\n])\n```","ref":"readme.html#typeadapter-for-schemaless-validation"},{"type":"extras","title":"Wrapper Models for Single-Field Validation - Exdantic","doc":"Temporary schemas for complex type coercion patterns:\n\n```elixir\n# Validate and extract a single complex value\n{:ok, score} = Exdantic.Wrapper.wrap_and_validate(\n  :test_score,\n  :integer,\n  \"85\",  # String input\n  coerce: true,\n  constraints: [gteq: 0, lteq: 100],\n  description: \"Test score out of 100\"\n)\n\n# Multiple wrapper validation\nwrappers = Exdantic.Wrapper.create_multiple_wrappers([\n  {:name, :string, [constraints: [min_length: 1]]},\n  {:age, :integer, [constraints: [gt: 0]]},\n  {:score, :float, [constraints: [gteq: 0.0, lteq: 1.0]]}\n])\n\ndata = %{name: \"Test\", age: \"25\", score: \"0.95\"}  # All strings\n{:ok, validated} = Exdantic.Wrapper.validate_multiple(wrappers, data)\n# All values are properly coerced to their target types\n```","ref":"readme.html#wrapper-models-for-single-field-validation"},{"type":"extras","title":"Root Schema for Non-Dictionary Validation - Exdantic","doc":"Validate non-dictionary types at the root level (similar to Pydantic's RootModel):\n\n```elixir\n# Validate an array of integers\ndefmodule IntegerListSchema do\n  use Exdantic.RootSchema, root: {:array, :integer}\nend\n\n{:ok, [1, 2, 3]} = IntegerListSchema.validate([1, 2, 3])\n\n# Validate a string with constraints\ndefmodule EmailSchema do\n  use Exdantic.RootSchema, \n    root: {:type, :string, [format: ~r/^[^\\s@]+@[^\\s@]+\\.[^\\s@]+$/]}\nend\n\n{:ok, \"user@example.com\"} = EmailSchema.validate(\"user@example.com\")\n\n# Validate union types\ndefmodule StringOrNumberSchema do\n  use Exdantic.RootSchema, root: {:union, [:string, :integer]}\nend\n\n{:ok, \"hello\"} = StringOrNumberSchema.validate(\"hello\")\n{:ok, 42} = StringOrNumberSchema.validate(42)\n\n# Validate arrays of complex schemas\ndefmodule UserListSchema do\n  use Exdantic.RootSchema, root: {:array, UserSchema}\nend\n\nusers = [%{name: \"John\", email: \"john@example.com\"}]\n{:ok, validated_users} = UserListSchema.validate(users)\n```","ref":"readme.html#root-schema-for-non-dictionary-validation"},{"type":"extras","title":"🏗️ Core Concepts - Exdantic","doc":"","ref":"readme.html#core-concepts"},{"type":"extras","title":"Schema Definition Approaches - Exdantic","doc":"**Compile-Time Schemas** (Best for static, performance-critical validation):\n```elixir\ndefmodule APIRequestSchema do\n  use Exdantic, define_struct: true\n  \n  schema do\n    field :action, :string, choices: [\"create\", \"update\", \"delete\"]\n    field :resource_id, :integer, gt: 0\n    field :data, :map, optional: true\n    \n    model_validator :validate_action_data_consistency\n  end\nend\n```\n\n**Runtime Schemas** (Best for dynamic validation, DSPy patterns):\n```elixir\n# Schema created from field definitions at runtime\nschema = Exdantic.Runtime.create_schema([\n  {:query, :string, [required: true]},\n  {:max_results, :integer, [optional: true, gt: 0, lteq: 100]}\n])\n```","ref":"readme.html#schema-definition-approaches"},{"type":"extras","title":"Type System - Exdantic","doc":"Exdantic supports a comprehensive type system:\n\n```elixir\n# Basic types\n:string, :integer, :float, :boolean, :atom, :any, :map\n\n# Complex types\n{:array, inner_type}                    # Arrays\n{:map, {key_type, value_type}}         # Maps with typed keys/values\n{:union, [type1, type2, ...]}          # Union types\n{:tuple, [type1, type2, ...]}          # Tuples\n\n# Schema references\nModuleName                              # Reference to other schemas\n\n# Custom types\nMyCustomType                            # Modules implementing Exdantic.Type\n```","ref":"readme.html#type-system"},{"type":"extras","title":"Validation Pipeline - Exdantic","doc":"1. **Field Validation**: Individual field type and constraint checking\n2. **Model Validation**: Cross-field validation and data transformation\n3. **Computed Fields**: Generation of derived fields\n4. **Struct Creation**: Optional struct instantiation (when `define_struct: true`)","ref":"readme.html#validation-pipeline"},{"type":"extras","title":"Configuration System - Exdantic","doc":"Control validation behavior with dynamic configuration:\n\n```elixir\n# Create configurations\nstrict_config = Exdantic.Config.create(strict: true, extra: :forbid)\nlenient_config = Exdantic.Config.preset(:development)\n\n# Builder pattern\nconfig = Exdantic.Config.builder()\n|> Exdantic.Config.Builder.strict(true)\n|> Exdantic.Config.Builder.safe_coercion()\n|> Exdantic.Config.Builder.for_api()\n|> Exdantic.Config.Builder.build()\n\n# Use with any validation\nExdantic.EnhancedValidator.validate(schema, data, config: config)\n```","ref":"readme.html#configuration-system"},{"type":"extras","title":"🔍 Advanced Features - Exdantic","doc":"","ref":"readme.html#advanced-features"},{"type":"extras","title":"Model Validators - Exdantic","doc":"Cross-field validation and data transformation:\n\n```elixir\nschema do\n  field :password, :string, min_length: 8\n  field :password_confirmation, :string\n  \n  # Named function validator\n  model_validator :validate_passwords_match\n  \n  # Anonymous function validator  \n  model_validator fn input ->\n    if input.password == input.password_confirmation do\n      # Transform: remove confirmation field\n      {:ok, Map.delete(input, :password_confirmation)}\n    else\n      {:error, \"Passwords do not match\"}\n    end\n  end\nend\n```","ref":"readme.html#model-validators"},{"type":"extras","title":"Computed Fields - Exdantic","doc":"Automatically derive fields from validated data:\n\n```elixir\nschema do\n  field :first_name, :string, required: true\n  field :last_name, :string, required: true\n  field :email, :string, required: true\n  \n  # Named function computed field\n  computed_field :full_name, :string, :generate_full_name\n  \n  # Anonymous function computed field\n  computed_field :email_domain, :string, fn input ->\n    domain = input.email |> String.split(\"@\") |> List.last()\n    {:ok, domain}\n  end\nend\n\ndef generate_full_name(input) do\n  {:ok, \"#{input.first_name} #{input.last_name}\"}\nend\n```","ref":"readme.html#computed-fields"},{"type":"extras","title":"Enhanced JSON Schema Generation - Exdantic","doc":"Generate optimized schemas for different use cases:\n\n```elixir\n# Basic JSON Schema\njson_schema = Exdantic.JsonSchema.from_schema(UserSchema)\n\n# Enhanced schema with full metadata\nenhanced_schema = Exdantic.JsonSchema.EnhancedResolver.resolve_enhanced(\n  UserSchema,\n  optimize_for_provider: :openai,\n  include_model_validators: true,\n  include_computed_fields: true\n)\n\n# DSPy-optimized schema\ndspy_schema = Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy(\n  UserSchema,\n  signature_mode: true,\n  field_descriptions: true,\n  strict_types: true\n)\n\n# Comprehensive analysis\nanalysis = Exdantic.JsonSchema.EnhancedResolver.comprehensive_analysis(\n  UserSchema,\n  sample_data,\n  test_llm_providers: [:openai, :anthropic, :generic]\n)\n```","ref":"readme.html#enhanced-json-schema-generation"},{"type":"extras","title":"Runtime Enhanced Schemas - Exdantic","doc":"Runtime schemas with model validators and computed fields:\n\n```elixir\n# Enhanced runtime schema with full pipeline\nfields = [{:name, :string, [required: true]}, {:age, :integer, [optional: true]}]\n\nvalidators = [\n  fn data -> {:ok, %{data | name: String.trim(data.name)}} end\n]\n\ncomputed_fields = [\n  {:display_name, :string, fn data -> {:ok, String.upcase(data.name)} end}\n]\n\nenhanced_schema = Exdantic.Runtime.create_enhanced_schema(fields,\n  model_validators: validators,\n  computed_fields: computed_fields,\n  title: \"Enhanced User Schema\"\n)\n\n{:ok, result} = Exdantic.Runtime.validate_enhanced(\n  %{name: \"  john  \", age: 25}, \n  enhanced_schema\n)\n# Result: %{name: \"john\", age: 25, display_name: \"JOHN\"}\n```","ref":"readme.html#runtime-enhanced-schemas"},{"type":"extras","title":"📊 JSON Schema & LLM Integration - Exdantic","doc":"","ref":"readme.html#json-schema-llm-integration"},{"type":"extras","title":"LLM Provider Optimization - Exdantic","doc":"Generate optimized schemas for different LLM providers:\n\n```elixir\n# OpenAI Function Calling\nopenai_schema = Exdantic.JsonSchema.Resolver.enforce_structured_output(\n  base_schema,\n  provider: :openai,\n  remove_unsupported: true\n)\n\n# Anthropic Tool Use\nanthropic_schema = Exdantic.JsonSchema.Resolver.enforce_structured_output(\n  base_schema,\n  provider: :anthropic\n)\n\n# Resolve all references for simplified schemas\nresolved_schema = Exdantic.JsonSchema.Resolver.resolve_references(complex_schema)\n```","ref":"readme.html#llm-provider-optimization"},{"type":"extras","title":"DSPy Integration Patterns - Exdantic","doc":"```elixir\n# Input schema (remove computed fields for input validation)\ninput_schema = Exdantic.JsonSchema.remove_computed_fields(full_schema)\n\n# Output schema (include all fields for output validation)  \noutput_schema = Exdantic.JsonSchema.EnhancedResolver.optimize_for_dspy(\n  full_schema,\n  signature_mode: true\n)\n\n# DSPy-compatible configuration\ndspy_config = Exdantic.Config.for_dspy(:signature, provider: :openai)\n```","ref":"readme.html#dspy-integration-patterns"},{"type":"extras","title":"⚙️ Available Constraints - Exdantic","doc":"","ref":"readme.html#available-constraints"},{"type":"extras","title":"String Constraints - Exdantic","doc":"- `min_length(n)` - Minimum string length\n- `max_length(n)` - Maximum string length  \n- `format(regex)` - String must match regex pattern\n- `choices(list)` - String must be one of the provided choices","ref":"readme.html#string-constraints"},{"type":"extras","title":"Numeric Constraints (Integer/Float) - Exdantic","doc":"- `gt(n)` - Greater than\n- `lt(n)` - Less than\n- `gteq(n)` - Greater than or equal to\n- `lteq(n)` - Less than or equal to\n- `choices(list)` - Number must be one of the provided choices","ref":"readme.html#numeric-constraints-integer-float"},{"type":"extras","title":"Array Constraints - Exdantic","doc":"- `min_items(n)` - Minimum number of items\n- `max_items(n)` - Maximum number of items","ref":"readme.html#array-constraints"},{"type":"extras","title":"Field Modifiers - Exdantic","doc":"- `required()` - Field is required (default)\n- `optional()` - Field is optional\n- `default(value)` - Default value if not provided (implies optional)\n- `description(text)` - Field description\n- `example(value)` - Example value for documentation\n- `examples(list)` - Multiple example values","ref":"readme.html#field-modifiers"},{"type":"extras","title":"🎨 Custom Types - Exdantic","doc":"Create reusable custom types:\n\n```elixir\ndefmodule Types.Email do\n  use Exdantic.Type\n\n  def type_definition do\n    {:type, :string, [format: ~r/^[^\\s@]+@[^\\s@]+\\.[^\\s@]+$/]}\n  end\n\n  def json_schema do\n    %{\n      \"type\" => \"string\",\n      \"format\" => \"email\",\n      \"pattern\" => \"^[^\\\\s@]+@[^\\\\s@]+\\\\.[^\\\\s@]+$\"\n    }\n  end\n\n  def validate(value) do\n    case type_definition() |> Exdantic.Validator.validate(value) do\n      {:ok, validated} -> {:ok, String.downcase(validated)}\n      {:error, _} -> {:error, \"Must be a valid email address\"}\n    end\n  end\nend\n\n# Use the custom type\nschema do\n  field :email, Types.Email\n  field :backup_email, Types.Email, optional: true\nend\n```","ref":"readme.html#custom-types"},{"type":"extras","title":"🚨 Error Handling - Exdantic","doc":"Exdantic provides structured, path-aware error information:\n\n```elixir\n%Exdantic.Error{\n  path: [:user, :address, :zip_code],  # Exact location of error\n  code: :format,                       # Machine-readable error type\n  message: \"invalid zip code format\"   # Human-readable message\n}\n\n# Format errors for display\ncase UserSchema.validate(invalid_data) do\n  {:ok, validated} -> \n    validated\n    \n  {:error, errors} ->\n    errors\n    |> Enum.map(&Exdantic.Error.format/1)\n    |> Enum.each(&IO.puts/1)\n    # Outputs: \"user.address.zip_code: invalid zip code format\"\nend\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"🔧 Configuration Options - Exdantic","doc":"","ref":"readme.html#configuration-options"},{"type":"extras","title":"Validation Behavior - Exdantic","doc":"- `:strict` - Enforce strict validation (no unknown fields)\n- `:extra` - Handle extra fields (`:allow`, `:forbid`, `:ignore`)\n- `:coercion` - Type coercion strategy (`:none`, `:safe`, `:aggressive`)\n- `:frozen` - Whether configuration is immutable\n- `:validate_assignment` - Validate field assignments","ref":"readme.html#validation-behavior"},{"type":"extras","title":"Error Handling - Exdantic","doc":"- `:error_format` - Error detail level (`:detailed`, `:simple`, `:minimal`)\n- `:case_sensitive` - Case sensitivity for field names","ref":"readme.html#error-handling-1"},{"type":"extras","title":"Schema Generation - Exdantic","doc":"- `:use_enum_values` - Use enum values instead of names\n- `:max_anyof_union_len` - Maximum length for anyOf unions\n- `:title_generator` - Function to generate field titles\n- `:description_generator` - Function to generate field descriptions","ref":"readme.html#schema-generation"},{"type":"extras","title":"📈 Performance Guidelines - Exdantic","doc":"","ref":"readme.html#performance-guidelines"},{"type":"extras","title":"Best Practices - Exdantic","doc":"1. **Reuse Schemas and Adapters**: Create once, use many times\n2. **Use Appropriate Configuration**: Avoid `:strict` if not needed\n3. **Batch Operations**: Use `validate_many` for multiple items\n4. **Cache JSON Schemas**: Generate once for repeated use\n5. **Choose the Right Tool**:\n   - Compile-time schemas for static, performance-critical validation\n   - Runtime schemas for dynamic validation\n   - TypeAdapter for simple type validation\n   - Wrapper for single-value coercion","ref":"readme.html#best-practices"},{"type":"extras","title":"Performance Characteristics - Exdantic","doc":"- **Simple validation**: Sub-millisecond for basic types\n- **Complex schemas**: ~5-20ms for typical business objects\n- **Runtime schema creation**: ~1000 schemas/second\n- **JSON schema generation**: <50ms for complex schemas\n- **Batch validation**: ~10k items/second for simple types","ref":"readme.html#performance-characteristics"},{"type":"extras","title":"🗺️ Migration Guide - Exdantic","doc":"","ref":"readme.html#migration-guide"},{"type":"extras","title":"From Basic Validation Libraries - Exdantic","doc":"```elixir\n# Before: Manual validation\ndef validate_user(data) do\n  with {:ok, name} <- validate_string(data[\"name\"]),\n       {:ok, age} <- validate_integer(data[\"age\"]),\n       {:ok, email} <- validate_email(data[\"email\"]) do\n    {:ok, %{name: name, age: age, email: email}}\n  end\nend\n\n# After: Exdantic schema\ndefmodule UserSchema do\n  use Exdantic\n  \n  schema do\n    field :name, :string, required: true\n    field :age, :integer, gt: 0\n    field :email, :string, format: ~r/@/\n  end\nend\n\n{:ok, user} = UserSchema.validate(data)\n```","ref":"readme.html#from-basic-validation-libraries"},{"type":"extras","title":"From Static to Dynamic Validation - Exdantic","doc":"```elixir\n# Static schema (compile-time)\ndefmodule APISchema do\n  use Exdantic\n  schema do\n    field :endpoint, :string\n    field :method, :string, choices: [\"GET\", \"POST\"]\n  end\nend\n\n# Dynamic schema (runtime)\napi_schema = Exdantic.Runtime.create_schema([\n  {:endpoint, :string, [required: true]},\n  {:method, :string, [choices: [\"GET\", \"POST\", \"PUT\", \"DELETE\"]]}\n])\n```","ref":"readme.html#from-static-to-dynamic-validation"},{"type":"extras","title":"📚 Examples - Exdantic","doc":"The `examples/` directory contains comprehensive examples showcasing all of Exdantic's features:","ref":"readme.html#examples"},{"type":"extras","title":"🟢 Getting Started - Exdantic","doc":"- [`basic_usage.exs`](examples/basic_usage.exs) - Core concepts and fundamental patterns\n- [`custom_validation.exs`](examples/custom_validation.exs) - Business logic and custom validators\n- [`advanced_features.exs`](examples/advanced_features.exs) - Complex validation patterns","ref":"readme.html#getting-started"},{"type":"extras","title":"🤖 LLM & DSPy Integration - Exdantic","doc":"- [`llm_integration.exs`](examples/llm_integration.exs) - LLM output validation\n- [`dspy_integration.exs`](examples/dspy_integration.exs) - Complete DSPy patterns\n- [`llm_pipeline_orchestration.exs`](examples/llm_pipeline_orchestration.exs) - Multi-stage pipelines","ref":"readme.html#llm-dspy-integration"},{"type":"extras","title":"⚡ Runtime Features - Exdantic","doc":"- [`runtime_schema.exs`](examples/runtime_schema.exs) - Dynamic schema creation\n- [`type_adapter.exs`](examples/type_adapter.exs) - Runtime type validation\n- [`wrapper_models.exs`](examples/wrapper_models.exs) - Single-field validation","ref":"readme.html#runtime-features"},{"type":"extras","title":"🔧 Advanced Features - Exdantic","doc":"- [`model_validators.exs`](examples/model_validators.exs) - Cross-field validation\n- [`computed_fields.exs`](examples/computed_fields.exs) - Derived fields\n- [`root_schema.exs`](examples/root_schema.exs) - Non-dictionary validation\n\n**Run any example:**\n```bash\nmix run examples/basic_usage.exs\n```\n\n**See the complete guide:** [`examples/README.md`](examples/README.md)","ref":"readme.html#advanced-features-1"},{"type":"extras","title":"🤝 Contributing - Exdantic","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b feature/amazing-feature`)\n3. Write tests for your changes\n4. Ensure all tests pass (`mix test`)\n5. Run quality checks:\n   ```bash\n   mix format\n   mix credo --strict\n   mix dialyzer\n   ```\n6. Commit your changes (`git commit -am 'Add amazing feature'`)\n7. Push to the branch (`git push origin feature/amazing-feature`)\n8. Open a Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"Development Setup - Exdantic","doc":"```bash\n# Clone the repository\ngit clone https://github.com/nshkrdotcom/exdantic.git\ncd exdantic\n\n# Install dependencies\nmix deps.get\n\n# Run tests\nmix test\n\n# Generate docs\nmix docs\n```","ref":"readme.html#development-setup"},{"type":"extras","title":"📄 License - Exdantic","doc":"This project is licensed under the MIT License - see the LICENSE file for details.","ref":"readme.html#license"},{"type":"extras","title":"🙏 Acknowledgments - Exdantic","doc":"- Inspired by Python's [Pydantic](https://pydantic-docs.helpmanual.io/) library\n- DSPy integration patterns inspired by [DSPy](https://dspy-docs.vercel.app/)\n- Built with ❤️ for the Elixir community\n\n---\n\n**Made with Elixir** 💜","ref":"readme.html#acknowledgments"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2025 NSHkr\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"},{"type":"extras","title":"Exdantic Examples","doc":"# Exdantic Examples\n\nThis directory contains comprehensive examples showcasing all of Exdantic's features, from basic validation to advanced LLM integration and DSPy patterns.","ref":"examples.html"},{"type":"extras","title":"Quick Start - Exdantic Examples","doc":"Run any example with:\n```bash\nmix run examples/ .exs\n```","ref":"examples.html#quick-start"},{"type":"extras","title":"📋 Complete Example Index - Exdantic Examples","doc":"","ref":"examples.html#complete-example-index"},{"type":"extras","title":"🟢 Core & Beginner Examples - Exdantic Examples","doc":"#### 📝 [`basic_usage.exs`](basic_usage.exs)\n- **What it covers**: Fundamental schema definition, validation, and error handling\n- **Key concepts**: Basic types, constraints, field definitions, complex types\n- **Best for**: Getting started with Exdantic\n- **Features**: Type validation, constraints, arrays, maps, unions, error messages\n\n#### 🏗️ [`advanced_features.exs`](advanced_features.exs)  \n- **What it covers**: Complex schema patterns, nested validation, custom types\n- **Key concepts**: Object types, unions, arrays, business domain modeling\n- **Best for**: Understanding Exdantic's advanced validation capabilities\n- **Features**: Nested objects, complex business rules, integration patterns\n\n#### 🎨 [`custom_validation.exs`](custom_validation.exs)\n- **What it covers**: Custom validation functions, error messages, business logic\n- **Key concepts**: Custom validators, error customization, transformation patterns\n- **Best for**: Implementing domain-specific validation logic\n- **Features**: Email validation, password strength, value transformation, SKU validation\n\n#### 📚 [`readme_examples.exs`](readme_examples.exs)\n- **What it covers**: All examples from the main README for verification\n- **Key concepts**: Complete feature overview and documentation testing\n- **Best for**: Verifying installation and basic functionality\n- **Features**: README code verification, comprehensive test suite","ref":"examples.html#core-beginner-examples"},{"type":"extras","title":"🟡 Intermediate Features - Exdantic Examples","doc":"#### 🚀 [`runtime_schema.exs`](runtime_schema.exs) ⭐\n- **What it covers**: Dynamic schema creation at runtime\n- **Key concepts**: Runtime schema generation, field definitions, dynamic validation\n- **DSPy pattern**: `pydantic.create_model(\"DSPyProgramOutputs\", **fields)`\n- **Best for**: Creating schemas programmatically based on runtime requirements\n- **Features**: Dynamic field definitions, runtime validation, JSON schema generation\n\n#### 🔧 [`type_adapter.exs`](type_adapter.exs) ⭐\n- **What it covers**: Runtime type validation without schemas\n- **Key concepts**: TypeAdapter system, type coercion, serialization\n- **DSPy pattern**: `TypeAdapter(type(value)).validate_python(value)`\n- **Best for**: One-off validations and dynamic type checking\n- **Features**: Type coercion, batch validation, performance optimization\n\n#### 🎁 [`wrapper_models.exs`](wrapper_models.exs) ⭐\n- **What it covers**: Temporary single-field validation schemas\n- **Key concepts**: Wrapper models, flexible input handling, factory patterns\n- **DSPy pattern**: `create_model(\"Wrapper\", value=(target_type, ...))`\n- **Best for**: Complex type coercion and single-field validation\n- **Features**: Single-field wrappers, factory patterns, flexible input handling\n\n#### 🏛️ [`root_schema.exs`](root_schema.exs) ⭐\n- **What it covers**: Validation of non-dictionary types at the root level\n- **Key concepts**: Root schemas, array validation, primitive type validation\n- **DSPy pattern**: Root-level validation for LLM outputs (arrays, single values)\n- **Best for**: Validating LLM outputs that aren't dictionaries\n- **Features**: Array validation, single values, union types, LLM output patterns\n\n#### ⚙️ [`advanced_config.exs`](advanced_config.exs) ⭐\n- **What it covers**: Runtime configuration modification and presets\n- **Key concepts**: Configuration system, builder pattern, presets\n- **DSPy pattern**: `ConfigDict(extra=\"forbid\", frozen=True)`\n- **Best for**: Flexible validation behavior based on context\n- **Features**: Configuration presets, builder pattern, environment-based config","ref":"examples.html#intermediate-features"},{"type":"extras","title":"🔴 Advanced Features - Exdantic Examples","doc":"#### 🚀 [`enhanced_validator.exs`](enhanced_validator.exs) ⭐\n- **What it covers**: Universal validation interface across all Exdantic features\n- **Key concepts**: Enhanced validator, configuration-driven validation, pipelines\n- **DSPy pattern**: Unified validation with dynamic configuration\n- **Best for**: Complex applications with varying validation requirements\n- **Features**: Universal interface, batch validation, LLM optimizations\n\n#### 🔗 [`json_schema_resolver.exs`](json_schema_resolver.exs) ⭐\n- **What it covers**: Advanced JSON schema manipulation for LLM integration\n- **Key concepts**: Reference resolution, schema flattening, provider optimization\n- **DSPy pattern**: LLM-compatible schema generation\n- **Best for**: Preparing schemas for different LLM providers\n- **Features**: Reference resolution, provider optimization, schema flattening\n\n#### 🔮 [`dspy_integration.exs`](dspy_integration.exs) ⭐\n- **What it covers**: Complete DSPy integration patterns\n- **Key concepts**: All DSPy patterns working together in realistic scenarios\n- **DSPy pattern**: Complete DSPy program simulation\n- **Best for**: Understanding how to build DSPy-style applications with Exdantic\n- **Features**: Complete DSPy simulation, provider optimization, error recovery","ref":"examples.html#advanced-features"},{"type":"extras","title":"🎯 Specialized Features - Exdantic Examples","doc":"#### 🧠 [`model_validators.exs`](model_validators.exs) ⭐\n- **What it covers**: Cross-field validation and data transformation\n- **Key concepts**: Model validators, business logic validation, transformation pipelines\n- **Best for**: Complex business rules that span multiple fields\n- **Features**: Password confirmation, business logic, data transformation, error handling\n\n#### 🔢 [`computed_fields.exs`](computed_fields.exs) ⭐\n- **What it covers**: Derived fields and automatic field generation\n- **Key concepts**: Computed fields, data derivation, automatic calculations\n- **Best for**: Generating additional data from validated input\n- **Features**: Automatic calculations, data derivation, analytics, error handling\n\n#### 🏷️ [`field_metadata_dspy.exs`](field_metadata_dspy.exs) ⭐\n- **What it covers**: DSPy-style field metadata and annotations\n- **Key concepts**: Field metadata, DSPy annotations, LLM hints\n- **Best for**: DSPy integration with field-level metadata\n- **Features**: DSPy field types, metadata extraction, runtime schema generation","ref":"examples.html#specialized-features"},{"type":"extras","title":"🤖 LLM Integration - Exdantic Examples","doc":"#### 🤖 [`llm_integration.exs`](llm_integration.exs) ⭐\n- **What it covers**: LLM structured output validation and optimization\n- **Key concepts**: LLM output validation, provider optimization, quality assessment\n- **Best for**: Validating and optimizing LLM outputs\n- **Features**: Structured output validation, provider optimization, quality metrics\n\n#### 🔄 [`llm_pipeline_orchestration.exs`](llm_pipeline_orchestration.exs) ⭐\n- **What it covers**: Multi-stage LLM validation pipelines\n- **Key concepts**: Pipeline orchestration, error handling strategies, quality assessment\n- **Best for**: Complex LLM workflows with multiple stages\n- **Features**: Multi-stage pipelines, error handling, quality assessment, performance analysis\n\n#### 🔀 [`conditional_recursive_validation.exs`](conditional_recursive_validation.exs) ⭐\n- **What it covers**: Conditional validation and recursive data structures\n- **Key concepts**: Conditional logic, recursive validation, dynamic schema selection\n- **Best for**: Complex validation scenarios with conditional logic\n- **Features**: Conditional validation, recursive trees, multi-step pipelines, cross-schema validation","ref":"examples.html#llm-integration"},{"type":"extras","title":"🧪 Development & Testing - Exdantic Examples","doc":"#### 🧪 [`phase_3_example.exs`](phase_3_example.exs)\n- **What it covers**: Comprehensive integration testing and advanced patterns\n- **Key concepts**: Integration testing, advanced patterns, performance analysis\n- **Best for**: Understanding complex integration scenarios\n- **Features**: Multiple schema types, integration patterns, performance testing","ref":"examples.html#development-testing"},{"type":"extras","title":"🚀 Running Examples - Exdantic Examples","doc":"","ref":"examples.html#running-examples"},{"type":"extras","title":"Quick Test - Run All Core Examples - Exdantic Examples","doc":"```bash\nmix run examples/basic_usage.exs\nmix run examples/advanced_features.exs\nmix run examples/custom_validation.exs\nmix run examples/readme_examples.exs\n```","ref":"examples.html#quick-test-run-all-core-examples"},{"type":"extras","title":"DSPy Development Workflow - Exdantic Examples","doc":"```bash\n# Start with the big picture\nmix run examples/dspy_integration.exs\n\n# Then explore specific features\nmix run examples/runtime_schema.exs\nmix run examples/type_adapter.exs\nmix run examples/wrapper_models.exs\nmix run examples/field_metadata_dspy.exs\n```","ref":"examples.html#dspy-development-workflow"},{"type":"extras","title":"LLM Integration Workflow - Exdantic Examples","doc":"```bash\n# Basic LLM integration\nmix run examples/llm_integration.exs\n\n# Advanced pipeline orchestration\nmix run examples/llm_pipeline_orchestration.exs\n\n# JSON schema optimization\nmix run examples/json_schema_resolver.exs\n\n# Root schema for non-dictionary outputs\nmix run examples/root_schema.exs\n```","ref":"examples.html#llm-integration-workflow"},{"type":"extras","title":"Advanced Validation Patterns - Exdantic Examples","doc":"```bash\n# Model validators for business logic\nmix run examples/model_validators.exs\n\n# Computed fields for derived data\nmix run examples/computed_fields.exs\n\n# Conditional and recursive validation\nmix run examples/conditional_recursive_validation.exs\n\n# Enhanced validator for complex scenarios\nmix run examples/enhanced_validator.exs\n```","ref":"examples.html#advanced-validation-patterns"},{"type":"extras","title":"Configuration and Optimization - Exdantic Examples","doc":"```bash\n# Advanced configuration patterns\nmix run examples/advanced_config.exs\n\n# JSON schema manipulation\nmix run examples/json_schema_resolver.exs\n\n# Performance optimization\nmix run examples/type_adapter.exs | grep \"Performance\"\n```","ref":"examples.html#configuration-and-optimization"},{"type":"extras","title":"Run All Examples - Exdantic Examples","doc":"```bash\n# Run all examples in sequence\nfor example in examples/*.exs; do\n  echo \"Running $(basename $example)...\"\n  mix run \"$example\"\n  echo \"---\"\ndone\n```","ref":"examples.html#run-all-examples"},{"type":"extras","title":"📊 Examples by Use Case - Exdantic Examples","doc":"","ref":"examples.html#examples-by-use-case"},{"type":"extras","title":"📝 Data Validation - Exdantic Examples","doc":"- **Basic**: `basic_usage.exs`, `custom_validation.exs`\n- **Advanced**: `advanced_features.exs`, `enhanced_validator.exs`\n- **Specialized**: `model_validators.exs`, `computed_fields.exs`","ref":"examples.html#data-validation"},{"type":"extras","title":"🤖 LLM Integration - Exdantic Examples","doc":"- **Basic**: `llm_integration.exs`, `root_schema.exs`\n- **Advanced**: `llm_pipeline_orchestration.exs`, `json_schema_resolver.exs`\n- **DSPy**: `dspy_integration.exs`, `field_metadata_dspy.exs`","ref":"examples.html#llm-integration-1"},{"type":"extras","title":"⚡ Performance & Optimization - Exdantic Examples","doc":"- **Fast Validation**: `type_adapter.exs`, `wrapper_models.exs`\n- **Batch Processing**: `enhanced_validator.exs`\n- **Schema Optimization**: `json_schema_resolver.exs`","ref":"examples.html#performance-optimization"},{"type":"extras","title":"🔧 Development Tools - Exdantic Examples","doc":"- **Dynamic Schemas**: `runtime_schema.exs`, `dspy_integration.exs`\n- **Configuration**: `advanced_config.exs`\n- **Testing**: `readme_examples.exs`, `phase_3_example.exs`","ref":"examples.html#development-tools"},{"type":"extras","title":"🏗️ Complex Patterns - Exdantic Examples","doc":"- **Conditional Logic**: `conditional_recursive_validation.exs`\n- **Multi-stage Processing**: `llm_pipeline_orchestration.exs`\n- **Business Rules**: `model_validators.exs`","ref":"examples.html#complex-patterns"},{"type":"extras","title":"🎯 Learning Path Recommendations - Exdantic Examples","doc":"","ref":"examples.html#learning-path-recommendations"},{"type":"extras","title":"🟢 Beginner Path - Exdantic Examples","doc":"1. `basic_usage.exs` - Learn core concepts\n2. `custom_validation.exs` - Add business logic\n3. `advanced_features.exs` - Complex patterns\n4. `readme_examples.exs` - Verify understanding","ref":"examples.html#beginner-path"},{"type":"extras","title":"🟡 Intermediate Path - Exdantic Examples","doc":"1. `runtime_schema.exs` - Dynamic schemas\n2. `type_adapter.exs` - Runtime validation\n3. `wrapper_models.exs` - Flexible input handling\n4. `enhanced_validator.exs` - Universal interface","ref":"examples.html#intermediate-path"},{"type":"extras","title":"🔴 Advanced Path - Exdantic Examples","doc":"1. `model_validators.exs` - Business logic validation\n2. `computed_fields.exs` - Derived data\n3. `conditional_recursive_validation.exs` - Complex patterns\n4. `json_schema_resolver.exs` - Schema manipulation","ref":"examples.html#advanced-path"},{"type":"extras","title":"🤖 LLM Integration Path - Exdantic Examples","doc":"1. `llm_integration.exs` - Basic LLM validation\n2. `root_schema.exs` - Non-dictionary outputs\n3. `field_metadata_dspy.exs` - DSPy metadata\n4. `llm_pipeline_orchestration.exs` - Complex pipelines\n5. `dspy_integration.exs` - Complete integration","ref":"examples.html#llm-integration-path"},{"type":"extras","title":"🔑 Key Concepts by Example - Exdantic Examples","doc":"| Concept | Primary Examples | Supporting Examples |\n|---------|------------------|-------------------|\n| **Schema Definition** | `basic_usage.exs`, `advanced_features.exs` | `readme_examples.exs` |\n| **Runtime Schemas** | `runtime_schema.exs`, `dspy_integration.exs` | `field_metadata_dspy.exs` |\n| **Type Validation** | `type_adapter.exs`, `enhanced_validator.exs` | `wrapper_models.exs` |\n| **Model Validators** | `model_validators.exs` | `conditional_recursive_validation.exs` |\n| **Computed Fields** | `computed_fields.exs` | `model_validators.exs` |\n| **Root Schemas** | `root_schema.exs` | `llm_integration.exs` |\n| **Configuration** | `advanced_config.exs`, `enhanced_validator.exs` | All examples |\n| **JSON Schema** | `json_schema_resolver.exs` | `runtime_schema.exs`, `dspy_integration.exs` |\n| **LLM Integration** | `llm_integration.exs`, `llm_pipeline_orchestration.exs` | `dspy_integration.exs` |\n| **DSPy Patterns** | `dspy_integration.exs`, `field_metadata_dspy.exs` | `runtime_schema.exs`, `wrapper_models.exs` |\n| **Error Handling** | All examples | `enhanced_validator.exs` |\n| **Performance** | All enhanced examples | `type_adapter.exs` |","ref":"examples.html#key-concepts-by-example"},{"type":"extras","title":"🛠️ Common Patterns - Exdantic Examples","doc":"","ref":"examples.html#common-patterns"},{"type":"extras","title":"Creating Dynamic Schemas - Exdantic Examples","doc":"```elixir\n# See: runtime_schema.exs, dspy_integration.exs\nfields = [\n  {:name, :string, [required: true, min_length: 2]},\n  {:email, :string, [required: true, format: ~r/@/]}\n]\nschema = Exdantic.Runtime.create_schema(fields)\n```","ref":"examples.html#creating-dynamic-schemas"},{"type":"extras","title":"Quick Type Validation - Exdantic Examples","doc":"```elixir\n# See: type_adapter.exs, enhanced_validator.exs  \n{:ok, validated} = Exdantic.TypeAdapter.validate(:integer, \"123\", coerce: true)\n```","ref":"examples.html#quick-type-validation"},{"type":"extras","title":"Wrapper Validation - Exdantic Examples","doc":"```elixir\n# See: wrapper_models.exs, dspy_integration.exs\n{:ok, score} = Exdantic.Wrapper.wrap_and_validate(:score, :integer, \"85\", \n  coerce: true, constraints: [gteq: 0, lteq: 100])\n```","ref":"examples.html#wrapper-validation"},{"type":"extras","title":"Model Validators - Exdantic Examples","doc":"```elixir\n# See: model_validators.exs, conditional_recursive_validation.exs\ndefmodule UserSchema do\n  use Exdantic\n  \n  schema do\n    field :password, :string\n    field :password_confirmation, :string\n    model_validator :validate_passwords_match\n  end\nend\n```","ref":"examples.html#model-validators"},{"type":"extras","title":"Computed Fields - Exdantic Examples","doc":"```elixir\n# See: computed_fields.exs\ndefmodule ProfileSchema do\n  use Exdantic\n  \n  schema do\n    field :first_name, :string\n    field :last_name, :string\n    computed_field :full_name, :string, :generate_full_name\n  end\nend\n```","ref":"examples.html#computed-fields"},{"type":"extras","title":"LLM Schema Optimization - Exdantic Examples","doc":"```elixir\n# See: json_schema_resolver.exs, llm_integration.exs\noptimized = json_schema\n|> Exdantic.JsonSchema.Resolver.resolve_references()\n|> Exdantic.JsonSchema.Resolver.optimize_for_provider(:openai)\n```","ref":"examples.html#llm-schema-optimization"},{"type":"extras","title":"📈 Performance Features - Exdantic Examples","doc":"Most examples include performance benchmarks showing:\n- **Validation Speed**: How fast different approaches validate data\n- **Memory Usage**: Memory efficiency of different patterns\n- **Batch Processing**: Performance gains from batch operations\n- **Schema Compilation**: Runtime vs compile-time schema performance","ref":"examples.html#performance-features"},{"type":"extras","title":"🚨 Error Handling Patterns - Exdantic Examples","doc":"Examples demonstrate comprehensive error handling:\n- **Field-level errors**: Individual field validation failures\n- **Model-level errors**: Cross-field validation failures  \n- **Type errors**: Type mismatch and coercion failures\n- **Business logic errors**: Custom validation failures\n- **Pipeline errors**: Multi-stage validation failures","ref":"examples.html#error-handling-patterns"},{"type":"extras","title":"🔍 Debugging Features - Exdantic Examples","doc":"Many examples include debugging helpers:\n- **Validation reports**: Detailed validation information\n- **Schema introspection**: Understanding schema structure\n- **Error path tracking**: Finding exactly where validation failed\n- **Performance profiling**: Identifying bottlenecks","ref":"examples.html#debugging-features"},{"type":"extras","title":"💡 Tips for Learning - Exdantic Examples","doc":"1. **Start Simple**: Begin with `basic_usage.exs` to understand core concepts\n2. **Follow Use Cases**: Pick examples based on your specific needs\n3. **Read the Code**: Each example is heavily commented with explanations\n4. **Experiment**: Modify the examples to test different scenarios\n5. **Check Performance**: Most examples include performance benchmarks\n6. **Understand Errors**: Examples demonstrate both success and failure cases","ref":"examples.html#tips-for-learning"},{"type":"extras","title":"🆘 Getting Help - Exdantic Examples","doc":"- Each example includes comprehensive comments explaining the concepts\n- Error messages in examples show common pitfalls and solutions\n- Performance sections help you understand when to use each feature\n- DSPy integration examples show real-world usage patterns","ref":"examples.html#getting-help"},{"type":"extras","title":"🤝 Contributing Examples - Exdantic Examples","doc":"When adding new examples:\n\n1. Follow the established format with clear sections and comments\n2. Include both success and failure scenarios  \n3. Add performance benchmarks for new features\n4. Provide practical, real-world use cases\n5. Update this README with the new example information\n\n---\n\n**Made with Elixir** 💜 | **Enhanced for DSPy** 🔮 | **Optimized for LLMs** 🤖","ref":"examples.html#contributing-examples"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}